@@ -33,12 +33,14 @@
 
 #include "config.h"
 
-#include "cache.h"
-
 #include <errno.h>
 #include <stdlib.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+
+#include "cache_varnishd.h"
 
 #include "cache_transport.h"
 #include "cache_pool.h"
@@ -50,9 +52,10 @@
 #include "vtim.h"
 
 static pthread_t	VCA_thread;
-static double vca_pace = 0.0;
+static vtim_dur vca_pace = 0.0;
 static struct lock pace_mtx;
 static unsigned pool_accepting;
+static pthread_mutex_t shut_mtx = PTHREAD_MUTEX_INITIALIZER;
 
 struct wrk_accept {
 	unsigned		magic;
@@ -85,25 +88,26 @@ static struct tcp_opt {
 	socklen_t	sz;
 	void		*ptr;
 	int		need;
+	int		iponly;
 } tcp_opts[] = {
-#define TCPO(lvl, nam, sz) { lvl, nam, #nam, sizeof(sz), 0, 0},
+#define TCPO(lvl, nam, sz, ip) { lvl, nam, #nam, sizeof(sz), 0, 0, ip},
 
-	TCPO(SOL_SOCKET, SO_LINGER, struct linger)
-	TCPO(SOL_SOCKET, SO_KEEPALIVE, int)
-	TCPO(IPPROTO_TCP, TCP_NODELAY, int)
+	TCPO(SOL_SOCKET, SO_LINGER, struct linger, 0)
+	TCPO(SOL_SOCKET, SO_KEEPALIVE, int, 0)
+	TCPO(IPPROTO_TCP, TCP_NODELAY, int, 1)
 
 #ifdef SO_SNDTIMEO_WORKS
-	TCPO(SOL_SOCKET, SO_SNDTIMEO, struct timeval)
+	TCPO(SOL_SOCKET, SO_SNDTIMEO, struct timeval, 0)
 #endif
 
 #ifdef SO_RCVTIMEO_WORKS
-	TCPO(SOL_SOCKET, SO_RCVTIMEO, struct timeval)
+	TCPO(SOL_SOCKET, SO_RCVTIMEO, struct timeval, 0)
 #endif
 
 #ifdef HAVE_TCP_KEEP
-	TCPO(IPPROTO_TCP, TCP_KEEPIDLE, int)
-	TCPO(IPPROTO_TCP, TCP_KEEPCNT, int)
-	TCPO(IPPROTO_TCP, TCP_KEEPINTVL, int)
+	TCPO(IPPROTO_TCP, TCP_KEEPIDLE, int, 1)
+	TCPO(IPPROTO_TCP, TCP_KEEPCNT, int, 1)
+	TCPO(IPPROTO_TCP, TCP_KEEPINTVL, int, 1)
 #endif
 
 #undef TCPO
@@ -128,6 +132,21 @@ static const struct linger linger = {
 
 static unsigned		need_test;
 
+/*--------------------------------------------------------------------
+ * lacking a better place, we put some generic periodic updates
+ * into the vca_acct() loop which we are running anyway
+ */
+static void
+vca_periodic(vtim_real t0)
+{
+	vtim_real now;
+
+	now = VTIM_real();
+	VSC_C_main->uptime = (uint64_t)(now - t0);
+
+	VTIM_postel = FEATURE(FEATURE_HTTP_DATE_POSTEL);
+}
+
 /*--------------------------------------------------------------------
  * Some kernels have bugs/limitations with respect to which options are
  * inherited from the accept/listen socket, so we have to keep track of
@@ -202,7 +221,7 @@ vca_tcp_opt_init(void)
 }
 
 static void
-vca_tcp_opt_test(int sock)
+vca_tcp_opt_test(const int sock, const unsigned uds)
 {
 	int i, n;
 	struct tcp_opt *to;
@@ -211,6 +230,8 @@ vca_tcp_opt_test(int sock)
 
 	for (n = 0; n < n_tcp_opts; n++) {
 		to = &tcp_opts[n];
+		if (to->iponly && uds)
+			continue;
 		to->need = 1;
 		ptr = calloc(1, to->sz);
 		AN(ptr);
@@ -225,13 +246,15 @@ vca_tcp_opt_test(int sock)
 }
 
 static void
-vca_tcp_opt_set(int sock, int force)
+vca_tcp_opt_set(const int sock, const unsigned uds, const int force)
 {
 	int n;
 	struct tcp_opt *to;
 
 	for (n = 0; n < n_tcp_opts; n++) {
 		to = &tcp_opts[n];
+		if (to->iponly && uds)
+			continue;
 		if (to->need || force) {
 			VTCP_Assert(setsockopt(sock,
 			    to->level, to->optname, to->ptr, to->sz));
@@ -247,7 +270,7 @@ vca_tcp_opt_set(int sock, int force)
 static void
 vca_pace_check(void)
 {
-	double p;
+	vtim_dur p;
 
 	if (vca_pace == 0.0)
 		return;
@@ -288,15 +311,57 @@ vca_pace_good(void)
  * Called from assigned worker thread
  */
 
-static void __match_proto__(task_func_t)
+static void
+vca_mk_tcp(const struct wrk_accept *wa,
+    struct sess *sp, char *laddr, char *lport, char *raddr, char *rport)
+{
+	struct suckaddr *sa;
+	struct sockaddr_storage ss;
+	socklen_t sl;
+
+	SES_Reserve_remote_addr(sp, &sa);
+	AN(VSA_Build(sa, &wa->acceptaddr, wa->acceptaddrlen));
+	sp->sattr[SA_CLIENT_ADDR] = sp->sattr[SA_REMOTE_ADDR];
+
+	VTCP_name(sa, raddr, VTCP_ADDRBUFSIZE, rport, VTCP_PORTBUFSIZE);
+	SES_Set_String_Attr(sp, SA_CLIENT_IP, raddr);
+	SES_Set_String_Attr(sp, SA_CLIENT_PORT, rport);
+
+	sl = sizeof ss;
+	AZ(getsockname(sp->fd, (void*)&ss, &sl));
+	SES_Reserve_local_addr(sp, &sa);
+	AN(VSA_Build(sa, &ss, sl));
+	sp->sattr[SA_SERVER_ADDR] = sp->sattr[SA_LOCAL_ADDR];
+	VTCP_name(sa, laddr, VTCP_ADDRBUFSIZE, lport, VTCP_PORTBUFSIZE);
+}
+
+static void
+vca_mk_uds(struct wrk_accept *wa, struct sess *sp, char *laddr, char *lport,
+	   char *raddr, char *rport)
+{
+	struct suckaddr *sa;
+
+	(void) wa;
+	SES_Reserve_remote_addr(sp, &sa);
+	AZ(SES_Set_remote_addr(sp, bogo_ip));
+	sp->sattr[SA_CLIENT_ADDR] = sp->sattr[SA_REMOTE_ADDR];
+	sp->sattr[SA_LOCAL_ADDR] = sp->sattr[SA_REMOTE_ADDR];
+	sp->sattr[SA_SERVER_ADDR] = sp->sattr[SA_REMOTE_ADDR];
+	SES_Set_String_Attr(sp, SA_CLIENT_IP, "0.0.0.0");
+	SES_Set_String_Attr(sp, SA_CLIENT_PORT, "0");
+
+	strcpy(laddr, "0.0.0.0");
+	strcpy(raddr, "0.0.0.0");
+	strcpy(lport, "0");
+	strcpy(rport, "0");
+}
+
+static void v_matchproto_(task_func_t)
 vca_make_session(struct worker *wrk, void *arg)
 {
 	struct sess *sp;
 	struct req *req;
 	struct wrk_accept *wa;
-	struct sockaddr_storage ss;
-	struct suckaddr *sa;
-	socklen_t sl;
 	char laddr[VTCP_ADDRBUFSIZE];
 	char lport[VTCP_PORTBUFSIZE];
 	char raddr[VTCP_ADDRBUFSIZE];
@@ -337,31 +402,20 @@ vca_make_session(struct worker *wrk, void *arg)
 
 	sp->fd = wa->acceptsock;
 	wa->acceptsock = -1;
+	sp->listen_sock = wa->acceptlsock;
 
 	assert(wa->acceptaddrlen <= vsa_suckaddr_len);
-	SES_Reserve_remote_addr(sp, &sa);
-	AN(VSA_Build(sa, &wa->acceptaddr, wa->acceptaddrlen));
-	sp->sattr[SA_CLIENT_ADDR] = sp->sattr[SA_REMOTE_ADDR];
 
-	VTCP_name(sa, raddr, sizeof raddr, rport, sizeof rport);
-	SES_Set_String_Attr(sp, SA_CLIENT_IP, raddr);
-	SES_Set_String_Attr(sp, SA_CLIENT_PORT, rport);
-
-	sl = sizeof ss;
-	AZ(getsockname(sp->fd, (void*)&ss, &sl));
-	SES_Reserve_local_addr(sp, &sa);
-	AN(VSA_Build(sa, &ss, sl));
-	sp->sattr[SA_SERVER_ADDR] = sp->sattr[SA_LOCAL_ADDR];
-
-	VTCP_name(sa, laddr, sizeof laddr, lport, sizeof lport);
+	if (wa->acceptlsock->uds)
+		vca_mk_uds(wa, sp, laddr, lport, raddr, rport);
+	else
+		vca_mk_tcp(wa, sp, laddr, lport, raddr, rport);
 
+	AN(wa->acceptlsock->name);
 	VSL(SLT_Begin, sp->vxid, "sess 0 %s",
 	    wa->acceptlsock->transport->name);
 	VSL(SLT_SessOpen, sp->vxid, "%s %s %s %s %s %.6f %d",
-	    raddr, rport,
-	    wa->acceptlsock->name != NULL ?
-		wa->acceptlsock->name : wa->acceptlsock->endpoint,
-	    laddr, lport,
+	    raddr, rport, wa->acceptlsock->name, laddr, lport,
 	    sp->t_open, sp->fd);
 
 	WS_Release(wrk->aws, 0);
@@ -370,10 +424,10 @@ vca_make_session(struct worker *wrk, void *arg)
 	wrk->stats->sess_conn++;
 
 	if (need_test) {
-		vca_tcp_opt_test(sp->fd);
+		vca_tcp_opt_test(sp->fd, wa->acceptlsock->uds);
 		need_test = 0;
 	}
-	vca_tcp_opt_set(sp->fd, 0);
+	vca_tcp_opt_set(sp->fd, wa->acceptlsock->uds, 0);
 
 	req = Req_New(wrk, sp);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -390,13 +444,15 @@ vca_make_session(struct worker *wrk, void *arg)
  * and handle the new connection ourselves.
  */
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 vca_accept_task(struct worker *wrk, void *arg)
 {
 	struct wrk_accept wa;
 	struct poolsock *ps;
 	struct listen_sock *ls;
 	int i;
+	char laddr[VTCP_ADDRBUFSIZE];
+	char lport[VTCP_PORTBUFSIZE];
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(ps, arg, POOLSOCK_MAGIC);
@@ -433,23 +489,44 @@ vca_accept_task(struct worker *wrk, void *arg)
 		if (i < 0) {
 			switch (errno) {
 			case ECONNABORTED:
+				wrk->stats->sess_fail_econnaborted++;
+				break;
+			case EINTR:
+				wrk->stats->sess_fail_eintr++;
 				break;
 			case EMFILE:
-				VSL(SLT_Debug, ls->sock, "Too many open files");
+				wrk->stats->sess_fail_emfile++;
 				vca_pace_bad();
 				break;
 			case EBADF:
-				VSL(SLT_Debug, ls->sock, "Accept failed: %s",
-				    strerror(errno));
+				wrk->stats->sess_fail_ebadf++;
+				vca_pace_bad();
+				break;
+			case ENOBUFS:
+			case ENOMEM:
+				wrk->stats->sess_fail_enomem++;
 				vca_pace_bad();
 				break;
 			default:
-				VSL(SLT_Debug, ls->sock, "Accept failed: %s",
-				    strerror(errno));
+				wrk->stats->sess_fail_other++;
 				vca_pace_bad();
 				break;
 			}
+
+			i = errno;
 			wrk->stats->sess_fail++;
+
+			if (wa.acceptlsock->uds) {
+				strcpy(laddr, "0.0.0.0");
+				strcpy(lport, "0");
+			} else {
+				VTCP_myname(ls->sock, laddr, VTCP_ADDRBUFSIZE,
+				    lport, VTCP_ADDRBUFSIZE);
+			}
+
+			VSL(SLT_SessError, 0, "%s %s %s %d %d %s",
+			    wa.acceptlsock->name, laddr, lport,
+			    ls->sock, i, strerror(i));
 			(void)Pool_TrySumstat(wrk);
 			continue;
 		}
@@ -517,77 +594,89 @@ VCA_DestroyPool(struct pool *pp)
 
 /*--------------------------------------------------------------------*/
 
-static void * __match_proto__()
+static void * v_matchproto_()
 vca_acct(void *arg)
 {
 	struct listen_sock *ls;
-	double t0, now;
+	vtim_real t0;
 
+	// XXX Actually a mis-nomer now because the accept happens in a pool
+	// thread. Rename to accept-nanny or so?
 	THR_SetName("cache-acceptor");
 	THR_Init();
 	(void)arg;
 
-	(void)vca_tcp_opt_init();
+	t0 = VTIM_real();
+	vca_periodic(t0);
 
-	VTAILQ_FOREACH(ls, &heritage.socks, list) {
-		CHECK_OBJ_NOTNULL(ls->transport, TRANSPORT_MAGIC);
-		assert (ls->sock > 0);		// We know where stdin is
-		if (cache_param->tcp_fastopen) {
-			int i;
-			i = VTCP_fastopen(ls->sock, cache_param->listen_depth);
-			if (i)
-				VSL(SLT_Error, ls->sock,
-				    "Kernel TCP Fast Open: sock=%d, ret=%d %s",
-				    ls->sock, i, strerror(errno));
-		}
-		AZ(listen(ls->sock, cache_param->listen_depth));
-		vca_tcp_opt_set(ls->sock, 1);
-		if (cache_param->accept_filter) {
-			int i;
-			i = VTCP_filter_http(ls->sock);
-			if (i)
-				VSL(SLT_Error, ls->sock,
-				    "Kernel filtering: sock=%d, ret=%d %s",
-				    ls->sock, i, strerror(errno));
-		}
-	}
-
-	need_test = 1;
 	pool_accepting = 1;
 
-	t0 = VTIM_real();
 	while (1) {
 		(void)sleep(1);
 		if (vca_tcp_opt_init()) {
+			AZ(pthread_mutex_lock(&shut_mtx));
 			VTAILQ_FOREACH(ls, &heritage.socks, list) {
 				if (ls->sock == -2)
-					continue;	// raced VCA_Shutdown
+					continue;	// VCA_Shutdown
 				assert (ls->sock > 0);
-				vca_tcp_opt_set(ls->sock, 1);
+				vca_tcp_opt_set(ls->sock, ls->uds, 1);
 			}
+			AZ(pthread_mutex_unlock(&shut_mtx));
 		}
-		now = VTIM_real();
-		VSC_C_main->uptime = (uint64_t)(now - t0);
+		vca_periodic(t0);
 	}
 	NEEDLESS(return NULL);
 }
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_start(struct cli *cli, const char * const *av, void *priv)
 {
+	struct listen_sock *ls;
 
 	(void)cli;
 	(void)av;
 	(void)priv;
 
+	(void)vca_tcp_opt_init();
+
+	VTAILQ_FOREACH(ls, &heritage.socks, list) {
+		CHECK_OBJ_NOTNULL(ls->transport, TRANSPORT_MAGIC);
+		assert (ls->sock > 0);	// We know where stdin is
+		if (cache_param->tcp_fastopen) {
+			int i;
+			i = VTCP_fastopen(ls->sock, cache_param->listen_depth);
+			if (i)
+				VSL(SLT_Error, 0,
+				    "Kernel TCP Fast Open: sock=%d, ret=%d %s",
+				    ls->sock, i, strerror(errno));
+		}
+		if (listen(ls->sock, cache_param->listen_depth)) {
+			VCLI_SetResult(cli, CLIS_CANT);
+			VCLI_Out(cli, "Listen failed on socket '%s': %s",
+			    ls->endpoint, strerror(errno));
+			return;
+		}
+		vca_tcp_opt_set(ls->sock, ls->uds, 1);
+		if (cache_param->accept_filter) {
+			int i;
+			i = VTCP_filter_http(ls->sock);
+			if (i)
+				VSL(SLT_Error, 0,
+				    "Kernel filtering: sock=%d, ret=%d %s",
+				    ls->sock, i, strerror(errno));
+		}
+	}
+
+	need_test = 1;
+
 	AZ(pthread_create(&VCA_thread, NULL, vca_acct, NULL));
 }
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 {
 	struct listen_sock *ls;
@@ -606,10 +695,16 @@ ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 	while (!pool_accepting)
 		VTIM_sleep(.1);
 
+	AZ(pthread_mutex_lock(&shut_mtx));
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
-		VTCP_myname(ls->sock, h, sizeof h, p, sizeof p);
-		VCLI_Out(cli, "%s %s\n", h, p);
+		if (!ls->uds) {
+			VTCP_myname(ls->sock, h, sizeof h, p, sizeof p);
+			VCLI_Out(cli, "%s %s\n", h, p);
+		}
+		else
+			VCLI_Out(cli, "%s -\n", ls->endpoint);
 	}
+	AZ(pthread_mutex_unlock(&shut_mtx));
 }
 
 /*--------------------------------------------------------------------*/
@@ -634,11 +729,13 @@ VCA_Shutdown(void)
 	struct listen_sock *ls;
 	int i;
 
+	AZ(pthread_mutex_lock(&shut_mtx));
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
 		i = ls->sock;
 		ls->sock = -2;
 		(void)close(i);
 	}
+	AZ(pthread_mutex_unlock(&shut_mtx));
 }
 
 /*--------------------------------------------------------------------
@@ -674,7 +771,8 @@ XPORT_Find(const char *name)
 	ASSERT_MGT();
 
 	VTAILQ_FOREACH(xp, &transports, list)
-		if (!strcasecmp(xp->name, name))
+		if (xp->proto_ident != NULL &&
+		    !strcasecmp(xp->proto_ident, name))
 			return (xp);
 	return (NULL);
 }
