@@ -53,82 +53,35 @@ struct backend {
 	unsigned		n_conn;
 
 	VTAILQ_ENTRY(backend)	list;
-	VTAILQ_ENTRY(backend)	vcl_list;
 	struct lock		mtx;
 
 	VRT_BACKEND_FIELDS()
 
-	struct vcl		*vcl;
-	char			*display_name;
-
-
 	struct vbp_target	*probe;
-	unsigned		healthy;
-	const char		*admin_health;
-	double			health_changed;
 
-	struct VSC_C_vbe	*vsc;
+	struct vsc_seg		*vsc_seg;
+	struct VSC_vbe		*vsc;
 
 	struct tcp_pool		*tcp_pool;
 
-	struct director		director[1];
+	VCL_BACKEND		director;
 
-	double			cooled;
-};
-
-/*---------------------------------------------------------------------
- * Backend connection -- private to cache_backend*.c
- */
-
-struct vbc {
-	unsigned		magic;
-#define VBC_MAGIC		0x0c5e6592
-	int			fd;
-	VTAILQ_ENTRY(vbc)	list;
-	const struct suckaddr	*addr;
-	uint8_t			state;
-#define VBC_STATE_AVAIL		(1<<0)
-#define VBC_STATE_USED		(1<<1)
-#define VBC_STATE_STOLEN	(1<<2)
-#define VBC_STATE_CLEANUP	(1<<3)
-	struct waited		waited[1];
-	struct tcp_pool		*tcp_pool;
-
-	pthread_cond_t		*cond;
+	vtim_real		cooled;
 };
 
 /*---------------------------------------------------------------------
  * Prototypes
  */
 
-/* cache_backend.c */
-void VBE_fill_director(struct backend *be);
-
 /* cache_backend_cfg.c */
-unsigned VBE_Healthy(const struct backend *b, double *changed);
-#ifdef VCL_MET_MAX
-void VBE_Event(struct backend *, enum vcl_event_e);
-#endif
-void VBE_Delete(struct backend *be);
+void VBE_SetHappy(const struct backend *, uint64_t);
 
 /* cache_backend_probe.c */
+void VBP_Update_Backend(struct vbp_target *vt);
 void VBP_Insert(struct backend *b, struct vrt_backend_probe const *p,
     struct tcp_pool *);
 void VBP_Remove(struct backend *b);
 void VBP_Control(const struct backend *b, int stop);
-void VBP_Status(struct cli *cli, const struct backend *, int details);
+void VBP_Status(struct vsb *, const struct backend *, int details, int json);
+void VBE_Connect_Error(struct VSC_vbe *, int err);
 
-/* cache_backend_tcp.c */
-struct tcp_pool *VBT_Ref(const struct suckaddr *ip4,
-    const struct suckaddr *ip6);
-void VBT_Rel(struct tcp_pool **tpp);
-int VBT_Open(const struct tcp_pool *tp, double tmo, const struct suckaddr **sa);
-void VBT_Recycle(const struct worker *, struct tcp_pool *, struct vbc **);
-void VBT_Close(struct tcp_pool *tp, struct vbc **vbc);
-struct vbc *VBT_Get(struct tcp_pool *, double tmo, const struct backend *,
-    struct worker *);
-void VBT_Wait(struct worker *, struct vbc *);
-
-/* cache_vcl.c */
-int VCL_AddBackend(struct vcl *, struct backend *);
-void VCL_DelBackend(struct backend *);
