@@ -37,21 +37,22 @@
 
 #include "config.h"
 
-#include "cache.h"
-
 #include <poll.h>
 #include <stdio.h>
+#include <errno.h>
+#include <string.h>
 #include <stdlib.h>
 
+#include "cache_varnishd.h"
+
 #include "binary_heap.h"
 #include "vcli_serve.h"
-#include "vrt.h"
 #include "vsa.h"
 #include "vtcp.h"
 #include "vtim.h"
 
-#include "cache_director.h"
 #include "cache_backend.h"
+#include "cache_tcp_pool.h"
 
 /* Default averaging rate, we want something pretty responsive */
 #define AVG_RATE			4
@@ -72,14 +73,14 @@ struct vbp_target {
 	unsigned			good;
 
 	/* Collected statistics */
-#define BITMAP(n, c, t, b)	uint64_t	n;
+#define BITMAP(n, c, t, b)	uintmax_t	n;
 #include "tbl/backend_poll.h"
 
-	double				last;
-	double				avg;
+	vtim_dur			last;
+	vtim_dur			avg;
 	double				rate;
 
-	double				due;
+	vtim_real			due;
 	int				running;
 	int				heap_idx;
 	struct pool_task		task;
@@ -102,7 +103,7 @@ vbp_delete(struct vbp_target *vt)
 #define DN(x)	/**/
 	VRT_BACKEND_PROBE_HANDLE();
 #undef DN
-	VBT_Rel(&vt->tcp_pool);
+	VTP_Rel(&vt->tcp_pool);
 	free(vt->req);
 	FREE_OBJ(vt);
 }
@@ -149,46 +150,53 @@ vbp_has_poked(struct vbp_target *vt)
 	vt->good = j;
 }
 
-static void
-vbp_update_backend(struct vbp_target *vt)
+void
+VBP_Update_Backend(struct vbp_target *vt)
 {
-	unsigned i;
+	unsigned i = 0;
 	char bits[10];
 	const char *logmsg;
 
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
 	Lck_Lock(&vbp_mtx);
-	if (vt->backend != NULL) {
-		i = 0;
-#define BITMAP(n, c, t, b) \
-		bits[i++] = (vt->n & 1) ? c : '-';
-#include "tbl/backend_poll.h"
-		bits[i] = '\0';
-		assert(i < sizeof bits);
-
-		if (vt->good >= vt->threshold) {
-			if (vt->backend->healthy)
-				logmsg = "Still healthy";
-			else {
-				logmsg = "Back healthy";
-				vt->backend->health_changed = VTIM_real();
-			}
-			vt->backend->healthy = 1;
-		} else {
-			if (vt->backend->healthy) {
-				logmsg = "Went sick";
-				vt->backend->health_changed = VTIM_real();
-			} else
-				logmsg = "Still sick";
-			vt->backend->healthy = 0;
-		}
-		VSL(SLT_Backend_health, 0, "%s %s %s %u %u %u %.6f %.6f %s",
-		    vt->backend->display_name, logmsg, bits,
-		    vt->good, vt->threshold, vt->window,
-		    vt->last, vt->avg, vt->resp_buf);
-		VBE_SetHappy(vt->backend, vt->happy);
+	if (vt->backend == NULL) {
+		Lck_Unlock(&vbp_mtx);
+		return;
 	}
+
+#define BITMAP(n, c, t, b) \
+	bits[i++] = (vt->n & 1) ? c : '-';
+#include "tbl/backend_poll.h"
+	bits[i] = '\0';
+	assert(i < sizeof bits);
+
+	if (vt->backend->director == NULL) {
+		Lck_Unlock(&vbp_mtx);
+		return;
+	}
+
+	if (vt->good >= vt->threshold) {
+		if (vt->backend->director->sick) {
+			logmsg = "Back healthy";
+			VRT_SetHealth(vt->backend->director, 1);
+		} else {
+			logmsg = "Still healthy";
+		}
+	} else {
+		if (vt->backend->director->sick) {
+			logmsg = "Still sick";
+		} else {
+			logmsg = "Went sick";
+			VRT_SetHealth(vt->backend->director, 0);
+		}
+	}
+	VSL(SLT_Backend_health, 0, "%s %s %s %u %u %u %.6f %.6f %s",
+	    vt->backend->director->vcl_name, logmsg, bits,
+	    vt->good, vt->threshold, vt->window,
+	    vt->last, vt->avg, vt->resp_buf);
+	VBE_SetHappy(vt->backend, vt->happy);
+
 	Lck_Unlock(&vbp_mtx);
 }
 
@@ -225,8 +233,15 @@ vbp_write(struct vbp_target *vt, int *sock, const void *buf, size_t len)
 
 	i = write(*sock, buf, len);
 	if (i != len) {
-		if (i < 0)
+		if (i < 0) {
 			vt->err_xmit |= 1;
+			bprintf(vt->resp_buf, "Write error %d (%s)",
+				errno, strerror(errno));
+		} else {
+			bprintf(vt->resp_buf,
+				"Short write (%d/%zu) error %d (%s)",
+				i, len, errno, strerror(errno));
+		}
 		VTCP_close(sock);
 		return (-1);
 	}
@@ -245,17 +260,15 @@ vbp_write_proxy_v1(struct vbp_target *vt, int *sock)
 
 	VTCP_myname(*sock, addr, sizeof addr, port, sizeof port);
 	AN(VSB_new(&vsb, buf, sizeof buf, VSB_FIXEDLEN));
-	AZ(VSB_cat(&vsb, "PROXY"));
 
 	l = sizeof ss;
 	AZ(getsockname(*sock, (void *)&ss, &l));
-	if (ss.ss_family == AF_INET6)
-		VSB_printf(&vsb, " TCP6 ");
-	else if (ss.ss_family == AF_INET)
-		VSB_printf(&vsb, " TCP4 ");
-	else
-		WRONG("Unknown family");
-	VSB_printf(&vsb, "%s %s %s %s\r\n", addr, addr, port, port);
+	if (ss.ss_family == AF_INET || ss.ss_family == AF_INET6) {
+		VSB_printf(&vsb, "PROXY %s %s %s %s %s\r\n",
+		    ss.ss_family == AF_INET ? "TCP4" : "TCP6",
+		    addr, addr, port, port);
+	} else
+		VSB_cat(&vsb, "PROXY UNKNOWN\r\n");
 	AZ(VSB_finish(&vsb));
 
 	return (vbp_write(vt, sock, VSB_data(&vsb), VSB_len(&vsb)));
@@ -264,8 +277,8 @@ vbp_write_proxy_v1(struct vbp_target *vt, int *sock)
 static void
 vbp_poke(struct vbp_target *vt)
 {
-	int s, tmo, i, proxy_header;
-	double t_start, t_now, t_end;
+	int s, tmo, i, proxy_header, err;
+	vtim_real t_start, t_now, t_end;
 	unsigned rlen, resp;
 	char buf[8192], *p;
 	struct pollfd pfda[1], *pfd = pfda;
@@ -274,14 +287,20 @@ vbp_poke(struct vbp_target *vt)
 	t_start = t_now = VTIM_real();
 	t_end = t_start + vt->timeout;
 
-	s = VBT_Open(vt->tcp_pool, t_end - t_now, &sa);
+	s = VTP_Open(vt->tcp_pool, t_end - t_now, (const void **)&sa, &err);
 	if (s < 0) {
-		/* Got no connection: failed */
+		bprintf(vt->resp_buf, "Open error %d (%s)", err, strerror(err));
+		Lck_Lock(&vbp_mtx);
+		if (vt->backend)
+			VBE_Connect_Error(vt->backend->vsc, err);
+		Lck_Unlock(&vbp_mtx);
 		return;
 	}
 
 	i = VSA_Get_Proto(sa);
-	if (i == AF_INET)
+	if (VSA_Compare(sa, bogo_ip) == 0)
+		vt->good_unix |= 1;
+	else if (i == AF_INET)
 		vt->good_ipv4 |= 1;
 	else if (i == AF_INET6)
 		vt->good_ipv6 |= 1;
@@ -291,7 +310,9 @@ vbp_poke(struct vbp_target *vt)
 	t_now = VTIM_real();
 	tmo = (int)round((t_end - t_now) * 1e3);
 	if (tmo <= 0) {
-		/* Spent too long time getting it */
+		bprintf(vt->resp_buf,
+			"Open timeout %.3fs exceeded by %.3fs",
+			vt->timeout, t_now - t_end);
 		VTCP_close(&s);
 		return;
 	}
@@ -303,8 +324,11 @@ vbp_poke(struct vbp_target *vt)
 		proxy_header = -1;
 	Lck_Unlock(&vbp_mtx);
 
-	if (proxy_header < 0)
+	if (proxy_header < 0) {
+		bprintf(vt->resp_buf, "%s", "No backend");
+		VTCP_close(&s);
 		return;
+	}
 
 	/* Send the PROXY header */
 	assert(proxy_header <= 2);
@@ -322,15 +346,23 @@ vbp_poke(struct vbp_target *vt)
 
 	pfd->fd = s;
 	rlen = 0;
-	do {
+	while (1) {
 		pfd->events = POLLIN;
 		pfd->revents = 0;
 		tmo = (int)round((t_end - t_now) * 1e3);
 		if (tmo > 0)
 			i = poll(pfd, 1, tmo);
-		if (i == 0 || tmo <= 0) {
-			if (i == 0)
-				vt->err_recv |= 1;
+		if (i == 0) {
+			vt->err_recv |= 1;
+			bprintf(vt->resp_buf, "Poll error %d (%s)",
+				errno, strerror(errno));
+			VTCP_close(&s);
+			return;
+		}
+		if (tmo <= 0) {
+			bprintf(vt->resp_buf,
+				"Poll (read) timeout %.3fs exceeded by %.3fs",
+				vt->timeout, t_now - t_end);
 			VTCP_close(&s);
 			return;
 		}
@@ -339,18 +371,27 @@ vbp_poke(struct vbp_target *vt)
 			    sizeof vt->resp_buf - rlen);
 		else
 			i = read(s, buf, sizeof buf);
+		if (i <= 0) {
+			if (i < 0)
+				bprintf(vt->resp_buf, "Read error %d (%s)",
+					errno, strerror(errno));
+			break;
+		}
 		rlen += i;
-	} while (i > 0);
+	}
 
 	VTCP_close(&s);
 
 	if (i < 0) {
+		/* errno reported above */
 		vt->err_recv |= 1;
 		return;
 	}
 
-	if (rlen == 0)
+	if (rlen == 0) {
+		bprintf(vt->resp_buf, "%s", "Empty response");
 		return;
+	}
 
 	/* So we have a good receive ... */
 	t_now = VTIM_real();
@@ -375,7 +416,7 @@ vbp_poke(struct vbp_target *vt)
 /*--------------------------------------------------------------------
  */
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 vbp_task(struct worker *wrk, void *priv)
 {
 	struct vbp_target *vt;
@@ -390,7 +431,7 @@ vbp_task(struct worker *wrk, void *priv)
 	vbp_start_poke(vt);
 	vbp_poke(vt);
 	vbp_has_poked(vt);
-	vbp_update_backend(vt);
+	VBP_Update_Backend(vt);
 
 	Lck_Lock(&vbp_mtx);
 	if (vt->running < 0) {
@@ -410,10 +451,10 @@ vbp_task(struct worker *wrk, void *priv)
 /*--------------------------------------------------------------------
  */
 
-static void * __match_proto__()
+static void * v_matchproto_(bgthread_t)
 vbp_thread(struct worker *wrk, void *priv)
 {
-	double now, nxt;
+	vtim_real now, nxt;
 	struct vbp_target *vt;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -452,58 +493,74 @@ vbp_thread(struct worker *wrk, void *priv)
  */
 
 static void
-vbp_bitmap(struct cli *cli, char c, uint64_t map, const char *lbl)
+vbp_bitmap(struct vsb *vsb, char c, uint64_t map, const char *lbl)
 {
 	int i;
 	uint64_t u = (1ULL << 63);
 
-	VCLI_Out(cli, "  ");
+	VSB_printf(vsb, "  ");
 	for (i = 0; i < 64; i++) {
 		if (map & u)
-			VCLI_Out(cli, "%c", c);
+			VSB_putc(vsb, c);
 		else
-			VCLI_Out(cli, "-");
+			VSB_putc(vsb, '-');
 		map <<= 1;
 	}
-	VCLI_Out(cli, " %s\n", lbl);
+	VSB_printf(vsb, " %s\n", lbl);
 }
 
 /*lint -e{506} constant value boolean */
 /*lint -e{774} constant value boolean */
-static void
-vbp_health_one(struct cli *cli, const struct vbp_target *vt)
+void
+VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 {
+	struct vbp_target *vt;
+	char buf[12];
 
-	VCLI_Out(cli,
-	    "  Current states  good: %2u threshold: %2u window: %2u\n",
+	CHECK_OBJ_NOTNULL(be, BACKEND_MAGIC);
+	vt = be->probe;
+	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
+
+	if (!details) {
+		bprintf(buf, "%u/%u %s", vt->good, vt->window,
+		    vt->backend->director->sick ? "bad" : "good");
+		if (json)
+			VSB_printf(vsb, "\"%s\"", buf);
+		else
+			VSB_printf(vsb, "%-10s", buf);
+		return;
+	}
+
+	if (json) {
+		VSB_printf(vsb, "{\n");
+		VSB_indent(vsb, 2);
+#define BITMAP(nn, cc, tt, bb)					\
+		VSB_printf(vsb, "\"bits_%c\": %ju,\n", cc, vt->nn);
+#include "tbl/backend_poll.h"
+		VSB_printf(vsb, "\"good\": %u,\n", vt->good);
+		VSB_printf(vsb, "\"threshold\": %u,\n", vt->threshold);
+		VSB_printf(vsb, "\"window\": %u", vt->window);
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "\n");
+		VSB_printf(vsb, "},\n");
+		return;
+	}
+
+	VSB_printf(vsb,
+	    "\nCurrent states  good: %2u threshold: %2u window: %2u\n",
 	    vt->good, vt->threshold, vt->window);
-	VCLI_Out(cli,
+	VSB_printf(vsb,
 	    "  Average response time of good probes: %.6f\n", vt->avg);
-	VCLI_Out(cli,
+	VSB_printf(vsb,
 	    "  Oldest ======================"
 	    "============================ Newest\n");
 
 #define BITMAP(n, c, t, b)					\
 		if ((vt->n != 0) || (b))			\
-			vbp_bitmap(cli, (c), vt->n, (t));
+			vbp_bitmap(vsb, (c), vt->n, (t));
 #include "tbl/backend_poll.h"
 }
 
-void
-VBP_Status(struct cli *cli, const struct backend *be, int details)
-{
-	struct vbp_target *vt;
-
-	CHECK_OBJ_NOTNULL(be, BACKEND_MAGIC);
-	vt = be->probe;
-	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
-	VCLI_Out(cli, "%d/%d", vt->good, vt->window);
-	if (details) {
-		VCLI_Out(cli, "\n");
-		vbp_health_one(cli, vt);
-	}
-}
-
 /*--------------------------------------------------------------------
  * Build request from probe spec
  */
@@ -578,7 +635,7 @@ VBP_Control(const struct backend *be, int enable)
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
 	vbp_reset(vt);
-	vbp_update_backend(vt);
+	VBP_Update_Backend(vt);
 
 	Lck_Lock(&vbp_mtx);
 	if (enable) {
@@ -612,6 +669,7 @@ VBP_Insert(struct backend *b, const struct vrt_backend_probe *vp,
 	XXXAN(vt);
 
 	vt->tcp_pool = tp;
+	VTP_AddRef(vt->tcp_pool);
 	vt->backend = b;
 	b->probe = vt;
 
@@ -619,7 +677,7 @@ VBP_Insert(struct backend *b, const struct vrt_backend_probe *vp,
 	vbp_build_req(vt, vp, b);
 
 	vbp_reset(vt);
-	vbp_update_backend(vt);
+	VBP_Update_Backend(vt);
 }
 
 void
@@ -632,7 +690,7 @@ VBP_Remove(struct backend *be)
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
 	Lck_Lock(&vbp_mtx);
-	be->healthy = 1;
+	VRT_SetHealth(be->director, 1);
 	be->probe = NULL;
 	vt->backend = NULL;
 	if (vt->running) {
@@ -648,7 +706,7 @@ VBP_Remove(struct backend *be)
 
 /*-------------------------------------------------------------------*/
 
-static int __match_proto__(binheap_cmp_t)
+static int v_matchproto_(binheap_cmp_t)
 vbp_cmp(void *priv, const void *a, const void *b)
 {
 	const struct vbp_target *aa, *bb;
@@ -663,7 +721,7 @@ vbp_cmp(void *priv, const void *a, const void *b)
 	return (aa->due < bb->due);
 }
 
-static void __match_proto__(binheap_update_t)
+static void v_matchproto_(binheap_update_t)
 vbp_update(void *priv, void *p, unsigned u)
 {
 	struct vbp_target *vt;
