@@ -32,10 +32,10 @@
 
 #include <pcre.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
 #include "cache_ban.h"
+#include "cache_objhead.h"
 
-#include "hash/hash_slinger.h"
 #include "vcli_serve.h"
 #include "vend.h"
 #include "vmb.h"
@@ -152,7 +152,7 @@ ban_equal(const uint8_t *bs1, const uint8_t *bs2)
 }
 
 void
-ban_mark_completed(struct ban *b)
+ban_mark_completed(struct ban *b, struct VSC_main *stats)
 {
 	unsigned ln;
 
@@ -166,8 +166,12 @@ ban_mark_completed(struct ban *b)
 		b->spec[BANS_FLAGS] |= BANS_FLAG_COMPLETED;
 		VWMB();
 		vbe32enc(b->spec + BANS_LENGTH, BANS_HEAD_LEN);
-		VSC_C_main->bans_completed++;
+		stats->bans_completed++;
 		bans_persisted_fragmentation += ln - ban_len(b->spec);
+		/*
+		 * XXX absolute update of gauges - may be inaccurate for
+		 * Pool_Sumstat race
+		 */
 		VSC_C_main->bans_persisted_fragmentation =
 		    bans_persisted_fragmentation;
 	}
@@ -315,6 +319,10 @@ ban_export(void)
 	assert(VSB_len(vsb) == ln);
 	STV_BanExport((const uint8_t *)VSB_data(vsb), VSB_len(vsb));
 	VSB_destroy(&vsb);
+	/*
+	 * XXX absolute update of gauges - may be inaccurate for Pool_Sumstat
+	 * race
+	 */
 	VSC_C_main->bans_persisted_bytes =
 	    bans_persisted_bytes = ln;
 	VSC_C_main->bans_persisted_fragmentation =
@@ -359,7 +367,7 @@ ban_reload(const uint8_t *ban, unsigned len)
 	struct ban *b, *b2;
 	int duplicate = 0;
 	double t0, t1, t2 = 9e99;
-
+	struct VSC_main *stats = VSC_C_main;	// XXX accurate?
 	ASSERT_CLI();
 	Lck_AssertHeld(&ban_mtx);
 
@@ -378,8 +386,8 @@ ban_reload(const uint8_t *ban, unsigned len)
 			duplicate = 1;
 	}
 
-	VSC_C_main->bans++;
-	VSC_C_main->bans_added++;
+	stats->bans++;
+	stats->bans_added++;
 
 	b2 = ban_alloc();
 	AN(b2);
@@ -387,18 +395,22 @@ ban_reload(const uint8_t *ban, unsigned len)
 	AN(b2->spec);
 	memcpy(b2->spec, ban, len);
 	if (ban[BANS_FLAGS] & BANS_FLAG_REQ) {
-		VSC_C_main->bans_req++;
+		stats->bans_req++;
 		b2->flags |= BANS_FLAG_REQ;
 	}
 	if (duplicate)
-		VSC_C_main->bans_dups++;
+		stats->bans_dups++;
 	if (duplicate || (ban[BANS_FLAGS] & BANS_FLAG_COMPLETED))
-		ban_mark_completed(b2);
+		ban_mark_completed(b2, stats);
 	if (b == NULL)
 		VTAILQ_INSERT_TAIL(&ban_head, b2, list);
 	else
 		VTAILQ_INSERT_BEFORE(b, b2, list);
 	bans_persisted_bytes += len;
+	/*
+	 * XXX absolute update of gauges - may be inaccurate for Pool_Sumstat
+	 * race
+	 */
 	VSC_C_main->bans_persisted_bytes = bans_persisted_bytes;
 
 	/* Hunt down older duplicates */
@@ -406,8 +418,8 @@ ban_reload(const uint8_t *ban, unsigned len)
 		if (b->flags & BANS_FLAG_COMPLETED)
 			continue;
 		if (ban_equal(b->spec, ban)) {
-			ban_mark_completed(b);
-			VSC_C_main->bans_dups++;
+			ban_mark_completed(b, stats);
+			stats->bans_dups++;
 		}
 	}
 }
@@ -536,13 +548,16 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 	struct vsl_log *vsl;
 	struct ban *b0, *bn;
 	unsigned tests;
+	struct VSC_main *stats;
 
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	Lck_AssertHeld(&oc->objhead->mtx);
 	assert(oc->refcnt > 0);
 
 	vsl = req->vsl;
+	stats = wrk->stats;
 
 	CHECK_OBJ_NOTNULL(oc->ban, BAN_MAGIC);
 
@@ -585,8 +600,8 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 
 	Lck_Lock(&ban_mtx);
 	bn->refcount--;
-	VSC_C_main->bans_tested++;
-	VSC_C_main->bans_tests_tested += tests;
+	stats->bans_tested++;
+	stats->bans_tests_tested += tests;
 
 	if (b == bn) {
 		/* not banned */
@@ -609,7 +624,7 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 		return (0);
 	} else {
 		VSLb(vsl, SLT_ExpBan, "%u banned lookup", ObjGetXID(wrk, oc));
-		VSC_C_main->bans_obj_killed++;
+		stats->bans_obj_killed++;
 		return (1);
 	}
 }
@@ -618,7 +633,7 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
  * CLI functions to add bans
  */
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_ban(struct cli *cli, const char * const *av, void *priv)
 {
 	int narg, i;
@@ -655,8 +670,10 @@ ccf_ban(struct cli *cli, const char * const *av, void *priv)
 			break;
 	}
 
-	if (err == NULL)
-		err = BAN_Commit(bp);
+	if (err == NULL) {
+		// XXX racy - grab wstat lock?
+		err = BAN_Commit(bp, VSC_C_main);
+	}
 
 	if (err != NULL) {
 		VCLI_Out(cli, "%s", err);
@@ -707,7 +724,7 @@ ban_render(struct cli *cli, const uint8_t *bs)
 	}
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
 {
 	struct ban *b, *bl;
@@ -804,9 +821,9 @@ BAN_Init(void)
 	bp = BAN_Build();
 	AN(bp);
 	AZ(pthread_cond_init(&ban_lurker_cond, NULL));
-	AZ(BAN_Commit(bp));
+	AZ(BAN_Commit(bp, VSC_C_main));
 	Lck_Lock(&ban_mtx);
-	ban_mark_completed(VTAILQ_FIRST(&ban_head));
+	ban_mark_completed(VTAILQ_FIRST(&ban_head), VSC_C_main);
 	Lck_Unlock(&ban_mtx);
 }
 
