@@ -32,13 +32,11 @@
 
 #include "config.h"
 
-#include <stddef.h>
 #include <stdlib.h>
 
-#include "cache.h"
-
-#include "hash/hash_slinger.h"
-#include "cache/cache_filter.h"
+#include "cache_varnishd.h"
+#include "cache_filter.h"
+#include "cache_objhead.h"
 
 static struct mempool		*vbopool;
 
@@ -123,6 +121,11 @@ VBO_GetBusyObj(struct worker *wrk, const struct req *req)
 	p = (void*)PRNDUP(p);
 	assert(p < bo->end);
 
+	bo->vfc = (void*)p;
+	p += sizeof (*bo->vfc);
+	p = (void*)PRNDUP(p);
+	INIT_OBJ(bo->vfc, VFP_CTX_MAGIC);
+
 	WS_Init(bo->ws, "bo", p, bo->end - p);
 
 	bo->do_stream = 1;
@@ -135,10 +138,6 @@ VBO_GetBusyObj(struct worker *wrk, const struct req *req)
 
 	memcpy(bo->digest, req->digest, sizeof bo->digest);
 
-	VRTPRIV_init(bo->privs);
-
-	VFP_Setup(bo->vfc);
-
 	return (bo);
 }
 
@@ -154,8 +153,7 @@ VBO_ReleaseBusyObj(struct worker *wrk, struct busyobj **pbo)
 	AZ(bo->htc);
 	AZ(bo->stale_oc);
 
-	VRTPRIV_dynamic_kill(bo->privs, (uintptr_t)bo);
-	assert(VTAILQ_EMPTY(&bo->privs->privs));
+	AZ(bo->privs->magic);
 
 	VSLb(bo->vsl, SLT_BereqAcct, "%ju %ju %ju %ju %ju %ju",
 	    (uintmax_t)bo->acct.bereq_hdrbytes,
