@@ -30,12 +30,13 @@
 
 #include "config.h"
 
-#include "cache.h"
+#include "cache_varnishd.h"
 
 #include <stdlib.h>
 
 #include "cache_transport.h"
 #include "cache_filter.h"
+#include "cache_vgz.h"
 
 #include "vtim.h"
 #include "cache_esi.h"
@@ -76,7 +77,7 @@ static const struct transport VED_transport = {
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(vtr_reembark_f)
+static void v_matchproto_(vtr_reembark_f)
 ved_reembark(struct worker *wrk, struct req *req)
 {
 	struct ecx *ecx;
@@ -164,11 +165,6 @@ ved_include(struct req *preq, const char *src, const char *host,
 	req->vcl = preq->vcl;
 	preq->vcl = NULL;
 
-	/*
-	 * XXX: We should decide if we should cache the director
-	 * XXX: or not (for session/backend coupling).  Until then
-	 * XXX: make sure we don't trip up the check in vcl_recv.
-	 */
 	req->req_step = R_STP_RECV;
 	req->t_req = preq->t_req;
 	assert(isnan(req->t_first));
@@ -201,8 +197,6 @@ ved_include(struct req *preq, const char *src, const char *host,
 		AZ(req->wrk);
 	}
 
-	VRTPRIV_dynamic_kill(sp->privs, (uintptr_t)req);
-
 	AZ(preq->vcl);
 	preq->vcl = req->vcl;
 	req->vcl = NULL;
@@ -253,8 +247,8 @@ ved_decode_len(struct req *req, const uint8_t **pp)
 /*---------------------------------------------------------------------
  */
 
-int __match_proto__(vdp_bytes)
-VDP_ESI(struct req *req, enum vdp_action act, void **priv,
+static int v_matchproto_(vdp_bytes)
+ved_vdp(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	uint8_t *q, *r;
@@ -435,6 +429,11 @@ VDP_ESI(struct req *req, enum vdp_action act, void **priv,
 	}
 }
 
+const struct vdp VDP_esi = {
+	.name =		"esi",
+	.func =		ved_vdp,
+};
+
 /*
  * Account body bytes on req
  * Push bytes to preq
@@ -465,7 +464,7 @@ ved_bytes(struct req *req, struct req *preq, enum vdp_action act,
  * the stream with a bit more overhead.
  */
 
-static int __match_proto__(vdp_bytes)
+static int v_matchproto_(vdp_bytes)
 ved_pretend_gzip(struct req *req, enum vdp_action act, void **priv,
     const void *pv, ssize_t l)
 {
@@ -522,6 +521,11 @@ ved_pretend_gzip(struct req *req, enum vdp_action act, void **priv,
 	return (ved_bytes(req, preq, VDP_FLUSH, NULL, 0));
 }
 
+static const struct vdp ved_vdp_pgz = {
+	.name =		"PGZ",
+	.func =		ved_pretend_gzip,
+};
+
 /*---------------------------------------------------------------------
  * Include an object in a gzip'ed ESI object delivery
  *
@@ -761,7 +765,7 @@ ved_stripgzip(struct req *req, const struct boc *boc)
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(vdp_bytes)
+static int v_matchproto_(vdp_bytes)
 ved_vdp_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
@@ -778,9 +782,14 @@ ved_vdp_bytes(struct req *req, enum vdp_action act, void **priv,
 	return (ved_bytes(req, preq, act, ptr, len));
 }
 
+static const struct vdp ved_ved = {
+	.name =		"VED",
+	.func =		ved_vdp_bytes,
+};
+
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(vtr_deliver_f)
+static void v_matchproto_(vtr_deliver_f)
 ved_deliver(struct req *req, struct boc *boc, int wantbody)
 {
 	int i;
@@ -804,9 +813,9 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 		ved_stripgzip(req, boc);
 	} else {
 		if (ecx->isgzip && !i)
-			VDP_push(req, ved_pretend_gzip, ecx, 1, "PGZ");
+			(void)VDP_push(req, &ved_vdp_pgz, ecx, 1);
 		else
-			VDP_push(req, ved_vdp_bytes, ecx->preq, 1, "VED");
+			(void)VDP_push(req, &ved_ved, ecx->preq, 1);
 		(void)VDP_DeliverObj(req);
 		(void)VDP_bytes(req, VDP_FLUSH, NULL, 0);
 	}
