@@ -29,9 +29,9 @@
 
 #include "config.h"
 
-#include "cache.h"
-#include "cache_director.h"
+#include "cache_varnishd.h"
 #include "cache_filter.h"
+#include "cache_objhead.h"
 #include "hash/hash_slinger.h"
 #include "storage/storage.h"
 #include "vcl.h"
@@ -62,7 +62,6 @@ vbf_allocobj(struct busyobj *bo, unsigned l)
 		stv = bo->storage;
 
 	bo->storage = NULL;
-	bo->storage_hint = NULL;
 
 	if (stv == NULL)
 		return (0);
@@ -129,8 +128,12 @@ vbf_beresp2obj(struct busyobj *bo)
 	if (bo->uncacheable)
 		bo->fetch_objcore->flags |= OC_F_PASS;
 
-	if (!vbf_allocobj(bo, l))
+	if (!vbf_allocobj(bo, l)) {
+		if (vary != NULL)
+			VSB_destroy(&vary);
+		AZ(vary);
 		return (-1);
+	}
 
 	if (vary != NULL) {
 		AN(ObjSetAttr(bo->wrk, bo->fetch_objcore, OA_VARY, varyl,
@@ -160,7 +163,7 @@ vbf_beresp2obj(struct busyobj *bo)
 }
 
 /*--------------------------------------------------------------------
- * Copy req->bereq and release req if not pass fetch
+ * Copy req->bereq and release req if no body
  */
 
 static enum fetch_step
@@ -174,28 +177,25 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 
 	assert(bo->fetch_objcore->boc->state == BOS_INVALID);
 	AZ(bo->storage);
-	AZ(bo->storage_hint);
 
 	HTTP_Setup(bo->bereq0, bo->ws, bo->vsl, SLT_BereqMethod);
 	http_FilterReq(bo->bereq0, bo->req->http,
 	    bo->do_pass ? HTTPH_R_PASS : HTTPH_R_FETCH);
 
-	if (!bo->do_pass) {
+	if (bo->do_pass)
+		AZ(bo->stale_oc);
+	else {
 		http_ForceField(bo->bereq0, HTTP_HDR_METHOD, "GET");
-		http_ForceField(bo->bereq0, HTTP_HDR_PROTO, "HTTP/1.1");
 		if (cache_param->http_gzip_support)
 			http_ForceHeader(bo->bereq0, H_Accept_Encoding, "gzip");
-	} else {
-		AZ(bo->stale_oc);
-		if (bo->bereq0->protover > 11)
-			http_ForceField(bo->bereq0, HTTP_HDR_PROTO, "HTTP/1.1");
 	}
+	http_ForceField(bo->bereq0, HTTP_HDR_PROTO, "HTTP/1.1");
 	http_CopyHome(bo->bereq0);
 
 	if (bo->stale_oc != NULL &&
 	    ObjCheckFlag(bo->wrk, bo->stale_oc, OF_IMSCAND) &&
 	    (bo->stale_oc->boc != NULL || ObjGetLen(wrk, bo->stale_oc) != 0)) {
-		AZ(bo->stale_oc->flags & OC_F_PASS);
+		AZ(bo->stale_oc->flags & (OC_F_PASS|OC_F_PRIVATE));
 		q = HTTP_GetHdrPack(bo->wrk, bo->stale_oc, H_Last_Modified);
 		if (q != NULL)
 			http_PrintfHeader(bo->bereq0,
@@ -225,12 +225,8 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 static enum fetch_step
 vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 {
-	struct vfp_ctx *vfc;
-
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
-	vfc = bo->vfc;
-	CHECK_OBJ_NOTNULL(vfc, VFP_CTX_MAGIC);
 
 	assert(bo->fetch_objcore->boc->state <= BOS_REQ_DONE);
 
@@ -241,12 +237,10 @@ vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 
 	/* reset other bo attributes - See VBO_GetBusyObj */
 	bo->storage = NULL;
-	bo->storage_hint = NULL;
 	bo->do_esi = 0;
 	bo->do_stream = 1;
-
-	/* reset fetch processors */
-	VFP_Setup(vfc);
+	bo->filter_list = NULL;
+	bo->was_304 = 0;
 
 	// XXX: BereqEnd + BereqAcct ?
 	VSL_ChgId(bo->vsl, "bereq", "retry", VXID_Get(wrk, VSL_BACKENDMARKER));
@@ -270,9 +264,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 
 	AZ(bo->storage);
-	AZ(bo->storage_hint);
-
-	bo->storage = STV_next();
+	bo->storage = bo->do_pass ? stv_transient : STV_next();
 
 	if (bo->retries > 0)
 		http_Unset(bo->bereq, "\012X-Varnish:");
@@ -292,7 +284,13 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	assert(bo->fetch_objcore->boc->state <= BOS_REQ_DONE);
 
 	AZ(bo->htc);
-	i = VDI_GetHdr(wrk, bo);
+
+	VFP_Setup(bo->vfc, wrk);
+	bo->vfc->oc = bo->fetch_objcore;
+	bo->vfc->resp = bo->beresp;
+	bo->vfc->req = bo->bereq;
+
+	i = VDI_GetHdr(bo);
 
 	now = W_TIM_real(wrk);
 	VSLb_ts_busyobj(bo, "Beresp", now);
@@ -304,6 +302,14 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 
 	http_VSL_log(bo->beresp);
 
+	if (bo->htc->body_status == BS_ERROR) {
+		bo->htc->doclose = SC_RX_BODY;
+		VDI_Finish(bo);
+		VSLb(bo->vsl, SLT_Error, "Body cannot be fetched");
+		assert(bo->director_state == DIR_S_NULL);
+		return (F_STP_ERROR);
+	}
+
 	if (!http_GetHdr(bo->beresp, H_Date, NULL)) {
 		/*
 		 * RFC 2616 14.18 Date: The Date general-header field
@@ -327,68 +333,6 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	http_CollectHdr(bo->beresp, H_Cache_Control);
 	http_CollectHdr(bo->beresp, H_Vary);
 
-	/*
-	 * Figure out how the fetch is supposed to happen, before the
-	 * headers are adultered by VCL
-	 */
-	if (!strcasecmp(http_GetMethod(bo->bereq), "head")) {
-		/*
-		 * A HEAD request can never have a body in the reply,
-		 * no matter what the headers might say.
-		 * [RFC7231 4.3.2 p25]
-		 */
-		wrk->stats->fetch_head++;
-		bo->htc->body_status = BS_NONE;
-	} else if (http_GetStatus(bo->beresp) <= 199) {
-		/*
-		 * 1xx responses never have a body.
-		 * [RFC7230 3.3.2 p31]
-		 * ... but we should never see them.
-		 */
-		wrk->stats->fetch_1xx++;
-		bo->htc->body_status = BS_ERROR;
-	} else if (http_IsStatus(bo->beresp, 204)) {
-		/*
-		 * 204 is "No Content", obviously don't expect a body.
-		 * [RFC7230 3.3.1 p29 and 3.3.2 p31]
-		 */
-		wrk->stats->fetch_204++;
-		if ((http_GetHdr(bo->beresp, H_Content_Length, NULL) &&
-		    bo->htc->content_length != 0) ||
-		    http_GetHdr(bo->beresp, H_Transfer_Encoding, NULL))
-			bo->htc->body_status = BS_ERROR;
-		else
-			bo->htc->body_status = BS_NONE;
-	} else if (http_IsStatus(bo->beresp, 304)) {
-		/*
-		 * 304 is "Not Modified" it has no body.
-		 * [RFC7230 3.3 p28]
-		 */
-		wrk->stats->fetch_304++;
-		bo->htc->body_status = BS_NONE;
-	} else if (bo->htc->body_status == BS_CHUNKED) {
-		wrk->stats->fetch_chunked++;
-	} else if (bo->htc->body_status == BS_LENGTH) {
-		assert(bo->htc->content_length > 0);
-		wrk->stats->fetch_length++;
-	} else if (bo->htc->body_status == BS_EOF) {
-		wrk->stats->fetch_eof++;
-	} else if (bo->htc->body_status == BS_ERROR) {
-		wrk->stats->fetch_bad++;
-	} else if (bo->htc->body_status == BS_NONE) {
-		wrk->stats->fetch_none++;
-	} else {
-		WRONG("wrong bodystatus");
-	}
-
-	if (bo->htc->body_status == BS_ERROR) {
-		bo->htc->doclose = SC_RX_BODY;
-		VDI_Finish(bo->wrk, bo);
-		VSLb(bo->vsl, SLT_Error, "Body cannot be fetched");
-		assert(bo->director_state == DIR_S_NULL);
-		return (F_STP_ERROR);
-	}
-
 	if (bo->fetch_objcore->flags & OC_F_PRIVATE) {
 		/* private objects have negative TTL */
 		bo->fetch_objcore->t_origin = now;
@@ -411,6 +355,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	if (http_IsStatus(bo->beresp, 304)) {
 		if (bo->stale_oc != NULL &&
 		    ObjCheckFlag(bo->wrk, bo->stale_oc, OF_IMSCAND)) {
+			AZ(bo->stale_oc->flags & (OC_F_PASS|OC_F_PRIVATE));
 			if (ObjCheckFlag(bo->wrk, bo->stale_oc, OF_CHGGZIP)) {
 				/*
 				 * If we changed the gzip status of the object
@@ -431,22 +376,16 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 			VSLb(bo->vsl, SLT_Error,
 			    "304 response but not conditional fetch");
 			bo->htc->doclose = SC_RX_BAD;
-			VDI_Finish(bo->wrk, bo);
+			VDI_Finish(bo);
 			return (F_STP_ERROR);
 		}
 	}
 
-	bo->vfc->bo = bo;
-	bo->vfc->oc = bo->fetch_objcore;
-	bo->vfc->wrk = bo->wrk;
-	bo->vfc->http = bo->beresp;
-	bo->vfc->esi_req = bo->bereq;
-
 	VCL_backend_response_method(bo->vcl, wrk, NULL, bo, NULL);
 
 	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL) {
 		bo->htc->doclose = SC_RESP_CLOSE;
-		VDI_Finish(bo->wrk, bo);
+		VDI_Finish(bo);
 		return (F_STP_FAIL);
 	}
 
@@ -454,7 +393,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 		if (bo->htc->body_status != BS_NONE)
 			bo->htc->doclose = SC_RESP_CLOSE;
 		if (bo->director_state != DIR_S_NULL)
-			VDI_Finish(bo->wrk, bo);
+			VDI_Finish(bo);
 
 		if (bo->retries++ < cache_param->max_retries)
 			return (F_STP_RETRY);
@@ -512,12 +451,12 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 		if (vfc->oc->flags & OC_F_ABANDON) {
 			/*
 			 * A pass object and delivery was terminated
-			 * We don't fail the fetch, in order for hit-for-pass
+			 * We don't fail the fetch, in order for HitMiss
 			 * objects to be created.
 			 */
 			AN(vfc->oc->flags & OC_F_PASS);
-			VSLb(wrk->vsl, SLT_FetchError,
-			    "Pass delivery abandoned");
+			VSLb(wrk->vsl, SLT_Debug,
+			    "Fetch: Pass delivery abandoned");
 			bo->htc->doclose = SC_RX_BODY;
 			break;
 		}
@@ -545,7 +484,7 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 		(void)VFP_Error(vfc, "Fetch pipeline failed to process");
 		bo->htc->doclose = SC_RX_BODY;
 		VFP_Close(vfc);
-		VDI_Finish(wrk, bo);
+		VDI_Finish(bo);
 		if (!bo->do_stream) {
 			assert(bo->fetch_objcore->boc->state < BOS_STREAM);
 			// XXX: doclose = ?
@@ -563,17 +502,92 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-#define vbf_vfp_push(bo, vfp, top)					\
-	do {								\
-		if (VFP_Push((bo)->vfc, (vfp), (top)) == NULL) {	\
-			assert (WS_Overflowed((bo)->vfc->http->ws));	\
-			(void)VFP_Error((bo)->vfc,			\
-			    "workspace_backend overflow");		\
-			(bo)->htc->doclose = SC_OVERLOAD;		\
-			VDI_Finish((bo)->wrk, bo);			\
-			return (F_STP_ERROR);				\
-		}							\
-	} while (0)
+static void
+vbf_default_filter_list(const struct busyobj *bo, struct vsb *vsb)
+{
+	const char *p;
+	int do_gzip = bo->do_gzip;
+	int do_gunzip = bo->do_gunzip;
+	int is_gzip = 0, is_gunzip = 0;
+
+	/*
+	 * The VCL variables beresp.do_g[un]zip tells us how we want the
+	 * object processed before it is stored.
+	 *
+	 * The backend Content-Encoding header tells us what we are going
+	 * to receive, which we classify in the following three classes:
+	 *
+	 *	"Content-Encoding: gzip"	--> object is gzip'ed.
+	 *	no Content-Encoding		--> object is not gzip'ed.
+	 *	anything else			--> do nothing wrt gzip
+	 */
+
+	/* No body -> done */
+	if (bo->htc->body_status == BS_NONE || bo->htc->content_length == 0)
+		return;
+
+	if (!cache_param->http_gzip_support)
+		do_gzip = do_gunzip = 0;
+
+	if (http_GetHdr(bo->beresp, H_Content_Encoding, &p))
+		is_gzip = !strcasecmp(p, "gzip");
+	else
+		is_gunzip = 1;
+
+	/* We won't gunzip unless it is gzip'ed */
+	if (do_gunzip && !is_gzip)
+		do_gunzip = 0;
+
+	/* We wont gzip unless if it already is gzip'ed */
+	if (do_gzip && !is_gunzip)
+		do_gzip = 0;
+
+	if (do_gunzip || (is_gzip && bo->do_esi))
+		VSB_cat(vsb, " gunzip");
+
+	if (bo->do_esi && (do_gzip || (is_gzip && !do_gunzip))) {
+		VSB_cat(vsb, " esi_gzip");
+		return;
+	}
+
+	if (bo->do_esi) {
+		VSB_cat(vsb, " esi");
+		return;
+	}
+
+	if (do_gzip)
+		VSB_cat(vsb, " gzip");
+
+	if (is_gzip && !do_gunzip)
+		VSB_cat(vsb, " testgunzip");
+}
+
+const char *
+VBF_Get_Filter_List(struct busyobj *bo)
+{
+	unsigned u;
+	struct vsb vsb[1];
+
+	u = WS_Reserve(bo->ws, 0);
+	if (u == 0) {
+		WS_Release(bo->ws, 0);
+		WS_MarkOverflow(bo->ws);
+		return (NULL);
+	}
+	AN(VSB_new(vsb, bo->ws->f, u, VSB_FIXEDLEN));
+	vbf_default_filter_list(bo, vsb);
+	if (VSB_finish(vsb)) {
+		WS_Release(bo->ws, 0);
+		WS_MarkOverflow(bo->ws);
+		return (NULL);
+	}
+	if (VSB_len(vsb)) {
+		WS_Release(bo->ws, VSB_len(vsb) + 1);
+		return (VSB_data(vsb) + 1);
+	}
+	WS_Release(bo->ws, 0);
+	return ("");
+}
 
 static enum fetch_step
 vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
@@ -586,79 +600,32 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 
 	assert(wrk->handling == VCL_RET_DELIVER);
 
-	/*
-	 * The VCL variables beresp.do_g[un]zip tells us how we want the
-	 * object processed before it is stored.
-	 *
-	 * The backend Content-Encoding header tells us what we are going
-	 * to receive, which we classify in the following three classes:
-	 *
-	 *	"Content-Encoding: gzip"	--> object is gzip'ed.
-	 *	no Content-Encoding		--> object is not gzip'ed.
-	 *	anything else			--> do nothing wrt gzip
-	 *
-	 */
-
-	/* We do nothing unless the param is set */
-	if (!cache_param->http_gzip_support)
-		bo->do_gzip = bo->do_gunzip = 0;
-
-	if (bo->htc->content_length == 0)
+	/* No body -> done */
+	if (bo->htc->body_status == BS_NONE || bo->htc->content_length == 0) {
 		http_Unset(bo->beresp, H_Content_Encoding);
-
-	if (bo->htc->body_status != BS_NONE) {
-		bo->is_gzip =
-		    http_HdrIs(bo->beresp, H_Content_Encoding, "gzip");
-		bo->is_gunzip =
-		    !http_GetHdr(bo->beresp, H_Content_Encoding, NULL);
-		assert(bo->is_gzip == 0 || bo->is_gunzip == 0);
+		bo->do_gzip = bo->do_gunzip = 0;
+		bo->do_stream = 0;
+		bo->filter_list = "";
+	} else if (bo->filter_list == NULL) {
+		bo->filter_list = VBF_Get_Filter_List(bo);
 	}
 
-	/* We won't gunzip unless it is non-empty and gzip'ed */
-	if (bo->htc->body_status == BS_NONE ||
-	    bo->htc->content_length == 0 ||
-	    (bo->do_gunzip && !bo->is_gzip))
-		bo->do_gunzip = 0;
-
-	/* We wont gzip unless it is non-empty and ungzip'ed */
-	if (bo->htc->body_status == BS_NONE ||
-	    bo->htc->content_length == 0 ||
-	    (bo->do_gzip && !bo->is_gunzip))
-		bo->do_gzip = 0;
-
-	/* But we can't do both at the same time */
-	assert(bo->do_gzip == 0 || bo->do_gunzip == 0);
-
-	if (bo->do_gunzip || (bo->is_gzip && bo->do_esi))
-		vbf_vfp_push(bo, &vfp_gunzip, 1);
-
-	if (bo->htc->content_length != 0) {
-		if (bo->do_esi && bo->do_gzip) {
-			vbf_vfp_push(bo, &vfp_esi_gzip, 1);
-		} else if (bo->do_esi && bo->is_gzip && !bo->do_gunzip) {
-			vbf_vfp_push(bo, &vfp_esi_gzip, 1);
-		} else if (bo->do_esi) {
-			vbf_vfp_push(bo, &vfp_esi, 1);
-		} else if (bo->do_gzip) {
-			vbf_vfp_push(bo, &vfp_gzip, 1);
-		} else if (bo->is_gzip && !bo->do_gunzip) {
-			vbf_vfp_push(bo, &vfp_testgunzip, 1);
-		}
+	if (bo->filter_list == NULL ||
+	    VCL_StackVFP(bo->vfc, bo->vcl, bo->filter_list)) {
+		(bo)->htc->doclose = SC_OVERLOAD;
+		VDI_Finish(bo);
+		return (F_STP_ERROR);
 	}
 
 	if (bo->fetch_objcore->flags & OC_F_PRIVATE)
 		AN(bo->uncacheable);
 
-	/* No reason to try streaming a non-existing body */
-	if (bo->htc->body_status == BS_NONE)
-		bo->do_stream = 0;
-
 	bo->fetch_objcore->boc->len_so_far = 0;
 
 	if (VFP_Open(bo->vfc)) {
 		(void)VFP_Error(bo->vfc, "Fetch pipeline failed to open");
 		bo->htc->doclose = SC_RX_BODY;
-		VDI_Finish(bo->wrk, bo);
+		VDI_Finish(bo);
 		return (F_STP_ERROR);
 	}
 
@@ -666,18 +633,14 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 		(void)VFP_Error(bo->vfc, "Could not get storage");
 		bo->htc->doclose = SC_RX_BODY;
 		VFP_Close(bo->vfc);
-		VDI_Finish(bo->wrk, bo);
+		VDI_Finish(bo);
 		return (F_STP_ERROR);
 	}
 
-	if (bo->do_esi)
-		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_ESIPROC, 1);
-
-	if (bo->do_gzip || (bo->is_gzip && !bo->do_gunzip))
-		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_GZIPED, 1);
-
-	if (bo->do_gzip || bo->do_gunzip)
-		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_CHGGZIP, 1);
+#define OBJ_FLAG(U, l, v)						\
+	if (bo->vfc->obj_flags & OF_##U)				\
+		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_##U, 1);
+#include "tbl/obj_attr.h"
 
 	if (!(bo->fetch_objcore->flags & OC_F_PASS) &&
 	    http_IsStatus(bo->beresp, 200) && (
@@ -685,16 +648,6 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 	      http_GetHdr(bo->beresp, H_ETag, &p)))
 		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_IMSCAND, 1);
 
-	if (bo->htc->body_status != BS_NONE &&
-	    VDI_GetBody(bo->wrk, bo) != 0) {
-		(void)VFP_Error(bo->vfc,
-		    "GetBody failed - workspace_backend overflow?");
-		VFP_Close(bo->vfc);
-		bo->htc->doclose = SC_OVERLOAD;
-		VDI_Finish(bo->wrk, bo);
-		return (F_STP_ERROR);
-	}
-
 	assert(bo->fetch_objcore->boc->refcount >= 1);
 
 	assert(bo->fetch_objcore->boc->state == BOS_REQ_DONE);
@@ -736,7 +689,7 @@ vbf_stp_fetchend(struct worker *wrk, struct busyobj *bo)
 
 	/* Recycle the backend connection before setting BOS_FINISHED to
 	   give predictable backend reuse behavior for varnishtest */
-	VDI_Finish(bo->wrk, bo);
+	VDI_Finish(bo);
 
 	ObjSetState(wrk, bo->fetch_objcore, BOS_FINISHED);
 	VSLb_ts_busyobj(bo, "BerespBody", W_TIM_real(wrk));
@@ -787,6 +740,7 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 		    OA_ESIDATA));
 
 	AZ(ObjCopyAttr(bo->wrk, bo->fetch_objcore, bo->stale_oc, OA_FLAGS));
+	ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_IMSCAND, 0);
 	AZ(ObjCopyAttr(bo->wrk, bo->fetch_objcore, bo->stale_oc, OA_GZIPBITS));
 
 	if (bo->do_stream) {
@@ -801,7 +755,7 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 	if (bo->stale_oc->flags & OC_F_FAILED)
 		(void)VFP_Error(bo->vfc, "Template object failed");
 	if (bo->vfc->failed) {
-		VDI_Finish(bo->wrk, bo);
+		VDI_Finish(bo);
 		wrk->stats->fetch_failed++;
 		return (F_STP_FAIL);
 	}
@@ -834,6 +788,9 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 	if (bo->fetch_objcore->stobj->stevedore != NULL)
 		ObjFreeObj(bo->wrk, bo->fetch_objcore);
 
+	if (bo->storage == NULL)
+		bo->storage = STV_next();
+
 	// XXX: reset all beresp flags ?
 
 	HTTP_Setup(bo->beresp, bo->ws, bo->vsl, SLT_BerespMethod);
@@ -881,11 +838,10 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 
 	assert(wrk->handling == VCL_RET_DELIVER);
 
-	bo->vfc->bo = bo;
-	bo->vfc->wrk = bo->wrk;
-	bo->vfc->oc = bo->fetch_objcore;
-	bo->vfc->http = bo->beresp;
-	bo->vfc->esi_req = bo->bereq;
+	assert(bo->vfc->wrk == bo->wrk);
+	assert(bo->vfc->oc == bo->fetch_objcore);
+	assert(bo->vfc->resp == bo->beresp);
+	assert(bo->vfc->req == bo->bereq);
 
 	if (vbf_beresp2obj(bo)) {
 		(void)VFP_Error(bo->vfc, "Could not get storage");
@@ -941,7 +897,7 @@ vbf_stp_done(void)
 	NEEDLESS(return(F_STP_DONE));
 }
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 vbf_fetch_thread(struct worker *wrk, void *priv)
 {
 	struct busyobj *bo;
@@ -970,9 +926,14 @@ vbf_fetch_thread(struct worker *wrk, void *priv)
 	}
 #endif
 
+	VCL_TaskEnter(bo->vcl, bo->privs);
 	while (stp != F_STP_DONE) {
 		CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 		assert(bo->fetch_objcore->boc->refcount >= 1);
+		if (bo->fetch_objcore->boc->state < BOS_REQ_DONE)
+			AN(bo->req);
+		else
+			AZ(bo->req);
 		switch (stp) {
 #define FETCH_STEP(l, U, arg)						\
 		case F_STP_##U:						\
@@ -986,6 +947,7 @@ vbf_fetch_thread(struct worker *wrk, void *priv)
 
 	assert(bo->director_state == DIR_S_NULL);
 
+	VCL_TaskLeave(bo->vcl, bo->privs);
 	http_Teardown(bo->bereq);
 	http_Teardown(bo->beresp);
 
