@@ -52,16 +52,17 @@
 
 #include "config.h"
 
-#include "cache.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "cache_varnishd.h"
+
+#include "cache/cache_objhead.h"
 #include "cache/cache_transport.h"
 
 #include "hash/hash_slinger.h"
+
 #include "vsha256.h"
-#include "vtim.h"
 
 struct rush {
 	unsigned		magic;
@@ -75,6 +76,7 @@ static struct objhead *private_oh;
 static void hsh_rush1(const struct worker *, struct objhead *,
     struct rush *, int);
 static void hsh_rush2(struct worker *, struct rush *);
+static int HSH_DerefObjHead(struct worker *wrk, struct objhead **poh);
 
 /*---------------------------------------------------------------------*/
 
@@ -175,10 +177,10 @@ HSH_AddString(struct req *req, void *ctx, const char *str)
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	AN(ctx);
 	if (str != NULL) {
-		SHA256_Update(ctx, str, strlen(str));
+		VSHA256_Update(ctx, str, strlen(str));
 		VSLb(req->vsl, SLT_Hash, "%s", str);
 	} else
-		SHA256_Update(ctx, &str, 1);
+		VSHA256_Update(ctx, &str, 1);
 }
 
 /*---------------------------------------------------------------------
@@ -189,8 +191,8 @@ HSH_AddString(struct req *req, void *ctx, const char *str)
  */
 
 static struct hsh_magiclist {
-	unsigned char was[SHA256_LEN];
-	unsigned char now[SHA256_LEN];
+	unsigned char was[VSHA256_LEN];
+	unsigned char now[VSHA256_LEN];
 } hsh_magiclist[] = {
 	{ .now = {	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -239,19 +241,19 @@ hsh_testmagic(void *result)
 	static int nused = 0;
 
 	for (i = 0; i < nused; i++)
-		if (!memcmp(hsh_magiclist[i].was, result, SHA256_LEN))
+		if (!memcmp(hsh_magiclist[i].was, result, VSHA256_LEN))
 			break;
 	if (i == nused && i < HSH_NMAGIC)
-		memcpy(hsh_magiclist[nused++].was, result, SHA256_LEN);
+		memcpy(hsh_magiclist[nused++].was, result, VSHA256_LEN);
 	if (i == nused)
 		return;
 	assert(i < HSH_NMAGIC);
 	fprintf(stderr, "HASHMAGIC: <");
-	for (j = 0; j < SHA256_LEN; j++)
+	for (j = 0; j < VSHA256_LEN; j++)
 		fprintf(stderr, "%02x", ((unsigned char*)result)[j]);
 	fprintf(stderr, "> -> <");
-	memcpy(result, hsh_magiclist[i].now, SHA256_LEN);
-	for (j = 0; j < SHA256_LEN; j++)
+	memcpy(result, hsh_magiclist[i].now, VSHA256_LEN);
+	for (j = 0; j < VSHA256_LEN; j++)
 		fprintf(stderr, "%02x", ((unsigned char*)result)[j]);
 	fprintf(stderr, ">\n");
 }
@@ -404,9 +406,8 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 		if (oc->flags & OC_F_FAILED)
 			continue;
 
+		CHECK_OBJ_ORNULL(oc->boc, BOC_MAGIC);
 		if (oc->boc != NULL && oc->boc->state < BOS_STREAM) {
-			CHECK_OBJ_ORNULL(oc->boc, BOC_MAGIC);
-
 			if (req->hash_ignore_busy)
 				continue;
 
@@ -434,7 +435,7 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 				continue;
 		}
 
-		if (EXP_Ttl(req, oc) >= req->t_req) {
+		if (EXP_Ttl(req, oc) > req->t_req) {
 			/* If still valid, use it */
 			assert(oh->refcnt > 1);
 			assert(oc->objhead == oh);
@@ -443,23 +444,25 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 				VSLb(req->vsl, SLT_HitPass, "%u %.6f",
 				    ObjGetXID(wrk, oc), EXP_Dttl(req, oc));
 				oc = NULL;
+				retval = HSH_MISS;
 			} else if (oc->flags & OC_F_PASS) {
 				wrk->stats->cache_hitmiss++;
 				VSLb(req->vsl, SLT_HitMiss, "%u %.6f",
 				    ObjGetXID(wrk, oc), EXP_Dttl(req, oc));
-				oc = NULL;
 				*bocp = hsh_insert_busyobj(wrk, oh);
+				oc->refcnt++;
+				retval = HSH_MISS;
 			} else {
 				oc->refcnt++;
 				if (oc->hits < LONG_MAX)
 					oc->hits++;
+				retval = HSH_HIT;
 			}
 			Lck_Unlock(&oh->mtx);
-			if (oc == NULL)
-				return (HSH_MISS);
-			assert(HSH_DerefObjHead(wrk, &oh));
 			*ocp = oc;
-			return (HSH_HIT);
+			if (*bocp == NULL)
+				assert(HSH_DerefObjHead(wrk, &oh));
+			return (retval);
 		}
 		if (EXP_Ttl(NULL, oc) < req->t_req && /* ignore req.ttl */
 		    oc->t_origin > exp_t_origin) {
@@ -477,33 +480,45 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 		busy_found = 0;
 	}
 
-	if (exp_oc != NULL) {
-		assert(oh->refcnt > 1);
-		assert(exp_oc->objhead == oh);
-		exp_oc->refcnt++;
+	if (!busy_found) {
+		/* Insert objcore in objecthead */
+		*bocp = hsh_insert_busyobj(wrk, oh);
 
-		if (!busy_found) {
-			*bocp = hsh_insert_busyobj(wrk, oh);
-			retval = HSH_EXPBUSY;
+		if (exp_oc != NULL) {
+			assert(oh->refcnt > 1);
+			assert(exp_oc->objhead == oh);
+			exp_oc->refcnt++;
+			Lck_Unlock(&oh->mtx);
+			*ocp = exp_oc;
+
+			if (EXP_Ttl_grace(req, exp_oc) < req->t_req) {
+				retval = HSH_MISS;
+			} else {
+				if (exp_oc->hits < LONG_MAX)
+					exp_oc->hits++;
+				retval = HSH_EXPBUSY;
+			}
 		} else {
-			AZ(req->hash_ignore_busy);
-			retval = HSH_EXP;
+			Lck_Unlock(&oh->mtx);
+			retval = HSH_MISS;
 		}
-		if (exp_oc->hits < LONG_MAX)
-			exp_oc->hits++;
-		Lck_Unlock(&oh->mtx);
-		if (retval == HSH_EXP)
-			assert(HSH_DerefObjHead(wrk, &oh));
-		*ocp = exp_oc;
+
 		return (retval);
 	}
 
-	if (!busy_found) {
-		/* Insert objcore in objecthead and release mutex */
-		*bocp = hsh_insert_busyobj(wrk, oh);
-		/* NB: no deref of objhead, new object inherits reference */
+	AN(busy_found);
+	if (exp_oc != NULL && EXP_Ttl_grace(req, exp_oc) >= req->t_req) {
+		/* we do not wait on the busy object if in grace */
+		assert(oh->refcnt > 1);
+		assert(exp_oc->objhead == oh);
+		exp_oc->refcnt++;
 		Lck_Unlock(&oh->mtx);
-		return (HSH_MISS);
+		*ocp = exp_oc;
+
+		assert(HSH_DerefObjHead(wrk, &oh));
+		if (exp_oc->hits < LONG_MAX)
+			exp_oc->hits++;
+		return (HSH_EXP);
 	}
 
 	/* There are one or more busy objects, wait for them */
@@ -578,8 +593,18 @@ hsh_rush2(struct worker *wrk, struct rush *r)
 		CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 		VTAILQ_REMOVE(&r->reqs, req, w_list);
 		DSL(DBG_WAITINGLIST, req->vsl->wid, "off waiting list");
-		AN(req->transport->reembark);
-		req->transport->reembark(wrk, req);
+		if (req->transport->reembark != NULL) {
+			// For ESI includes
+			req->transport->reembark(wrk, req);
+		} else {
+			/*
+			 * We ignore the queue limits which apply to new
+			 * requests because if we fail to reschedule there
+			 * may be vmod_privs to cleanup and we need a proper
+			 * workerthread for that.
+			 */
+			AZ(Pool_Task(req->sp->pool, &req->task, TASK_QUEUE_RUSH));
+		}
 	}
 }
 
@@ -588,13 +613,12 @@ hsh_rush2(struct worker *wrk, struct rush *r)
  */
 
 unsigned
-HSH_Purge(struct worker *wrk, struct objhead *oh, double ttl, double grace,
-double keep)
+HSH_Purge(struct worker *wrk, struct objhead *oh, double ttl_now, double ttl,
+double grace, double keep)
 {
 	struct objcore *oc, **ocp;
 	unsigned spc, ospc, nobj, n, n_tot = 0;
 	int more = 0;
-	double now;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
@@ -621,7 +645,6 @@ double keep)
 		ocp = (void*)wrk->aws->f;
 		Lck_Lock(&oh->mtx);
 		assert(oh->refcnt > 0);
-		now = VTIM_real();
 		VTAILQ_FOREACH(oc, &oh->objcs, hsh_list) {
 			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 			assert(oc->objhead == oh);
@@ -661,7 +684,7 @@ double keep)
 		for (n = 0; n < nobj; n++) {
 			oc = ocp[n];
 			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-			EXP_Rearm(oc, now, ttl, grace, keep);
+			EXP_Rearm(oc, ttl_now, ttl, grace, keep);
 			(void)HSH_DerefObjCore(wrk, &oc, 0);
 		}
 		n_tot += nobj;
@@ -928,7 +951,7 @@ HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp, int rushmax)
 	return (0);
 }
 
-int
+static int
 HSH_DerefObjHead(struct worker *wrk, struct objhead **poh)
 {
 	struct objhead *oh;
@@ -976,7 +999,7 @@ void
 HSH_Init(const struct hash_slinger *slinger)
 {
 
-	assert(DIGEST_LEN == SHA256_LEN);	/* avoid #include pollution */
+	assert(DIGEST_LEN == VSHA256_LEN);	/* avoid #include pollution */
 	hash = slinger;
 	if (hash->start != NULL)
 		hash->start();
