@@ -31,8 +31,7 @@
 
 #include "config.h"
 
-#include <stddef.h>
-#include "cache.h"
+#include "cache_varnishd.h"
 #include <stdio.h>
 
 #include "vend.h"
@@ -103,6 +102,7 @@ http_fail(const struct http *hp)
 {
 
 	VSC_C_main->losthdr++;
+	hp->ws->id[0] |= 0x20;		// cheesy tolower()
 	VSLb(hp->vsl, SLT_Error, "out of workspace (%s)", hp->ws->id);
 	WS_MarkOverflow(hp->ws);
 }
@@ -156,7 +156,7 @@ HTTP_create(void *p, uint16_t nhttp, unsigned len)
 	hp->hd = (void*)(hp + 1);
 	hp->shd = nhttp;
 	hp->hdf = (void*)(hp->hd + nhttp);
-	assert((unsigned char*)p + len == hp->hdf + nhttp);
+	assert((unsigned char*)p + len == hp->hdf + PRNDUP(nhttp));
 	return (hp);
 }
 
@@ -669,8 +669,9 @@ http_GetHdrField(const struct http *hp, const char *hdr,
 ssize_t
 http_GetContentLength(const struct http *hp)
 {
-	ssize_t cl, cll;
+	ssize_t cl;
 	const char *b;
+	unsigned n;
 
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 
@@ -680,11 +681,13 @@ http_GetContentLength(const struct http *hp)
 	if (!vct_isdigit(*b))
 		return (-2);
 	for (; vct_isdigit(*b); b++) {
-		cll = cl;
-		cl *= 10;
-		cl += *b - '0';
-		if (cll != cl / 10)
+		if (cl > (SSIZE_MAX / 10))
 			return (-2);
+		cl *= 10;
+		n = *b - '0';
+		if (cl > (SSIZE_MAX - n))
+			return (-2);
+		cl += n;
 	}
 	while (vct_islws(*b))
 		b++;
@@ -1230,6 +1233,11 @@ http_TimeHeader(struct http *to, const char *fmt, double now)
 	char *p;
 
 	CHECK_OBJ_NOTNULL(to, HTTP_MAGIC);
+	if (to->nhd >= to->shd) {
+		VSLb(to->vsl, SLT_LostHeader, "%s", fmt);
+		http_fail(to);
+		return;
+	}
 	p = WS_Alloc(to->ws, strlen(fmt) + VTIM_FORMAT_SIZE);
 	if (p == NULL) {
 		http_fail(to);
