@@ -37,7 +37,7 @@
 
 #include <stdlib.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
 #include "cache_pool.h"
 
 static pthread_t		thr_pool_herder;
@@ -51,7 +51,7 @@ static VTAILQ_HEAD(,pool)	pools = VTAILQ_HEAD_INITIALIZER(pools);
  */
 
 static void
-pool_sumstat(const struct dstat *src)
+pool_sumstat(const struct VSC_main *src)
 {
 
 	Lck_AssertHeld(&wstat_mtx);
@@ -118,10 +118,10 @@ Pool_PurgeStat(unsigned nobj)
  * Special function to summ stats
  */
 
-void __match_proto__(task_func_t)
+void v_matchproto_(task_func_t)
 pool_stat_summ(struct worker *wrk, void *priv)
 {
-	struct dstat *src;
+	struct VSC_main *src;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(wrk->pool, POOL_MAGIC);
@@ -178,7 +178,7 @@ pool_mkpool(unsigned pool_no)
  * NB: be maintained for params which require action.
  */
 
-static void * __match_proto__()
+static void * v_matchproto_()
 pool_poolherder(void *priv)
 {
 	unsigned nwq;
