@@ -32,13 +32,15 @@
 
 #include <stdlib.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
 #include "cache_filter.h"
-#include "vtim.h"
-#include "hash/hash_slinger.h"
-#include "storage/storage.h"
+#include "cache_objhead.h"
 #include "cache_transport.h"
 
+#include "vtim.h"
+#include "storage/storage.h"
+#include "hash/hash_slinger.h"
+
 /*----------------------------------------------------------------------
  * Pull the req.body in via/into a objcore
  *
@@ -223,7 +225,7 @@ VRB_Iterate(struct req *req, objiterate_f *func, void *priv)
  * just iterate it into oblivion.
  */
 
-static int __match_proto__(objiterate_f)
+static int v_matchproto_(objiterate_f)
 httpq_req_body_discard(void *priv, int flush, const void *ptr, ssize_t len)
 {
 
