@@ -34,9 +34,7 @@
 
 #include <stdlib.h>
 
-#include "cache.h"
-
-#include "vrt.h"
+#include "cache_varnishd.h"
 
 struct vrt_priv {
 	unsigned			magic;
@@ -48,7 +46,7 @@ struct vrt_priv {
 	uintptr_t			vmod_id;
 };
 
-static struct vmod_priv cli_task_priv;
+struct vrt_privs cli_task_privs[1];
 
 /*--------------------------------------------------------------------
  */
@@ -85,7 +83,7 @@ pan_privs(struct vsb *vsb, const struct vrt_privs *privs)
 /*--------------------------------------------------------------------
  */
 
-void
+static void
 VRTPRIV_init(struct vrt_privs *privs)
 {
 
@@ -94,91 +92,72 @@ VRTPRIV_init(struct vrt_privs *privs)
 }
 
 static struct vmod_priv *
-vrt_priv_dynamic(VRT_CTX, uintptr_t id, uintptr_t vmod_id)
+vrt_priv_dynamic(const struct vcl *vcl, struct ws *ws,
+     struct vrt_privs *vps, uintptr_t id, uintptr_t vmod_id)
 {
-	struct vrt_privs *vps;
 	struct vrt_priv *vp;
 
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(vps, VRT_PRIVS_MAGIC);
 	AN(vmod_id);
-	if (ctx->req) {
-		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-		CHECK_OBJ_NOTNULL(ctx->req->sp, SESS_MAGIC);
-		CAST_OBJ_NOTNULL(vps, ctx->req->sp->privs, VRT_PRIVS_MAGIC);
-	} else {
-		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-		CAST_OBJ_NOTNULL(vps, ctx->bo->privs, VRT_PRIVS_MAGIC);
-	}
 
 	VTAILQ_FOREACH(vp, &vps->privs, list) {
 		CHECK_OBJ_NOTNULL(vp, VRT_PRIV_MAGIC);
-		if (vp->vcl == ctx->vcl && vp->id == id
-		    && vp->vmod_id == vmod_id)
+		if (vp->vcl == vcl && vp->id == id &&
+		    vp->vmod_id == vmod_id)
 			return (vp->priv);
 	}
-	ALLOC_OBJ(vp, VRT_PRIV_MAGIC);
-	AN(vp);
-	vp->vcl = ctx->vcl;
+	vp = WS_Alloc(ws, sizeof *vp);
+	if (vp == NULL)
+		return (NULL);
+	INIT_OBJ(vp, VRT_PRIV_MAGIC);
+	vp->vcl = vcl;
 	vp->id = id;
 	vp->vmod_id = vmod_id;
 	VTAILQ_INSERT_TAIL(&vps->privs, vp, list);
 	return (vp->priv);
 }
 
-void
-VRTPRIV_dynamic_kill(struct vrt_privs *privs, uintptr_t id)
-{
-	struct vrt_priv *vp, *vp1;
-
-	if (privs == NULL && id == 0) {
-		ASSERT_CLI();
-		VRT_priv_fini(&cli_task_priv);
-		memset(&cli_task_priv, 0, sizeof cli_task_priv);
-		return;
-	}
-	CHECK_OBJ_NOTNULL(privs, VRT_PRIVS_MAGIC);
-	AN(id);
-
-	VTAILQ_FOREACH_SAFE(vp, &privs->privs, list, vp1) {
-		CHECK_OBJ_NOTNULL(vp, VRT_PRIV_MAGIC);
-		if (id == vp->id) {
-			VTAILQ_REMOVE(&privs->privs, vp, list);
-			VRT_priv_fini(vp->priv);
-			FREE_OBJ(vp);
-		}
-	}
-}
-
 struct vmod_priv *
 VRT_priv_task(VRT_CTX, const void *vmod_id)
 {
 	uintptr_t id;
+	struct vrt_privs *vps;
+	struct vmod_priv *vp;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	assert(ctx->req == NULL || ctx->bo == NULL);
 	if (ctx->req) {
 		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 		id = (uintptr_t)ctx->req;
+		CAST_OBJ_NOTNULL(vps, ctx->req->privs, VRT_PRIVS_MAGIC);
 	} else if (ctx->bo) {
 		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
 		id = (uintptr_t)ctx->bo;
+		CAST_OBJ_NOTNULL(vps, ctx->bo->privs, VRT_PRIVS_MAGIC);
 	} else {
 		ASSERT_CLI();
-		return (&cli_task_priv);
+		id = (uintptr_t)cli_task_privs;
+		CAST_OBJ_NOTNULL(vps, cli_task_privs, VRT_PRIVS_MAGIC);
 	}
-	return (vrt_priv_dynamic(ctx, id, (uintptr_t)vmod_id));
+
+	vp = vrt_priv_dynamic(ctx->vcl, ctx->ws, vps, id, (uintptr_t)vmod_id);
+	return (vp);
 }
 
 struct vmod_priv *
 VRT_priv_top(VRT_CTX, const void *vmod_id)
 {
 	uintptr_t id;
+	struct vrt_privs *vps;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (ctx->req) {
 		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 		CHECK_OBJ_NOTNULL(ctx->req->top, REQ_MAGIC);
 		id = (uintptr_t)&ctx->req->top->top;
-		return (vrt_priv_dynamic(ctx, id, (uintptr_t)vmod_id));
+		CAST_OBJ_NOTNULL(vps, ctx->req->top->privs, VRT_PRIVS_MAGIC);
+		return (vrt_priv_dynamic(ctx->vcl, ctx->req->top->ws,
+					 vps, id, (uintptr_t)vmod_id));
 	} else
 		WRONG("PRIV_TOP is only accessible in client VCL context");
 	NEEDLESS(return NULL);
@@ -194,3 +173,29 @@ VRT_priv_fini(const struct vmod_priv *p)
 	if (p->priv != NULL && p->free != NULL)
 		p->free(p->priv);
 }
+
+/*--------------------------------------------------------------------*/
+
+void
+VCL_TaskEnter(const struct vcl *vcl, struct vrt_privs *privs)
+{
+
+	AN(vcl);
+	AZ(privs->magic);
+	VRTPRIV_init(privs);
+}
+
+void
+VCL_TaskLeave(const struct vcl *vcl, struct vrt_privs *privs)
+{
+	struct vrt_priv *vp, *vp1;
+
+	AN(vcl);
+	CHECK_OBJ_NOTNULL(privs, VRT_PRIVS_MAGIC);
+	VTAILQ_FOREACH_SAFE(vp, &privs->privs, list, vp1) {
+		VTAILQ_REMOVE(&privs->privs, vp, list);
+		VRT_priv_fini(vp->priv);
+	}
+	INIT_OBJ(privs, 0);
+}
+
