@@ -29,7 +29,8 @@
 
 #include "config.h"
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
+#include "cache/cache_filter.h"
 
 #include <errno.h>
 #include <stdio.h>
@@ -44,7 +45,7 @@
  * Pass the request body to the backend
  */
 
-static int __match_proto__(objiterate_f)
+static int v_matchproto_(objiterate_f)
 vbf_iter_req_body(void *priv, int flush, const void *ptr, ssize_t l)
 {
 	struct busyobj *bo;
@@ -52,7 +53,7 @@ vbf_iter_req_body(void *priv, int flush, const void *ptr, ssize_t l)
 	CAST_OBJ_NOTNULL(bo, priv, BUSYOBJ_MAGIC);
 
 	if (l > 0) {
-		bo->acct.bereq_bodybytes += V1L_Write(bo->wrk, ptr, l);
+		(void)V1L_Write(bo->wrk, ptr, l);
 		if (flush && V1L_Flush(bo->wrk))
 			return (-1);
 	}
@@ -68,21 +69,22 @@ vbf_iter_req_body(void *priv, int flush, const void *ptr, ssize_t l)
  */
 
 int
-V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr,
-    int onlycached)
+V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr_hdrbytes,
+    uint64_t *ctr_bodybytes, int onlycached, char *abuf, char *pbuf)
 {
 	struct http *hp;
 	int j;
 	ssize_t i;
+	uint64_t bytes, hdrbytes;
 	struct http_conn *htc;
 	int do_chunked = 0;
-	char abuf[VTCP_ADDRBUFSIZE];
-	char pbuf[VTCP_PORTBUFSIZE];
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	CHECK_OBJ_NOTNULL(bo->htc, HTTP_CONN_MAGIC);
 	CHECK_OBJ_ORNULL(bo->req, REQ_MAGIC);
+	AN(ctr_hdrbytes);
+	AN(ctr_bodybytes);
 
 	htc = bo->htc;
 	assert(*htc->rfd > 0);
@@ -94,12 +96,11 @@ V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr,
 		do_chunked = 1;
 	}
 
-	VTCP_hisname(*htc->rfd, abuf, sizeof abuf, pbuf, sizeof pbuf);
 	VSLb(bo->vsl, SLT_BackendStart, "%s %s", abuf, pbuf);
 
 	(void)VTCP_blocking(*htc->rfd);	/* XXX: we should timeout instead */
-	V1L_Reserve(wrk, wrk->aws, htc->rfd, bo->vsl, bo->t_prev);
-	*ctr += HTTP1_Write(wrk, hp, HTTP1_Req);
+	V1L_Open(wrk, wrk->aws, htc->rfd, bo->vsl, bo->t_prev, 0);
+	hdrbytes = HTTP1_Write(wrk, hp, HTTP1_Req);
 
 	/* Deal with any message-body the request might (still) have */
 	i = 0;
@@ -121,7 +122,16 @@ V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr,
 			V1L_EndChunk(wrk);
 	}
 
-	j = V1L_FlushRelease(wrk);
+	j = V1L_Close(wrk, &bytes);
+
+	/* Bytes accounting */
+	if (bytes < hdrbytes)
+		*ctr_hdrbytes += bytes;
+	else {
+		*ctr_hdrbytes += hdrbytes;
+		*ctr_bodybytes += bytes - hdrbytes;
+	}
+
 	if (j != 0 || i < 0) {
 		VSLb(bo->vsl, SLT_FetchError, "backend write error: %d (%s)",
 		    errno, strerror(errno));
@@ -182,7 +192,8 @@ V1F_FetchRespHdr(struct busyobj *bo)
 			htc->doclose = SC_RX_OVERFLOW;
 			break;
 		default:
-			VSLb(bo->vsl, SLT_FetchError, "HTC status %d", hs);
+			VSLb(bo->vsl, SLT_FetchError, "HTC %s (%d)",
+			     HTC_Status(hs), hs);
 			htc->doclose = SC_RX_BAD;
 			break;
 		}
@@ -201,6 +212,16 @@ V1F_FetchRespHdr(struct busyobj *bo)
 	}
 
 	htc->doclose = http_DoConnection(hp);
+	RFC2616_Response_Body(bo->wrk, bo);
+
+	assert(bo->vfc->resp == bo->beresp);
+	if (bo->htc->body_status != BS_NONE &&
+	    bo->htc->body_status != BS_ERROR)
+		if (V1F_Setup_Fetch(bo->vfc, bo->htc)) {
+			VSLb(bo->vsl, SLT_FetchError, "overflow");
+			htc->doclose = SC_RX_OVERFLOW;
+			return (-1);
+		}
 
 	return (0);
 }
