@@ -33,21 +33,20 @@
 
 #include "config.h"
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
 
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "cache/cache_objhead.h"
 #include "cache/cache_transport.h"
 #include "cache_http1.h"
-#include "hash/hash_slinger.h"
 
 #include "vtcp.h"
 
 static const char H1NEWREQ[] = "HTTP1::NewReq";
 static const char H1PROC[] = "HTTP1::Proc";
-static const char H1BUSY[] = "HTTP1::Busy";
 static const char H1CLEANUP[] = "HTTP1::Cleanup";
 
 static void HTTP1_Session(struct worker *, struct req *);
@@ -58,7 +57,7 @@ http1_setstate(const struct sess *sp, const char *s)
 	uintptr_t p;
 
 	p = (uintptr_t)s;
-	AZ(SES_Set_xport_priv(sp, &p));
+	AZ(SES_Set_proto_priv(sp, &p));
 }
 
 static const char *
@@ -66,7 +65,7 @@ http1_getstate(const struct sess *sp)
 {
 	uintptr_t *p;
 
-	AZ(SES_Get_xport_priv(sp, &p));
+	AZ(SES_Get_proto_priv(sp, &p));
 	return (const char *)*p;
 }
 
@@ -74,7 +73,7 @@ http1_getstate(const struct sess *sp)
  * Call protocol for this request
  */
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 http1_req(struct worker *wrk, void *arg)
 {
 	struct req *req;
@@ -101,7 +100,7 @@ http1_req(struct worker *wrk, void *arg)
  * must obey this calling convention with a dummy reservation.
  */
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 http1_new_session(struct worker *wrk, void *arg)
 {
 	struct sess *sp;
@@ -114,13 +113,13 @@ http1_new_session(struct worker *wrk, void *arg)
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 
 	HTC_RxInit(req->htc, req->ws);
-	SES_Reserve_xport_priv(sp, &u);
+	SES_Reserve_proto_priv(sp, &u);
 	http1_setstate(sp, H1NEWREQ);
 	wrk->task.func = http1_req;
 	wrk->task.priv = req;
 }
 
-static void __match_proto__(task_func_t)
+static void v_matchproto_(task_func_t)
 http1_unwait(struct worker *wrk, void *arg)
 {
 	struct sess *sp;
@@ -138,7 +137,7 @@ http1_unwait(struct worker *wrk, void *arg)
 	wrk->task.priv = req;
 }
 
-static void __match_proto__(vtr_req_body_t)
+static void v_matchproto_(vtr_req_body_t)
 http1_req_body(struct req *req)
 {
 
@@ -169,7 +168,7 @@ http1_req_panic(struct vsb *vsb, const struct req *req)
 	VSB_printf(vsb, "state = %s\n", http1_getstate(req->sp));
 }
 
-static void __match_proto__(vtr_req_fail_f)
+static void v_matchproto_(vtr_req_fail_f)
 http1_req_fail(struct req *req, enum sess_close reason)
 {
 	assert(reason > 0);
@@ -178,35 +177,34 @@ http1_req_fail(struct req *req, enum sess_close reason)
 		SES_Close(req->sp, reason);
 }
 
-static void __match_proto__(vtr_reembark_f)
-http1_reembark(struct worker *wrk, struct req *req)
+/*----------------------------------------------------------------------
+ */
+
+static int
+http1_req_cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 {
-	struct sess *sp;
+	AZ(wrk->aws->r);
+	AZ(req->ws->r);
+	Req_Cleanup(sp, wrk, req);
 
-	sp = req->sp;
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	if (sp->fd >= 0 && req->doclose != SC_NULL)
+		SES_Close(sp, req->doclose);
 
-	http1_setstate(sp, H1BUSY);
+	if (sp->fd < 0) {
+		wrk->stats->sess_closed++;
+		AZ(req->vcl);
+		Req_Release(req);
+		SES_Delete(sp, SC_NULL, NAN);
+		return (1);
+	}
 
-	if (!SES_Reschedule_Req(req))
-		return;
-
-	/* Couldn't schedule, ditch */
-	wrk->stats->busy_wakeup--;
-	wrk->stats->busy_killed++;
-	AN (req->vcl);
-	VCL_Rel(&req->vcl);
-	Req_AcctLogCharge(wrk->stats, req);
-	Req_Release(req);
-	SES_Delete(sp, SC_OVERLOAD, NAN);
-	DSL(DBG_WAITINGLIST, req->vsl->wid, "kill from waiting list");
-	usleep(10000);
+	return (0);
 }
 
-static int __match_proto__(vtr_minimal_response_f)
+static int v_matchproto_(vtr_minimal_response_f)
 http1_minimal_response(struct req *req, uint16_t status)
 {
-	size_t wl, l;
+	ssize_t wl, l;
 	char buf[80];
 	const char *reason;
 
@@ -230,7 +228,8 @@ http1_minimal_response(struct req *req, uint16_t status)
 	if (wl > 0)
 		req->acct.resp_hdrbytes += wl;
 	if (wl != l) {
-		VTCP_Assert(1);
+		if (wl < 0)
+			VTCP_Assert(1);
 		if (!req->doclose)
 			req->doclose = SC_REM_CLOSE;
 		return (-1);
@@ -240,11 +239,11 @@ http1_minimal_response(struct req *req, uint16_t status)
 
 struct transport HTTP1_transport = {
 	.name =			"HTTP/1",
+	.proto_ident =		"HTTP",
 	.magic =		TRANSPORT_MAGIC,
 	.deliver =		V1D_Deliver,
 	.minimal_response =	http1_minimal_response,
 	.new_session =		http1_new_session,
-	.reembark =		http1_reembark,
 	.req_body =		http1_req_body,
 	.req_fail =		http1_req_fail,
 	.req_panic =		http1_req_panic,
@@ -323,30 +322,6 @@ http1_dissect(struct worker *wrk, struct req *req)
 	return (0);
 }
 
-/*----------------------------------------------------------------------
- */
-
-static int
-http1_req_cleanup(struct sess *sp, struct worker *wrk, struct req *req)
-{
-	AZ(wrk->aws->r);
-	AZ(req->ws->r);
-	Req_Cleanup(sp, wrk, req);
-
-	if (sp->fd >= 0 && req->doclose != SC_NULL)
-		SES_Close(sp, req->doclose);
-
-	if (sp->fd < 0) {
-		wrk->stats->sess_closed++;
-		AZ(req->vcl);
-		Req_Release(req);
-		SES_Delete(sp, SC_NULL, NAN);
-		return (1);
-	}
-
-	return (0);
-}
-
 /*----------------------------------------------------------------------
  */
 
@@ -469,23 +444,6 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 			}
 			req->req_step = R_STP_TRANSPORT;
 			http1_setstate(sp, H1PROC);
-		} else if (st == H1BUSY) {
-			CHECK_OBJ_NOTNULL(req->transport, TRANSPORT_MAGIC);
-			/*
-			 * Return from waitinglist.
-			 * Check to see if the remote has left.
-			 */
-			if (VTCP_check_hup(sp->fd)) {
-				AN(req->ws->r);
-				WS_Release(req->ws, 0);
-				AN(req->hash_objhead);
-				(void)HSH_DerefObjHead(wrk, &req->hash_objhead);
-				AZ(req->hash_objhead);
-				SES_Close(sp, SC_REM_CLOSE);
-				AN(http1_req_cleanup(sp, wrk, req));
-				return;
-			}
-			http1_setstate(sp, H1PROC);
 		} else if (st == H1PROC) {
 			req->task.func = http1_req;
 			req->task.priv = req;
