@@ -29,7 +29,7 @@
 
 #include "config.h"
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
 
 #include <netinet/in.h>
 
@@ -72,7 +72,7 @@ V2D_Init(void)
 
 /**********************************************************************/
 
-static int __match_proto__(vdp_bytes)
+static int v_matchproto_(vdp_bytes)
 h2_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
@@ -84,18 +84,23 @@ h2_bytes(struct req *req, enum vdp_action act, void **priv,
 
 	if (act == VDP_INIT)
 		return (0);
-	if (r2->error && act != VDP_FINI)
+	if ((r2->h2sess->error || r2->error) && act != VDP_FINI)
 		return (-1);
 	H2_Send_Get(req->wrk, r2->h2sess, r2);
 	H2_Send(req->wrk, r2,
 	    H2_F_DATA,
 	    act == VDP_FINI ? H2FF_DATA_END_STREAM : H2FF_NONE,
 	    len, ptr);
-	req->acct.resp_bodybytes += len;
 	H2_Send_Rel(r2->h2sess, r2);
+	req->acct.resp_bodybytes += len;
 	return (0);
 }
 
+static const struct vdp h2_vdp = {
+	.name =		"H2B",
+	.func =		h2_bytes,
+};
+
 static inline size_t
 h2_status(uint8_t *p, uint16_t status) {
 	size_t l = 1;
@@ -121,7 +126,7 @@ h2_status(uint8_t *p, uint16_t status) {
 	return (l);
 }
 
-int __match_proto__(vtr_minimal_response_f)
+int v_matchproto_(vtr_minimal_response_f)
 h2_minimal_response(struct req *req, uint16_t status)
 {
 	struct h2_req *r2;
@@ -155,35 +160,54 @@ h2_minimal_response(struct req *req, uint16_t status)
 	return (0);
 }
 
-static uint8_t *
-h2_enc_len(uint8_t *p, unsigned bits, unsigned val)
+static int
+h2_enc_len(struct vsb *vsb, unsigned bits, unsigned val, uint8_t b0)
 {
 	assert(bits < 8);
 	unsigned mask = (1U << bits) - 1U;
 
 	if (val >= mask) {
-		*p++ |= (uint8_t)mask;
+		VSB_putc(vsb, b0 | (uint8_t)mask);
 		val -= mask;
 		while (val >= 128) {
-			*p++ = 0x80 | ((uint8_t)val & 0x7f);
+			VSB_putc(vsb, 0x80 | ((uint8_t)val & 0x7f));
 			val >>= 7;
 		}
 	}
-	*p++ = (uint8_t)val;
-	return (p);
+	VSB_putc(vsb, (uint8_t)val);
+	return (0);
 }
 
-void __match_proto__(vtr_deliver_f)
+/*
+ * Hand-crafted-H2-HEADERS-R-Us:
+ *
+ * This is a handbuilt HEADERS frame for when we run out of workspace
+ * during delivery.
+ */
+
+static const uint8_t h2_500_resp[] = {
+	// :status 500
+	0x8e,
+
+	// content-length 0
+	0x1f, 0x0d, 0x01, 0x30,
+
+	// server Varnish
+	0x1f, 0x27, 0x07, 'V', 'a', 'r', 'n', 'i', 's', 'h',
+};
+
+void v_matchproto_(vtr_deliver_f)
 h2_deliver(struct req *req, struct boc *boc, int sendbody)
 {
 	ssize_t sz, sz1;
-	uint8_t *p;
-	unsigned u;
+	unsigned u, l;
+	uint8_t buf[6];
 	const char *r;
 	struct http *hp;
 	struct sess *sp;
 	struct h2_req *r2;
-	int i, err;
+	struct vsb resp;
+	int i;
 	const struct hpack_static *hps;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -193,15 +217,14 @@ h2_deliver(struct req *req, struct boc *boc, int sendbody)
 	sp = req->sp;
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 
-	(void)WS_Reserve(req->ws, 0);
-	p = (void*)req->ws->f;
+	l = WS_Reserve(req->ws, 0);
+	AN(VSB_new(&resp, req->ws->f, l, VSB_FIXEDLEN));
 
-	p += h2_status(p, req->resp->status);
+	l = h2_status(buf, req->resp->status);
+	VSB_bcat(&resp, buf, l);
 
 	hp = req->resp;
-	for (u = HTTP_HDR_FIRST; u < hp->nhd; u++) {
-		assert(WS_Inside(req->ws, p, NULL));
-
+	for (u = HTTP_HDR_FIRST; u < hp->nhd && !VSB_error(&resp); u++) {
 		r = strchr(hp->hd[u].b, ':');
 		AN(r);
 
@@ -222,48 +245,53 @@ h2_deliver(struct req *req, struct boc *boc, int sendbody)
 			VSLb(req->vsl, SLT_Debug,
 			    "HP {%d, \"%s\", \"%s\"} <%s>",
 			    hps->idx, hps->name, hps->val, hp->hd[u].b);
-			*p = 0x10;
-			p = h2_enc_len(p, 4, hps->idx);
+			h2_enc_len(&resp, 4, hps->idx, 0x10);
 		} else {
-
-			*p++ = 0x10;
+			VSB_putc(&resp, 0x10);
 			sz--;
-			p = h2_enc_len(p, 7, sz);
+			h2_enc_len(&resp, 7, sz, 0);
 			for (sz1 = 0; sz1 < sz; sz1++)
-				*p++ = (uint8_t)tolower(hp->hd[u].b[sz1]);
+				VSB_putc(&resp, tolower(hp->hd[u].b[sz1]));
 
 		}
 
 		while (vct_islws(*++r))
 			continue;
 		sz = hp->hd[u].e - r;
-		p = h2_enc_len(p, 7, sz);
-		memcpy(p, r, sz);
-		p += sz;
-		assert(WS_Inside(req->ws, p, NULL));
+		h2_enc_len(&resp, 7, sz, 0);
+		VSB_bcat(&resp, r, sz);
+	}
+	if (VSB_finish(&resp)) {
+		// We ran out of workspace, return minimal 500
+		// XXX: VSC counter ?
+		r = (const char*)h2_500_resp;
+		sz = sizeof h2_500_resp;
+		sendbody = 0;
+	} else {
+		sz = VSB_len(&resp);
+		r = req->ws->f;
 	}
-	sz = (char*)p - req->ws->f;
 
 	AZ(req->wrk->v1l);
 
-	/* XXX: Optimize !sendbody case */
-
 	if (sendbody && req->resp_len == 0)
 		sendbody = 0;
 
+	r2->t_send = req->t_prev;
+
 	H2_Send_Get(req->wrk, r2->h2sess, r2);
 	H2_Send(req->wrk, r2, H2_F_HEADERS,
 	    (sendbody ? 0 : H2FF_HEADERS_END_STREAM) | H2FF_HEADERS_END_HEADERS,
-	    sz, req->ws->f);
-	req->acct.resp_hdrbytes += sz;
+	    sz, r);
 	H2_Send_Rel(r2->h2sess, r2);
+	req->acct.resp_hdrbytes += sz;
 
 	WS_Release(req->ws, 0);
 
+	/* XXX someone into H2 please add appropriate error handling */
 	if (sendbody) {
-		VDP_push(req, h2_bytes, NULL, 1, "H2");
-		err = VDP_DeliverObj(req);
-		/*XXX*/(void)err;
+		if (!VDP_push(req, &h2_vdp, NULL, 1))
+			(void)VDP_DeliverObj(req);
 	}
 
 	AZ(req->wrk->v1l);
