@@ -44,9 +44,7 @@
 #include <syslog.h>
 #include <unistd.h>
 
-#include "mgt/mgt.h"
-#include "common/heritage.h"
-#include "common/common_vsm.h"
+#include "mgt.h"
 
 #include "vbm.h"
 #include "vcli_serve.h"
@@ -54,7 +52,9 @@
 #include "vfil.h"
 #include "vlu.h"
 #include "vtim.h"
-#include "vsmw.h"
+
+#include "common/heritage.h"
+#include "common/vsmw.h"
 
 static pid_t		child_pid = -1;
 
@@ -86,11 +86,6 @@ static struct vlu	*child_std_vlu;
 
 static struct vsb *child_panic = NULL;
 
-#ifdef HAVE_SIGALTSTACK
-#include <sys/mman.h>
-stack_t altstack;
-#endif
-
 static void mgt_reap_child(void);
 
 /*=====================================================================
@@ -122,7 +117,7 @@ mgt_panic_clear(void)
 	VSB_destroy(&child_panic);
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
@@ -138,7 +133,7 @@ mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
 	VCLI_Out(cli, "%s\n", VSB_data(child_panic));
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mch_cli_panic_clear(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)priv;
@@ -210,7 +205,7 @@ MCH_Fd_Inherit(int fd, const char *what)
  * Listen to stdout+stderr from the child
  */
 
-static int __match_proto__(vlu_f)
+static int v_matchproto_(vlu_f)
 child_line(void *priv, const char *p)
 {
 	(void)priv;
@@ -223,11 +218,11 @@ child_line(void *priv, const char *p)
  * NB: Notice cleanup call from mgt_reap_child()
  */
 
-static int __match_proto__(vev_cb_f)
+static int v_matchproto_(vev_cb_f)
 child_listener(const struct vev *e, int what)
 {
 
-	if ((what & ~EV_RD) || VLU_Fd(child_output, child_std_vlu)) {
+	if ((what & ~VEV__RD) || VLU_Fd(child_std_vlu, child_output)) {
 		ev_listen = NULL;
 		if (e != NULL)
 			mgt_reap_child();
@@ -240,7 +235,7 @@ child_listener(const struct vev *e, int what)
  * Periodically poke the child, to see that it still lives
  */
 
-static int __match_proto__(vev_cb_f)
+static int v_matchproto_(vev_cb_f)
 child_poker(const struct vev *e, int what)
 {
 	char *r = NULL;
@@ -262,33 +257,6 @@ child_poker(const struct vev *e, int what)
 	return 0;
 }
 
-/*=====================================================================
- * signal handler for child process
- */
-
-static void __match_proto__()
-child_signal_handler(int s, siginfo_t *si, void *c)
-{
-	char buf[1024];
-	struct sigaction sa;
-
-	(void)c;
-
-	/* Don't come back */
-	memset(&sa, 0, sizeof sa);
-	sa.sa_handler = SIG_DFL;
-	(void)sigaction(SIGSEGV, &sa, NULL);
-	(void)sigaction(SIGABRT, &sa, NULL);
-
-	bprintf(buf, "Signal %d (%s) received at %p si_code %d",
-		s, strsignal(s), si->si_addr, si->si_code);
-	VAS_Fail(__func__,
-		 __FILE__,
-		 __LINE__,
-		 buf,
-		 VAS_WRONG);
-}
-
 /*=====================================================================
  * Launch the child process
  */
@@ -301,7 +269,6 @@ mgt_launch_child(struct cli *cli)
 	char *p;
 	struct vev *e;
 	int i, cp[2];
-	struct sigaction sa;
 
 	if (child_state != CH_STOPPED && child_state != CH_DIED)
 		return;
@@ -311,6 +278,8 @@ mgt_launch_child(struct cli *cli)
 	/* Open pipe for mgt->child CLI */
 	AZ(pipe(cp));
 	heritage.cli_in = cp[0];
+	assert(cp[0] > STDERR_FILENO);	// See #2782
+	assert(cp[1] > STDERR_FILENO);
 	MCH_Fd_Inherit(heritage.cli_in, "cli_in");
 	child_cli_out = cp[1];
 
@@ -338,7 +307,6 @@ mgt_launch_child(struct cli *cli)
 	}
 	if (pid == 0) {
 
-
 		/* Redirect stdin/out/err */
 		VFIL_null_fd(STDIN_FILENO);
 		assert(dup2(heritage.std_fd, STDOUT_FILENO) == STDOUT_FILENO);
@@ -363,40 +331,31 @@ mgt_launch_child(struct cli *cli)
 			if (close(i) == 0)
 				VFIL_null_fd(i);
 		}
+
 		mgt_ProcTitle("Child");
-		if (mgt_param.sigsegv_handler) {
-			memset(&sa, 0, sizeof sa);
-			sa.sa_sigaction = child_signal_handler;
-			sa.sa_flags = SA_SIGINFO;
-			(void)sigaction(SIGBUS, &sa, NULL);
-			(void)sigaction(SIGABRT, &sa, NULL);
 
-#ifdef HAVE_SIGALTSTACK
-			size_t sz = SIGSTKSZ + 4096;
-			if (sz < mgt_param.wthread_stacksize)
-				sz = mgt_param.wthread_stacksize;
-			altstack.ss_sp = mmap(NULL, sz,  PROT_READ | PROT_WRITE,
-					      MAP_PRIVATE | MAP_ANONYMOUS,
-					      -1, 0);
-			AN(altstack.ss_sp != MAP_FAILED);
-			AN(altstack.ss_sp);
-			altstack.ss_size = sz;
-			altstack.ss_flags = 0;
-			AZ(sigaltstack(&altstack, NULL));
-			sa.sa_flags |= SA_ONSTACK;
-#endif
-			(void)sigaction(SIGSEGV, &sa, NULL);
-
-		}
-		(void)signal(SIGINT, SIG_DFL);
-		(void)signal(SIGTERM, SIG_DFL);
+		heritage.cls = mgt_cls;
+		heritage.ident = VSB_data(vident) + 1;
 
 		VJ_subproc(JAIL_SUBPROC_WORKER);
 
-		proc_vsmw = VSMW_New(heritage.vsm_fd, 0640, "_.index");
-		AN(proc_vsmw);
+		heritage.proc_vsmw = VSMW_New(heritage.vsm_fd, 0640, "_.index");
+		AN(heritage.proc_vsmw);
 
-		child_main();
+		/*
+		 * We pass these two params because child_main needs them
+		 * Well before it has found its own param struct.
+		 */
+		child_main(mgt_param.sigsegv_handler,
+		    mgt_param.wthread_stacksize);
+
+		/*
+		 * It would be natural to clean VSMW up here, but it is apt
+		 * to fail in some scenarios because of the fall-back
+		 * "rm -rf" in mgt_SHM_ChildDestroy() which is there to
+		 * catch the cases were we don't get here.
+		 */
+		// VSMW_Destroy(&heritage.proc_vsmw);
 
 		exit(0);
 	}
@@ -413,40 +372,52 @@ mgt_launch_child(struct cli *cli)
 	MCH_Fd_Inherit(heritage.cli_out, NULL);
 	closefd(&heritage.cli_out);
 
-	child_std_vlu = VLU_New(NULL, child_line, 0);
+	child_std_vlu = VLU_New(child_line, NULL, 0);
 	AN(child_std_vlu);
 
 	AZ(ev_listen);
-	e = vev_new();
+	e = VEV_Alloc();
 	XXXAN(e);
 	e->fd = child_output;
-	e->fd_flags = EV_RD;
+	e->fd_flags = VEV__RD;
 	e->name = "Child listener";
 	e->callback = child_listener;
-	AZ(vev_add(mgt_evb, e));
+	AZ(VEV_Start(mgt_evb, e));
 	ev_listen = e;
 	AZ(ev_poker);
 	if (mgt_param.ping_interval > 0) {
-		e = vev_new();
+		e = VEV_Alloc();
 		XXXAN(e);
 		e->timeout = mgt_param.ping_interval;
 		e->callback = child_poker;
 		e->name = "child poker";
-		AZ(vev_add(mgt_evb, e));
+		AZ(VEV_Start(mgt_evb, e));
 		ev_poker = e;
 	}
 
 	mgt_cli_start_child(child_cli_in, child_cli_out);
 	child_pid = pid;
-	if (mgt_push_vcls_and_start(cli, &u, &p)) {
+
+	if (mgt_push_vcls(cli, &u, &p)) {
 		VCLI_SetResult(cli, u);
 		MGT_Complain(C_ERR, "Child (%jd) Pushing vcls failed:\n%s",
 		    (intmax_t)child_pid, p);
 		free(p);
-		child_state = CH_RUNNING;
 		MCH_Stop_Child();
-	} else
-		child_state = CH_RUNNING;
+		return;
+	}
+
+	if (mgt_cli_askchild(&u, &p, "start\n")) {
+		VCLI_SetResult(cli, u);
+		MGT_Complain(C_ERR, "Child (%jd) Acceptor start failed:\n%s",
+		    (intmax_t)child_pid, p);
+		free(p);
+		MCH_Stop_Child();
+		return;
+	}
+
+	free(p);
+	child_state = CH_RUNNING;
 }
 
 /*=====================================================================
@@ -491,14 +462,14 @@ mgt_reap_child(void)
 
 	/* Stop the poker */
 	if (ev_poker != NULL) {
-		vev_del(mgt_evb, ev_poker);
+		VEV_Stop(mgt_evb, ev_poker);
 		free(ev_poker);
+		ev_poker = NULL;
 	}
-	ev_poker = NULL;
 
 	/* Stop the listener */
 	if (ev_listen != NULL) {
-		vev_del(mgt_evb, ev_listen);
+		VEV_Stop(mgt_evb, ev_listen);
 		free(ev_listen);
 		ev_listen = NULL;
 	}
@@ -522,11 +493,9 @@ mgt_reap_child(void)
 		r = waitpid(child_pid, &status, 0);
 	}
 	if (r != child_pid)
-		fprintf(stderr, "WAIT 0x%jx\n", (uintmax_t)r);
+		fprintf(stderr, "WAIT 0x%jd\n", (intmax_t)r);
 	assert(r == child_pid);
 
-	MAC_reopen_sockets(NULL);
-
 	VSB_printf(vsb, "Child (%jd) %s", (intmax_t)r,
 	    status ? "died" : "ended");
 	if (WIFEXITED(status) && WEXITSTATUS(status)) {
@@ -565,14 +534,29 @@ mgt_reap_child(void)
 		child_state = CH_DIED;
 
 	/* Pick up any stuff lingering on stdout/stderr */
-	(void)child_listener(NULL, EV_RD);
+	(void)child_listener(NULL, VEV__RD);
 	closefd(&child_output);
-	VLU_Destroy(child_std_vlu);
+	VLU_Destroy(&child_std_vlu);
 
 	child_pid = -1;
 
 	MGT_Complain(C_DEBUG, "Child cleanup complete");
 
+	/* XXX number of retries? interval? */
+	for (i = 0; i < 3; i++) {
+		if (MAC_reopen_sockets() == 0)
+			break;
+		/* error already logged */
+		(void)sleep(1);
+	}
+	if (i == 3) {
+		/* We failed to reopen our listening sockets. No choice
+		 * but to exit. */
+		MGT_Complain(C_ERR,
+		    "Could not reopen listening sockets. Exiting.");
+		exit(1);
+	}
+
 	if (child_state == CH_DIED && mgt_param.auto_restart)
 		mgt_launch_child(NULL);
 	else if (child_state == CH_DIED)
@@ -595,7 +579,7 @@ void
 MCH_Cli_Fail(void)
 {
 
-	if (child_state != CH_RUNNING)
+	if (child_state != CH_RUNNING && child_state != CH_STARTING)
 		return;
 	if (child_pid < 0)
 		return;
@@ -617,7 +601,7 @@ void
 MCH_Stop_Child(void)
 {
 
-	if (child_state != CH_RUNNING)
+	if (child_state != CH_RUNNING && child_state != CH_STARTING)
 		return;
 
 	child_state = CH_STOPPING;
@@ -654,7 +638,7 @@ MCH_Running(void)
  * CLI commands
  */
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mch_cli_server_start(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -673,7 +657,7 @@ mch_cli_server_start(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mch_cli_server_stop(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -687,7 +671,7 @@ mch_cli_server_stop(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mch_cli_server_status(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
