@@ -41,11 +41,13 @@
 #include <syslog.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 
 #include "mgt/mgt.h"
 #include "common/heritage.h"
 
 #include "hash/hash_slinger.h"
+#include "libvcc.h"
 #include "vav.h"
 #include "vcli_serve.h"
 #include "vend.h"
@@ -58,16 +60,16 @@
 #include "vsub.h"
 #include "vtim.h"
 #include "waiter/mgt_waiter.h"
+#include "vsa.h"
 
 struct heritage		heritage;
 unsigned		d_flag = 0;
-pid_t			mgt_pid;
-struct vev_base		*mgt_evb;
+struct vev_root		*mgt_evb;
 int			exit_status = 0;
 struct vsb		*vident;
 struct VSC_mgt		*VSC_C_mgt;
 static int		I_fd = -1;
-static char		Cn_arg[] = "/tmp/varnishd_C_XXXXXXX";
+static char		*Cn_arg;
 
 static struct vpf_fh *pfh1 = NULL;
 static struct vpf_fh *pfh2 = NULL;
@@ -75,6 +77,8 @@ static struct vpf_fh *pfh2 = NULL;
 static struct vfil_path *vcl_path = NULL;
 static VTAILQ_HEAD(,f_arg) f_args = VTAILQ_HEAD_INITIALIZER(f_args);
 
+static const char opt_spec[] = "a:b:Cdf:Fh:i:I:j:l:M:n:P:p:r:S:s:T:t:VW:x:";
+
 int optreset;	// Some has it, some doesn't.  Cheaper than auto*
 
 /*--------------------------------------------------------------------*/
@@ -90,9 +94,14 @@ usage(void)
 
 	printf(FMT, "-a address[:port][,proto]",
 	    "HTTP listen address and port");
-	printf(FMT, "", "Can be specified multiple times.");
-	printf(FMT, "", "  default: \":80,HTTP/1\"");
-	printf(FMT, "-b address[:port]", "Backend address and port");
+	printf(FMT, "   [,user=<u>][,group=<g>]",
+	    "Can be specified multiple times.");
+	printf(FMT, "   [,mode=<m>]", "  default: \":80,HTTP\"");
+	printf(FMT, "", "Proto can be \"PROXY\" or \"HTTP\" (default)");
+	printf(FMT, "", "user, group and mode set permissions for");
+	printf(FMT, "", "  a Unix domain socket.");
+	printf(FMT, "-b [addr[:port]|path]", "Backend address and port");
+	printf(FMT, "", "  or socket file path");
 	printf(FMT, "", "  default: \":80\"");
 	printf(FMT, "-f vclfile", "VCL program");
 	printf(FMT, "", "Can be specified multiple times.");
@@ -106,6 +115,7 @@ usage(void)
 	printf(FMT, "-x vsl", "VSL record documentation");
 	printf(FMT, "-x cli", "CLI command documentation");
 	printf(FMT, "-x builtin", "Builtin VCL program");
+	printf(FMT, "-x optstring", "List of getopt options");
 
 	printf("\nOperations options:\n");
 
@@ -126,6 +136,12 @@ usage(void)
 
 	printf(FMT, "-s [name=]kind[,options]", "Storage specification");
 	printf(FMT, "", "Can be specified multiple times.");
+#ifdef HAVE_UMEM_H
+	printf(FMT, "", "  -s default (=umem)");
+	printf(FMT, "", "  -s umem");
+#else
+	printf(FMT, "", "  -s default (=malloc)");
+#endif
 	printf(FMT, "", "  -s malloc");
 	printf(FMT, "", "  -s file");
 
@@ -172,24 +188,12 @@ cli_check(const struct cli *cli)
  * This function is called when the CLI on stdin is closed.
  */
 
-static void __match_proto__(mgt_cli_close_f)
+static int v_matchproto_(mgt_cli_close_f)
 mgt_stdin_close(void *priv)
 {
 
 	(void)priv;
-
-	if (d_flag) {
-		MCH_Stop_Child();
-		mgt_cli_close_all();
-		(void)VPF_Remove(pfh1);
-		if (pfh2 != NULL)
-			(void)VPF_Remove(pfh2);
-		exit(0);
-	} else {
-		VFIL_null_fd(STDIN_FILENO);
-		VFIL_null_fd(STDOUT_FILENO);
-		VFIL_null_fd(STDERR_FILENO);
-	}
+	return (-42);
 }
 
 /*--------------------------------------------------------------------
@@ -201,7 +205,7 @@ mgt_secret_atexit(void)
 {
 
 	/* Only master process */
-	if (getpid() != mgt_pid)
+	if (getpid() != heritage.mgt_pid)
 		return;
 	VJ_master(JAIL_MASTER_FILE);
 	(void)unlink("_.secret");
@@ -239,7 +243,7 @@ mgt_Cflag_atexit(void)
 {
 
 	/* Only master process */
-	if (getpid() != mgt_pid)
+	if (getpid() != heritage.mgt_pid)
 		return;
 	(void)rmdir("vmod_cache");
 	(void)unlink("_.pid");
@@ -248,49 +252,6 @@ mgt_Cflag_atexit(void)
 
 /*--------------------------------------------------------------------*/
 
-static void
-init_params(struct cli *cli)
-{
-	ssize_t def, low;
-
-	MCF_CollectParams();
-
-	MCF_TcpParams();
-
-	if (sizeof(void *) < 8) {		/*lint !e506 !e774  */
-		/*
-		 * Adjust default parameters for 32 bit systems to conserve
-		 * VM space.
-		 */
-		MCF_ParamConf(MCF_DEFAULT, "workspace_client", "24k");
-		MCF_ParamConf(MCF_DEFAULT, "workspace_backend", "16k");
-		MCF_ParamConf(MCF_DEFAULT, "http_resp_size", "8k");
-		MCF_ParamConf(MCF_DEFAULT, "http_req_size", "12k");
-		MCF_ParamConf(MCF_DEFAULT, "gzip_buffer", "4k");
-		MCF_ParamConf(MCF_MAXIMUM, "vsl_space", "1G");
-	}
-
-#if !defined(HAVE_ACCEPT_FILTERS) || defined(__linux)
-	MCF_ParamConf(MCF_DEFAULT, "accept_filter", "off");
-#endif
-
-	low = sysconf(_SC_THREAD_STACK_MIN);
-	MCF_ParamConf(MCF_MINIMUM, "thread_pool_stack", "%jdb", (intmax_t)low);
-
-#if defined(WITH_SANITIZERS)
-	def = 92 * 1024;
-#else
-	def = 48 * 1024;
-#endif
-	if (def < low)
-		def = low;
-	MCF_ParamConf(MCF_DEFAULT, "thread_pool_stack", "%jdb", (intmax_t)def);
-
-	MCF_ParamConf(MCF_MAXIMUM, "thread_pools", "%d", MAX_THREAD_POOLS);
-
-	MCF_InitParams(cli);
-}
-
 static void
 mgt_tests(void)
 {
@@ -298,8 +259,8 @@ mgt_tests(void)
 	assert(VTIM_parse("Sunday, 06-Nov-94 08:49:37 GMT") == 784111777);
 	assert(VTIM_parse("Sun Nov  6 08:49:37 1994") == 784111777);
 
-	/* Check that our SHA256 works */
-	SHA256_Test();
+	/* Check that our VSHA256 works */
+	VSHA256_Test();
 }
 
 static void
@@ -308,7 +269,7 @@ mgt_initialize(struct cli *cli)
 	static unsigned clilim = 32768;
 
 	/* for ASSERT_MGT() */
-	mgt_pid = getpid();
+	heritage.mgt_pid = getpid();
 
 	/* Create a cli for convenience in otherwise CLI functions */
 	INIT_OBJ(cli, CLI_MAGIC);
@@ -319,7 +280,10 @@ mgt_initialize(struct cli *cli)
 
 	mgt_cli_init_cls();		// CLI commands can be registered
 
-	init_params(cli);
+	MCF_InitParams(cli);
+
+	VCC_VCL_Range(&heritage.min_vcl, &heritage.max_vcl);
+
 	cli_check(cli);
 }
 
@@ -334,6 +298,8 @@ mgt_x_arg(const char *x_arg)
 		mgt_DumpRstCli();
 	else if (!strcmp(x_arg, "builtin"))
 		mgt_DumpBuiltin();
+	else if (!strcmp(x_arg, "optstring"))
+		(void)printf("%s\n", opt_spec);
 	else
 		ARGV_ERR("Invalid -x argument\n");
 }
@@ -391,22 +357,22 @@ mgt_eric_im_done(int eric_fd, unsigned u)
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(vev_cb_f)
+static int v_matchproto_(vev_cb_f)
 mgt_sigint(const struct vev *e, int what)
 {
 
 	(void)e;
 	(void)what;
-	MGT_Complain(C_ERR, "Manager got SIGINT");
+	MGT_Complain(C_ERR, "Manager got %s", e->name);
 	(void)fflush(stdout);
 	if (MCH_Running())
 		MCH_Stop_Child();
-	exit(0);
+	return (-42);
 }
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(vev_cb_f)
+static int v_matchproto_(vev_cb_f)
 mgt_uptime(const struct vev *e, int what)
 {
 	static double mgt_uptime_t0 = 0;
@@ -422,12 +388,13 @@ mgt_uptime(const struct vev *e, int what)
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(mgt_cli_close_f)
+static int v_matchproto_(mgt_cli_close_f)
 mgt_I_close(void *priv)
 {
 	(void)priv;
 	fprintf(stderr, "END of -I file processing\n");
-	closefd(&I_fd);
+	I_fd = -1;
+	return (0);
 }
 
 /*--------------------------------------------------------------------*/
@@ -460,8 +427,6 @@ mgt_f_read(const char *fn)
 	VTAILQ_INSERT_TAIL(&f_args, fa, list);
 }
 
-static const char opt_spec[] = "a:b:Cdf:Fh:i:I:j:l:M:n:P:p:r:S:s:T:t:VW:x:";
-
 int
 main(int argc, char * const *argv)
 {
@@ -477,7 +442,7 @@ main(int argc, char * const *argv)
 	const char *n_arg = NULL;
 	const char *P_arg = NULL;
 	const char *S_arg = NULL;
-	const char *s_arg = "malloc,100m";
+	const char *s_arg = "default,100m";
 	const char *W_arg = NULL;
 	int s_arg_given = 0;
 	int novcl = 0;
@@ -523,8 +488,7 @@ main(int argc, char * const *argv)
 	do {
 		switch (o) {
 		case '?':
-			if (optopt == '?')
-				usage();
+			usage();
 			exit(2);
 		case 'V':
 		case 'x':
@@ -588,7 +552,7 @@ main(int argc, char * const *argv)
 	if (!C_flag && !d_flag && !F_flag) {
 		eric_fd = mgt_eric();
 		MCH_TrackHighFd(eric_fd);
-		mgt_pid = getpid();
+		heritage.mgt_pid = getpid();
 	}
 
 	VRND_SeedAll();
@@ -597,7 +561,7 @@ main(int argc, char * const *argv)
 
 	/* Various initializations */
 	VTAILQ_INIT(&heritage.socks);
-	mgt_evb = vev_new_base();
+	mgt_evb = VEV_New();
 	AN(mgt_evb);
 
 	/* Initialize transport protocols */
@@ -605,6 +569,9 @@ main(int argc, char * const *argv)
 
 	VJ_Init(j_arg);
 
+	/* Initialize the bogo-IP VSA */
+	VSA_Init();
+
 	optind = 1;
 	optreset = 1;
 	while ((o = getopt(argc, argv, opt_spec)) != -1) {
@@ -624,9 +591,14 @@ main(int argc, char * const *argv)
 			REPLACE(fa->farg, "<-b argument>");
 			vsb = VSB_new_auto();
 			AN(vsb);
-			VSB_printf(vsb, "vcl 4.0;\n");
+			VSB_printf(vsb, "vcl 4.1;\n");
 			VSB_printf(vsb, "backend default {\n");
-			VSB_printf(vsb, "    .host = \"%s\";\n", optarg);
+			if (*optarg != '/')
+				VSB_printf(vsb, "    .host = \"%s\";\n",
+					   optarg);
+			else
+				VSB_printf(vsb, "    .path = \"%s\";\n",
+					   optarg);
 			VSB_printf(vsb, "}\n");
 			AZ(VSB_finish(vsb));
 			fa->src = strdup(VSB_data(vsb));
@@ -732,6 +704,17 @@ main(int argc, char * const *argv)
 
 	if (C_flag) {
 		if (n_arg == NULL) {
+			vsb = VSB_new_auto();
+			AN(vsb);
+			if (getenv("TMPDIR") != NULL)
+				VSB_printf(vsb, "%s", getenv("TMPDIR"));
+			else
+				VSB_printf(vsb, "/tmp");
+			VSB_printf(vsb, "/varnishd_C_XXXXXXX");
+			AZ(VSB_finish(vsb));
+			Cn_arg = strdup(VSB_data(vsb));
+			AN(Cn_arg);
+			VSB_destroy(&vsb);
 			AN(mkdtemp(Cn_arg));
 			AZ(chmod(Cn_arg, 0755));
 			AZ(atexit(mgt_Cflag_atexit));
@@ -767,7 +750,7 @@ main(int argc, char * const *argv)
 		ARGV_ERR("Cannot create working directory (%s): %s\n",
 		    dirname, strerror(errno));
 
-	if (VJ_make_vcldir("vmod_cache")) {
+	if (VJ_make_subdir("vmod_cache", "VMOD cache", NULL)) {
 		ARGV_ERR(
 		    "Cannot create vmod directory (%s/vmod_cache): %s\n",
 		    dirname, strerror(errno));
@@ -842,7 +825,7 @@ main(int argc, char * const *argv)
 	mgt_SHM_Init();
 
 	mgt_SHM_static_alloc(i_arg, strlen(i_arg) + 1L, "Arg", "-i");
-	VSC_C_mgt = VSC_mgt_New("");
+	VSC_C_mgt = VSC_mgt_New(NULL, NULL, "");
 
 	if (M_arg != NULL)
 		mgt_cli_master(M_arg);
@@ -858,6 +841,7 @@ main(int argc, char * const *argv)
 	assert(!VPF_Write(pfh1));
 	assert(pfh2 == NULL || !VPF_Write(pfh2));
 
+	MGT_Complain(C_DEBUG, "Version: %s", VCS_version);
 	MGT_Complain(C_DEBUG, "Platform: %s", VSB_data(vident) + 1);
 
 	if (d_flag)
@@ -877,12 +861,13 @@ main(int argc, char * const *argv)
 
 	if (I_fd >= 0) {
 		fprintf(stderr, "BEGIN of -I file processing\n");
-		mgt_cli_setup(I_fd, 2, 1, "-I file", mgt_I_close, stderr);
+		/* We must dup stderr, because VCLS closes the output fd */
+		mgt_cli_setup(I_fd, dup(2), 1, "-I file", mgt_I_close, stderr);
 		while (I_fd >= 0) {
-			o = vev_schedule_one(mgt_evb);
+			o = VEV_Once(mgt_evb);
 			if (o != 1)
 				MGT_Complain(C_ERR,
-				    "vev_schedule_one() = %d", o);
+				    "VEV_Once() = %d", o);
 		}
 	}
 
@@ -905,32 +890,36 @@ main(int argc, char * const *argv)
 	if (F_flag)
 		VFIL_null_fd(STDIN_FILENO);
 
-	e = vev_new();
+	e = VEV_Alloc();
 	AN(e);
 	e->callback = mgt_uptime;
 	e->timeout = 1.0;
 	e->name = "mgt_uptime";
-	AZ(vev_add(mgt_evb, e));
+	AZ(VEV_Start(mgt_evb, e));
 
-	e = vev_new();
+	e = VEV_Alloc();
 	AN(e);
 	e->sig = SIGTERM;
 	e->callback = mgt_sigint;
-	e->name = "mgt_sigterm";
-	AZ(vev_add(mgt_evb, e));
+	e->name = "SIGTERM";
+	AZ(VEV_Start(mgt_evb, e));
 
-	e = vev_new();
+	e = VEV_Alloc();
 	AN(e);
 	e->sig = SIGINT;
 	e->callback = mgt_sigint;
-	e->name = "mgt_sigint";
-	AZ(vev_add(mgt_evb, e));
+	e->name = "SIGINT";
+	AZ(VEV_Start(mgt_evb, e));
 
-	o = vev_schedule(mgt_evb);
-	if (o != 0)
-		MGT_Complain(C_ERR, "vev_schedule() = %d", o);
+	o = VEV_Loop(mgt_evb);
+	if (o != 0 && o != -42)
+		MGT_Complain(C_ERR, "VEV_Loop() = %d", o);
 
+	MGT_Complain(C_INFO, "manager stopping child");
+	MCH_Stop_Child();
 	MGT_Complain(C_INFO, "manager dies");
+	mgt_cli_close_all();
+	VEV_Destroy(&mgt_evb);
 	(void)VPF_Remove(pfh1);
 	if (pfh2 != NULL)
 		(void)VPF_Remove(pfh2);
