@@ -34,10 +34,8 @@
 
 #include <limits.h>
 #include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "mgt/mgt.h"
 
@@ -53,7 +51,7 @@ static int
 tweak_generic_double(struct vsb *vsb, volatile double *dest,
     const char *arg, const char *min, const char *max, const char *fmt)
 {
-	double u, minv = 0, maxv = 0;
+	volatile double u, minv = 0, maxv = 0;
 
 	if (arg != NULL) {
 		if (min != NULL) {
@@ -223,7 +221,7 @@ fmt_bytes(struct vsb *vsb, uintmax_t t)
 {
 	const char *p;
 
-	if (t & 0xff) {
+	if (t == 0 || t & 0xff) {
 		VSB_printf(vsb, "%jub", t);
 		return;
 	}
@@ -421,7 +419,7 @@ tweak_poolparam(struct vsb *vsb, const struct parspec *par, const char *arg)
 				break;
 			}
 			*pp = px;
-		} while(0);
+		} while (0);
 		VAV_Free(av);
 	}
 	return (retval);
