@@ -41,11 +41,11 @@
 #include <unistd.h>
 
 #include "mgt/mgt.h"
-#include "common/heritage.h"
-#include "common/common_vsm.h"
 
 #include "vsm_priv.h"
-#include "vsmw.h"
+
+#include "common/heritage.h"
+#include "common/vsmw.h"
 
 static struct vsmw *mgt_vsmw;
 
@@ -58,7 +58,7 @@ mgt_SHM_static_alloc(const void *ptr, ssize_t size,
 {
 	void *p;
 
-	p = VSMW_Allocf(mgt_vsmw, class, size, "%s", ident);
+	p = VSMW_Allocf(mgt_vsmw, NULL, class, size, "%s", ident);
 	AN(p);
 	memcpy(p, ptr, size);
 }
@@ -72,7 +72,7 @@ mgt_shm_atexit(void)
 {
 
 	/* Do not let VCC kill our VSM */
-	if (getpid() != mgt_pid)
+	if (getpid() != heritage.mgt_pid)
 		return;
 	VJ_master(JAIL_MASTER_FILE);
 	VSMW_Destroy(&mgt_vsmw);
@@ -101,7 +101,7 @@ mgt_SHM_Init(void)
 	mgt_vsmw = VSMW_New(fd, 0640, "_.index");
 	AN(mgt_vsmw);
 
-	proc_vsmw = mgt_vsmw;
+	heritage.proc_vsmw = mgt_vsmw;
 
 	/* Setup atexit handler */
 	AZ(atexit(mgt_shm_atexit));
@@ -122,13 +122,13 @@ mgt_SHM_ChildNew(void)
 
 	MCH_Fd_Inherit(heritage.vsm_fd, "VSMW");
 
-	heritage.param = VSMW_Allocf(mgt_vsmw, VSM_CLASS_PARAM,
+	heritage.param = VSMW_Allocf(mgt_vsmw, NULL, VSM_CLASS_PARAM,
 	    sizeof *heritage.param, "");
 	AN(heritage.param);
 	*heritage.param = mgt_param;
 
 	heritage.panic_str_len = 64 * 1024;
-	heritage.panic_str = VSMW_Allocf(mgt_vsmw, "Panic",
+	heritage.panic_str = VSMW_Allocf(mgt_vsmw, NULL, "Panic",
 	    heritage.panic_str_len, "");
 	AN(heritage.panic_str);
 }
