@@ -40,6 +40,7 @@
 #include <sys/stat.h>
 
 #include "mgt/mgt.h"
+#include "common/heritage.h"
 
 #include "libvcc.h"
 #include "vcli_serve.h"
@@ -51,6 +52,7 @@ static const char * const VCL_STATE_COLD = "cold";
 static const char * const VCL_STATE_WARM = "warm";
 static const char * const VCL_STATE_AUTO = "auto";
 static const char * const VCL_STATE_LABEL = "label";
+static int vcl_count;
 
 struct vclprog;
 struct vmodfile;
@@ -101,6 +103,8 @@ static VTAILQ_HEAD(, vmodfile)	vmodhead = VTAILQ_HEAD_INITIALIZER(vmodhead);
 static struct vclprog		*active_vcl;
 static struct vev *e_poker;
 
+static int mgt_vcl_setstate(struct cli *, struct vclprog *, const char *);
+
 /*--------------------------------------------------------------------*/
 
 static struct vclprog *
@@ -187,6 +191,8 @@ mgt_vcl_dep_add(struct vclprog *vp_from, struct vclprog *vp_to)
 	vd->to = vp_to;
 	VTAILQ_INSERT_TAIL(&vp_to->dto, vd, lto);
 	vp_to->nto++;
+	assert(vp_to->state == VCL_STATE_WARM ||	/* vcl.label ... */
+	    vp_to->state == VCL_STATE_LABEL);		/* return(vcl(...)) */
 }
 
 static void
@@ -197,6 +203,12 @@ mgt_vcl_dep_del(struct vcldep *vd)
 	VTAILQ_REMOVE(&vd->from->dfrom, vd, lfrom);
 	VTAILQ_REMOVE(&vd->to->dto, vd, lto);
 	vd->to->nto--;
+	if (vd->to->nto == 0 && vd->to->state == VCL_STATE_WARM) {
+		vd->to->state = VCL_STATE_AUTO;
+		AZ(vd->to->go_cold);
+		(void)mgt_vcl_setstate(NULL, vd->to, VCL_STATE_AUTO);
+		AN(vd->to->go_cold);
+	}
 	FREE_OBJ(vd);
 }
 
@@ -223,6 +235,8 @@ mgt_vcl_add(const char *name, const char *state)
 		vp->warm = 1;
 
 	VTAILQ_INSERT_TAIL(&vclhead, vp, list);
+	if (vp->state != VCL_STATE_LABEL)
+		vcl_count++;
 	return (vp);
 }
 
@@ -240,6 +254,8 @@ mgt_vcl_del(struct vclprog *vp)
 		mgt_vcl_dep_del(VTAILQ_FIRST(&vp->dfrom));
 
 	VTAILQ_REMOVE(&vclhead, vp, list);
+	if (vp->state != VCL_STATE_LABEL)
+		vcl_count--;
 	if (vp->fname != NULL) {
 		AZ(unlink(vp->fname));
 		p = strrchr(vp->fname, '/');
@@ -368,27 +384,46 @@ mgt_has_vcl(void)
 	return (!VTAILQ_EMPTY(&vclhead));
 }
 
+static unsigned
+mgt_vcl_cooldown(struct vclprog *vp)
+{
+	double now;
+
+	if (vp->state != VCL_STATE_AUTO)
+		return (0);
+
+	now = VTIM_mono();
+	if (vp->go_cold > 0 && vp->go_cold + mgt_param.vcl_cooldown < now)
+		return (1);
+
+	if (vp->go_cold == 0 && vp != active_vcl)
+		vp->go_cold = now;
+
+	return (0);
+}
+
 static int
 mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 {
 	unsigned status, warm;
-	double now;
 	char *p;
 	int i;
 
+	assert(vs != VCL_STATE_LABEL);
+
 	if (vp == active_vcl || mcf_is_label(vp)) {
 		AN(vp->warm);
+		/* Only the poker sends COLD indiscriminately, ignore it */
+		if (vs == VCL_STATE_COLD)
+			AZ(cli);
 		return (0);
 	}
-	if (vs == VCL_STATE_AUTO) {
-		now = VTIM_mono();
-		vs = (vp->warm ? VCL_STATE_WARM : VCL_STATE_COLD);
-		if (vp->go_cold > 0 && vp->state == VCL_STATE_AUTO &&
-		    vp->go_cold + mgt_param.vcl_cooldown < now)
-			vs = VCL_STATE_COLD;
-	}
 
-	assert(vs != VCL_STATE_AUTO);
+	if (vs == VCL_STATE_AUTO)
+		vs = (mgt_vcl_cooldown(vp) ? VCL_STATE_COLD : VCL_STATE_WARM);
+	else
+		vp->go_cold = 0;
+
 	warm = (vs == VCL_STATE_WARM ? 1 : 0);
 
 	if (vp->warm == warm)
@@ -396,13 +431,11 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 
 	if (!MCH_Running()) {
 		vp->warm = warm;
-		if (vp->warm == 0)
-			vp->go_cold = 0;
 		return (0);
 	}
 
 	i = mgt_cli_askchild(&status, &p, "vcl.state %s %d%s\n",
-	    vp->name, warm, vp->state);
+	    vp->name, warm, vs);
 	if (i && cli != NULL) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
@@ -415,8 +448,6 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 		/* Success, update mgt's VCL state to reflect child's
 		   state */
 		vp->warm = warm;
-		if (vp->warm == 0)
-			vp->go_cold = 0;
 	}
 
 	free(p);
@@ -439,6 +470,12 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	if (C_flag) {
 		bprintf(buf, ".CflagTest.%d", (int)getpid());
 		vclname = buf;
+	} else if (vcl_count >= mgt_param.max_vcl &&
+	    mgt_param.max_vcl_handling == 2) {
+		VCLI_Out(cli, "Too many (%d) VCLs already loaded\n", vcl_count);
+		VCLI_Out(cli, "(See max_vcl and max_vcl_handling parameters)");
+		VCLI_SetResult(cli, CLIS_CANT);
+		return;
 	}
 
 	if (state == NULL)
@@ -468,6 +505,14 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	if (active_vcl == NULL)
 		active_vcl = vp;
 
+	if (cli->result == CLIS_OK &&
+	    vcl_count > mgt_param.max_vcl &&
+	    mgt_param.max_vcl_handling == 1) {
+		VCLI_Out(cli, "%d VCLs loaded\n", vcl_count);
+		VCLI_Out(cli, "Remember to vcl.discard the old/unused VCLs.\n");
+		VCLI_Out(cli, "(See max_vcl and max_vcl_handling parameters)");
+	}
+
 	if (!MCH_Running())
 		return;
 
@@ -519,7 +564,7 @@ mgt_vcl_export_labels(struct vcc *vcc)
 /*--------------------------------------------------------------------*/
 
 int
-mgt_push_vcls_and_start(struct cli *cli, unsigned *status, char **p)
+mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 {
 	struct vclprog *vp;
 	struct vcldep *vd;
@@ -568,16 +613,12 @@ mgt_push_vcls_and_start(struct cli *cli, unsigned *status, char **p)
 		return (1);
 	free(*p);
 	*p = NULL;
-	if (mgt_cli_askchild(status, p, "start\n"))
-		return (1);
-	free(*p);
-	*p = NULL;
 	return (0);
 }
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_inline(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -589,7 +630,7 @@ mcf_vcl_inline(struct cli *cli, const char * const *av, void *priv)
 	mgt_new_vcl(cli, av[2], av[3], "<vcl.inline>", av[4], 0);
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_load(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -601,7 +642,7 @@ mcf_vcl_load(struct cli *cli, const char * const *av, void *priv)
 }
 
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vclprog *vp;
@@ -630,10 +671,7 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 
 	if (!strcmp(av[3], VCL_STATE_AUTO)) {
 		vp->state = VCL_STATE_AUTO;
-		if (vp != active_vcl) {
-			vp->go_cold = VTIM_mono();
-			(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
-		}
+		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
 	} else if (!strcmp(av[3], VCL_STATE_COLD)) {
 		if (vp == active_vcl) {
 			VCLI_Out(cli, "Cannot set the active VCL cold.");
@@ -651,7 +689,7 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 {
 	unsigned status;
@@ -670,21 +708,21 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 	    mgt_cli_askchild(&status, &p, "vcl.use %s\n", av[2])) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
-		vp->go_cold = VTIM_mono();
+		AZ(vp->go_cold);
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
 	} else {
 		VCLI_Out(cli, "VCL '%s' now active", av[2]);
 		vp2 = active_vcl;
 		active_vcl = vp;
 		if (vp2 != NULL) {
-			vp2->go_cold = VTIM_mono();
+			AZ(vp2->go_cold);
 			(void)mgt_vcl_setstate(cli, vp2, VCL_STATE_AUTO);
 		}
 	}
 	free(p);
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 {
 	unsigned status;
@@ -727,13 +765,14 @@ mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
 	if (MCH_Running()) {
 		/* XXX If this fails the child is crashing, figure that later */
+		assert(vp->state != VCL_STATE_WARM);
 		(void)mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]);
 		free(p);
 	}
 	mgt_vcl_del(vp);
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 {
 	unsigned status;
@@ -775,7 +814,7 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vclprog *vpl;
@@ -816,17 +855,23 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 			    vpt->name);
 			return;
 		}
+	}
+
+	if (mgt_vcl_setstate(cli, vpt, VCL_STATE_WARM))
+		return;
+	vpt->state = VCL_STATE_WARM; /* XXX: race with the poker? */
+
+	if (vpl != NULL) {
 		mgt_vcl_dep_del(VTAILQ_FIRST(&vpl->dfrom));
 		AN(VTAILQ_EMPTY(&vpl->dfrom));
 	} else {
 		vpl = mgt_vcl_add(av[2], VCL_STATE_LABEL);
 	}
+
 	AN(vpl);
-	mgt_vcl_dep_add(vpl, vpt);
 	vpl->warm = 1;
-	if (vpt->state == VCL_STATE_COLD)
-		vpt->state = VCL_STATE_AUTO;
-	(void)mgt_vcl_setstate(cli, vpt, VCL_STATE_WARM);
+	mgt_vcl_dep_add(vpl, vpt);
+
 	if (!MCH_Running())
 		return;
 
@@ -840,7 +885,7 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(vev_cb_f)
+static int v_matchproto_(vev_cb_f)
 mgt_vcl_poker(const struct vev *e, int what)
 {
 	struct vclprog *vp;
@@ -849,7 +894,8 @@ mgt_vcl_poker(const struct vev *e, int what)
 	(void)what;
 	e_poker->timeout = mgt_param.vcl_cooldown * .45;
 	VTAILQ_FOREACH(vp, &vclhead, list)
-		(void)mgt_vcl_setstate(NULL, vp, VCL_STATE_AUTO);
+		if (mgt_vcl_cooldown(vp))
+			(void)mgt_vcl_setstate(NULL, vp, VCL_STATE_COLD);
 	return (0);
 }
 
@@ -873,7 +919,7 @@ mgt_vcl_atexit(void)
 {
 	struct vclprog *vp;
 
-	if (getpid() != mgt_pid)
+	if (getpid() != heritage.mgt_pid)
 		return;
 	active_vcl = NULL;
 	do {
@@ -887,13 +933,13 @@ void
 mgt_vcl_init(void)
 {
 
-	e_poker = vev_new();
+	e_poker = VEV_Alloc();
 	AN(e_poker);
 	e_poker->timeout = 3;		// random, prime
 
 	e_poker->callback = mgt_vcl_poker;
 	e_poker->name = "vcl poker";
-	AZ(vev_add(mgt_evb, e_poker));
+	AZ(VEV_Start(mgt_evb, e_poker));
 
 	AZ(atexit(mgt_vcl_atexit));
 
