@@ -39,6 +39,7 @@
 #include <unistd.h>
 
 #include "mgt/mgt.h"
+#include "common/heritage.h"
 #include "vcli_serve.h"
 
 #include "storage/storage.h"
@@ -46,6 +47,9 @@
 static VTAILQ_HEAD(, stevedore) stevedores =
     VTAILQ_HEAD_INITIALIZER(stevedores);
 
+/* Name of transient storage */
+#define TRANSIENT_STORAGE	"Transient"
+
 struct stevedore *stv_transient;
 
 /*--------------------------------------------------------------------*/
@@ -65,7 +69,7 @@ STV__iter(struct stevedore ** const pp)
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 stv_cli_list(struct cli *cli, const char * const *av, void *priv)
 {
 	struct stevedore *stv;
@@ -88,7 +92,8 @@ static struct cli_proto cli_stv[] = {
 /*--------------------------------------------------------------------
  */
 
-static void __match_proto__(storage_init_f)
+#ifdef WITH_PERSISTENT_STORAGE
+static void v_matchproto_(storage_init_f)
 smp_fake_init(struct stevedore *parent, int ac, char * const *av)
 {
 
@@ -107,6 +112,7 @@ static const struct stevedore smp_fake_stevedore = {
 	.name = "deprecated_persistent",
 	.init = smp_fake_init,
 };
+#endif
 
 /*--------------------------------------------------------------------
  * Parse a stevedore argument on the form:
@@ -116,8 +122,16 @@ static const struct stevedore smp_fake_stevedore = {
 static const struct choice STV_choice[] = {
 	{ "file",			&smf_stevedore },
 	{ "malloc",			&sma_stevedore },
+#ifdef WITH_PERSISTENT_STORAGE
 	{ "deprecated_persistent",	&smp_stevedore },
 	{ "persistent",			&smp_fake_stevedore },
+#endif
+#if defined(HAVE_UMEM_H)
+	{ "umem",			&smu_stevedore },
+	{ "default",			&smu_stevedore },
+#else
+	{ "default",			&sma_stevedore },
+#endif
 	{ NULL,		NULL }
 };
 
@@ -145,13 +159,13 @@ void
 STV_Config(const char *spec)
 {
 	char **av, buf[8];
-	const char *name;
+	const char *ident;
 	struct stevedore *stv;
 	const struct stevedore *stv2;
 	int ac;
 	static unsigned seq = 0;
 
-	av = MGT_NamedArg(spec, &name, "-s");
+	av = MGT_NamedArg(spec, &ident, "-s");
 	AN(av);
 
 	if (av[1] == NULL)
@@ -175,12 +189,12 @@ STV_Config(const char *spec)
 	*stv = *stv2;
 	AN(stv->name);
 
-	if (name == NULL) {
+	if (ident) {
+		stv->ident = ident;
+	} else {
 		bprintf(buf, "s%u", seq++);
-		name = buf;
+		stv->ident = strdup(buf);
 	}
-
-	stv->ident = strdup(name);
 	AN(stv->ident);
 	stv_check_ident(spec, stv->ident);
 
@@ -210,7 +224,7 @@ STV_Config_Transient(void)
 
 	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_stv);
 	if (stv_transient == NULL)
-		STV_Config(TRANSIENT_STORAGE "=malloc");
+		STV_Config(TRANSIENT_STORAGE "=default");
 	AN(stv_transient);
 	VTAILQ_INSERT_TAIL(&stevedores, stv_transient, list);
 }
