@@ -63,12 +63,12 @@ static struct VUT *vut;
 static int hist_low;
 static int hist_high;
 static int hist_range;
-static int hist_buckets;
+static unsigned hist_buckets;
 
 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 
 static int end_of_file = 0;
-static double delay = 1;
+static int ms_delay = 1000;
 static unsigned rr_hist[HIST_N];
 static unsigned nhist;
 static unsigned next_hist;
@@ -82,7 +82,7 @@ static pthread_cond_t timebend_cv;
 static double log_ten;
 static char *ident;
 
-static const int scales[] = {
+static const unsigned scales[] = {
 	1,
 	2,
 	3,
@@ -103,7 +103,7 @@ static const int scales[] = {
 	25000,
 	50000,
 	100000,
-	INT_MAX
+	UINT_MAX
 };
 
 struct profile {
@@ -143,7 +143,7 @@ update(void)
 	unsigned bm[n], bh[n];
 	unsigned max;
 	unsigned i, j, scale;
-	int k, l;
+	unsigned k, l;
 
 	erase();
 
@@ -161,10 +161,11 @@ update(void)
 		mvprintw(0, 0, "%*s", COLS - 1, ident);
 
 	/* count our flock */
-	for (i = 0; i < n; ++i)
-		bm[i] = bh[i] = 0;
+	memset(bm, 0, sizeof bm);
+	memset(bh, 0, sizeof bh);
 	for (k = 0, max = 1; k < hist_buckets; ++k) {
 		l = k * n / hist_buckets;
+		assert(l < n);
 		bm[l] += bucket_miss[k];
 		bh[l] += bucket_hit[k];
 		if (bm[l] + bh[l] > max)
@@ -181,20 +182,20 @@ update(void)
 		VTIM_format(vsl_ts, t);
 
 		mvprintw(0, 0, "1:%u, n = %u, d = %g @ %s x %g",
-		    scale, nhist, delay, t, timebend);
+		    scale, nhist, 1e-3 * ms_delay, t, timebend);
 	} else
 		mvprintw(0, 0, "1:%u, n = %u, d = %g",
-		    scale, nhist, delay);
+		    scale, nhist, 1e-3 * ms_delay);
 
-	for (j = 2; j < LINES - 3; j += 5)
-		mvprintw(j, 0, "%u_", (LINES - 3 - j) * scale);
+	for (j = 5; j < LINES - 2; j += 5)
+		mvprintw((LINES - 2) - j, 0, "%u_", j * scale);
 
 	/* show them */
 	for (i = 0; i < n; ++i) {
 		for (j = 0; j < bm[i] / scale; ++j)
-			(void)mvaddch(LINES - 3 - j, i, '#');
+			(void)mvaddch((LINES - 3) - j, i, '#');
 		for (; j < (bm[i] + bh[i]) / scale; ++j)
-			(void)mvaddch(LINES - 3 - j, i, '|');
+			(void)mvaddch((LINES - 3) - j, i, '|');
 	}
 
 	refresh();
@@ -219,7 +220,7 @@ upd_vsl_ts(const char *p)
 		vsl_ts = t;
 }
 
-static int __match_proto__ (VSLQ_dispatch_f)
+static int v_matchproto_ (VSLQ_dispatch_f)
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *priv)
 {
@@ -258,6 +259,9 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			/* get the value we want and register if it's a hit */
 			tag = VSL_TAG(tr->c->rec.ptr);
 
+			if (VSL_tagflags[tag])
+				continue;
+
 			switch (tag) {
 			case SLT_Hit:
 				hit = 1;
@@ -291,11 +295,11 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			}
 		}
 
-		if (skip || !match)
+		if (skip || !match || value <= 0)
 			continue;
 
 		/* select bucket */
-		i = HIST_RES * (log(value) / log_ten);
+		i = lround(HIST_RES * log(value) / log_ten);
 		if (i < hist_low * HIST_RES)
 			i = hist_low * HIST_RES;
 		if (i >= hist_high * HIST_RES)
@@ -368,7 +372,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	return (0);
 }
 
-static int __match_proto__(VUT_cb_f)
+static int v_matchproto_(VUT_cb_f)
 sighup(struct VUT *v)
 {
 	assert(v == vut);
@@ -376,7 +380,7 @@ sighup(struct VUT *v)
 	return (1);
 }
 
-static void * __match_proto__(pthread_t)
+static void * v_matchproto_(pthread_t)
 do_curses(void *arg)
 {
 	int ch;
@@ -394,7 +398,8 @@ do_curses(void *arg)
 		update();
 		AZ(pthread_mutex_unlock(&mtx));
 
-		timeout(delay * 1000);
+		assert(ms_delay > 0);
+		timeout(ms_delay);
 		switch ((ch = getch())) {
 		case ERR:
 			break;
@@ -410,15 +415,15 @@ do_curses(void *arg)
 			break;
 		case '\032':	/* Ctrl-Z */
 			endwin();
-			raise(SIGTSTP);
+			AZ(raise(SIGTSTP));
 			break;
 		case '\003':	/* Ctrl-C */
 		case '\021':	/* Ctrl-Q */
 		case 'Q':
 		case 'q':
-			raise(SIGINT);
-			endwin();
-			return (NULL);
+			AZ(raise(SIGINT));
+			quit = 1;
+			break;
 		case '0':
 		case '1':
 		case '2':
@@ -429,15 +434,15 @@ do_curses(void *arg)
 		case '7':
 		case '8':
 		case '9':
-			delay = 1 << (ch - '0');
+			ms_delay = 1000U << (ch - '0');
 			break;
 		case '+':
-			delay /= 2;
-			if (delay < 1e-3)
-				delay = 1e-3;
+			ms_delay /= 2;
+			if (ms_delay < 1)
+				ms_delay = 1;
 			break;
 		case '-':
-			delay *= 2;
+			ms_delay *= 2;
 			break;
 		case '>':
 		case '<':
@@ -468,7 +473,7 @@ do_curses(void *arg)
 
 /*--------------------------------------------------------------------*/
 
-static void __attribute__((__noreturn__))
+static void v_noreturn_
 usage(int status)
 {
 	const char **opt;
@@ -480,7 +485,7 @@ usage(int status)
 	exit(status);
 }
 
-static void
+static void v_noreturn_
 profile_error(const char *s)
 {
 	fprintf(stderr, "-P: '%s' is not a valid"
@@ -491,19 +496,20 @@ profile_error(const char *s)
 static void
 vut_sighandler(int sig)
 {
-	VUT_Signaled(vut, sig);
+
+	if (vut != NULL)
+		VUT_Signaled(vut, sig);
 }
 
 int
 main(int argc, char **argv)
 {
 	int i;
-	const char *colon, *ptag;
-	const char *profile = "responsetime";
+	char *colon;
+	const char *ptag, *profile = "responsetime";
 	pthread_t thr;
-	int fnum = -1;
+	int fnum;
 	struct profile cli_p = {0};
-	cli_p.name = 0;
 
 	vut = VUT_InitProg(argc, argv, &vopt_spec);
 	AN(vut);
@@ -515,8 +521,8 @@ main(int argc, char **argv)
 			/* Usage help */
 			usage(0);
 		case 'p':
-			delay = strtod(optarg, NULL);
-			if (delay <= 0)
+			ms_delay = lround(1e3 * strtod(optarg, NULL));
+			if (ms_delay <= 0)
 				VUT_Error(vut, 1, "-p: invalid '%s'", optarg);
 			break;
 		case 'P':
@@ -526,7 +532,7 @@ main(int argc, char **argv)
 				profile = optarg;
 				break;
 			}
-			/* else it's a definition, we hope */
+			/* else check if valid definition */
 			if (colon == optarg + 1 &&
 			    (*optarg == 'b' || *optarg == 'c')) {
 				cli_p.VSL_arg = *optarg;
@@ -540,20 +546,46 @@ main(int argc, char **argv)
 			}
 
 			assert(colon);
-			if (sscanf(colon + 1, "%d:%d:%d", &cli_p.field,
-			    &cli_p.hist_low, &cli_p.hist_high) != 3)
-				profile_error(optarg);
 
 			match_tag = VSL_Name2Tag(ptag, colon - ptag);
 			if (match_tag < 0)
 				VUT_Error(vut, 1,
 				    "-P: '%s' is not a valid tag name",
 				    optarg);
+
+			if (VSL_tagflags[match_tag])
+				VUT_Error(vut, 1,
+				    "-P: '%s' is an unsafe or binary record",
+				    optarg);
+
+			cli_p.prefix = colon + 1;
+
+			colon = strchr(colon + 1, ':');
+			if (colon == NULL)
+				profile_error(optarg);
+
+			*colon = '\0';
+			if (*cli_p.prefix == '\0')
+				cli_p.prefix = NULL;
+
+			if (sscanf(colon + 1, "%d",  &cli_p.field) != 1)
+				profile_error(optarg);
+
 			cli_p.name = "custom";
-			cli_p.tag = match_tag;
+			cli_p.tag = (enum VSL_tag_e)match_tag;
+			cli_p.hist_low = -6;
+			cli_p.hist_high = 3;
 			profile = NULL;
 			active_profile = &cli_p;
 
+			colon = strchr(colon + 1, ':');
+			if (colon == NULL)
+				break;
+
+			if (sscanf(colon + 1, "%d:%d",  &cli_p.hist_low,
+				   &cli_p.hist_high) != 2)
+				profile_error(optarg);
+
 			break;
 		case 'B':
 			timebend = strtod(optarg, NULL);
@@ -621,9 +653,11 @@ main(int argc, char **argv)
 
 	VUT_Signal(vut_sighandler);
 	VUT_Setup(vut);
-	ident = VSM_Dup(vut->vsm, "Arg", "-i");
-	if (pthread_create(&thr, NULL, do_curses, NULL) != 0)
-		VUT_Error(vut, 1, "pthread_create(): %s", strerror(errno));
+	if (vut->vsm)
+		ident = VSM_Dup(vut->vsm, "Arg", "-i");
+	else
+		ident = strdup("");
+	AZ(pthread_create(&thr, NULL, do_curses, NULL));
 	vut->dispatch_f = accumulate;
 	vut->dispatch_priv = NULL;
 	vut->sighup_f = sighup;
