@@ -59,7 +59,7 @@ VSL_OPT_b
 VSL_OPT_c
 VSL_OPT_C
 VUT_OPT_d
-VUT_OPT_D
+VUT_GLOBAL_OPT_D
 VUT_OPT_g
 VUT_OPT_h
 VSL_OPT_i
@@ -67,14 +67,13 @@ VSL_OPT_I
 VUT_OPT_k
 VSL_OPT_L
 VUT_OPT_n
-VUT_OPT_N
-VUT_OPT_P
+VUT_GLOBAL_OPT_P
 VUT_OPT_q
 VUT_OPT_r
 VUT_OPT_t
 VSL_OPT_T
 VSL_OPT_v
-VUT_OPT_V
+VUT_GLOBAL_OPT_V
 LOG_OPT_w
 VSL_OPT_x
 VSL_OPT_X
