@@ -37,7 +37,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
-#include <poll.h>
 #include <stdint.h>
 #include <math.h>
 
@@ -119,7 +118,7 @@ static int sample = 0;
 static int scale = 1;
 static double t_sample = 0.;
 static double interval = 1.;
-static int vsm_status = 0;
+static unsigned vsm_status = 0;
 
 static void
 init_hitrate(void)
@@ -258,7 +257,7 @@ sample_points(void)
 			update_ma(&pt->ma_1000, (int64_t)pt->cur);
 		} else if (pt->vpt->semantics == 'c') {
 			if (main_uptime != NULL && *main_uptime)
-				pt->avg = pt->cur / *main_uptime;
+				pt->avg = pt->cur / (double)*main_uptime;
 			else
 				pt->avg = 0.;
 			if (pt->t_last) {
@@ -396,16 +395,16 @@ make_windows(void)
 }
 
 static void
-print_duration(WINDOW *w, time_t t)
+print_duration(WINDOW *w, uint64_t t)
 {
 
-	wprintw(w, "%4jd+%02jd:%02jd:%02jd",
-	    (intmax_t)t / 86400, (intmax_t)(t % 86400) / 3600,
-	    (intmax_t)(t % 3600) / 60, (intmax_t)t % 60);
+	wprintw(w, "%4ju+%02ju:%02ju:%02ju",
+	    (uintmax_t)t / 86400, (uintmax_t)(t % 86400) / 3600,
+	    (uintmax_t)(t % 3600) / 60, (uintmax_t)t % 60);
 }
 
 static void
-running(WINDOW *w, time_t up, int flg)
+running(WINDOW *w, uint64_t up, int flg)
 {
 	if (vsm_status & flg) {
 		print_duration(w_status, up);
@@ -419,8 +418,8 @@ running(WINDOW *w, time_t up, int flg)
 static void
 draw_status(void)
 {
-	time_t up_mgt = 0;
-	time_t up_chld = 0;
+	uint64_t up_mgt = 0;
+	uint64_t up_chld = 0;
 
 	AN(w_status);
 
@@ -472,7 +471,7 @@ draw_bar_t(void)
 	x += 4;
 	mvwprintw(w_bar_t, 0, x, "%.*s", colw_name - 4, "NAME");
 	x += colw_name - 4;
-	col = 0;
+	col = COL_CUR;
 	while (col < COL_LAST) {
 		if (X - x < COLW)
 			break;
@@ -521,7 +520,7 @@ draw_line_default(WINDOW *w, int y, int x, int X, const struct pt *pt)
 	AN(w);
 	AN(pt);
 
-	col = 0;
+	col = COL_CUR;
 	while (col < COL_LAST) {
 		if (X - x < COLW)
 			break;
@@ -609,7 +608,7 @@ draw_line_bytes(WINDOW *w, int y, int x, int X, const struct pt *pt)
 	AN(w);
 	AN(pt);
 
-	col = 0;
+	col = COL_CUR;
 	while (col < COL_LAST) {
 		if (X - x < COLW)
 			break;
@@ -653,7 +652,7 @@ draw_line_bytes(WINDOW *w, int y, int x, int X, const struct pt *pt)
 static void
 draw_line_bitmap(WINDOW *w, int y, int x, int X, const struct pt *pt)
 {
-	int ch;
+	unsigned ch;
 	enum {
 		COL_VAL,
 		COL_MAP,
@@ -664,7 +663,7 @@ draw_line_bitmap(WINDOW *w, int y, int x, int X, const struct pt *pt)
 	AN(pt);
 	assert(pt->vpt->format == 'b');
 
-	col = 0;
+	col = COL_VAL;
 	while (col < COL_LAST) {
 		switch (col) {
 		case COL_VAL:
@@ -704,7 +703,7 @@ draw_line_duration(WINDOW *w, int y, int x, int X, const struct pt *pt)
 	AN(w);
 	AN(pt);
 
-	col = 0;
+	col = COL_DUR;
 	while (col < COL_LAST) {
 		if (X - x < COLW)
 			break;
@@ -928,14 +927,15 @@ handle_keypress(int ch)
 	redraw = 1;
 }
 
-static void * __match_proto__(VSC_new_f)
+static void * v_matchproto_(VSC_new_f)
 newpt(void *priv, const struct VSC_point *const vpt)
 {
 	struct pt *pt;
 
-	ALLOC_OBJ(pt, PT_MAGIC);
-	AN(pt);
 	AZ(priv);
+	ALLOC_OBJ(pt, PT_MAGIC);
+	rebuild |= 1;
+	AN(pt);
 	pt->vpt = vpt;
 	pt->last = *pt->vpt->ptr;
 	pt->ma_10.nmax = 10;
@@ -958,13 +958,14 @@ newpt(void *priv, const struct VSC_point *const vpt)
 	return (pt);
 }
 
-static void __match_proto__(VSC_destroy_f)
+static void v_matchproto_(VSC_destroy_f)
 delpt(void *priv, const struct VSC_point *const vpt)
 {
 	struct pt *pt;
 
 	AZ(priv);
 	CAST_OBJ_NOTNULL(pt, vpt->priv, PT_MAGIC);
+	rebuild |= 2;
 	VTAILQ_REMOVE(&ptlist, pt, list);
 	n_ptlist--;
 	FREE_OBJ(pt);
@@ -981,7 +982,6 @@ delpt(void *priv, const struct VSC_point *const vpt)
 void
 do_curses(struct vsm *vsm, struct vsc *vsc, double delay)
 {
-	struct pollfd pollfd;
 	long t;
 	int ch;
 	double now;
@@ -996,24 +996,22 @@ do_curses(struct vsm *vsm, struct vsc *vsc, double delay)
 	nonl();
 	curs_set(0);
 
-	pollfd.fd = STDIN_FILENO;
-	pollfd.events = POLLIN;
-
 	make_windows();
 	doupdate();
 
 	VSC_State(vsc, newpt, delpt, NULL);
 
-	rebuild = 1;
+	(void)VSC_Iter(vsc, vsm, NULL, NULL);
+	build_pt_array();
+	init_hitrate();
+
 	while (keep_running) {
+		(void)VSC_Iter(vsc, vsm, NULL, NULL);
 		vsm_status = VSM_Status(vsm);
-		rebuild |= vsm_status & ~(VSM_MGT_RUNNING|VSM_WRK_RUNNING);
-		if (rebuild) {
-			(void)VSC_Iter(vsc, vsm, NULL, NULL);
+		if (vsm_status & (VSM_MGT_RESTARTED|VSM_WRK_RESTARTED))
 			init_hitrate();
+		if (rebuild)
 			build_pt_array();
-			redraw = 1;
-		}
 
 		now = VTIM_mono();
 		if (now - t_sample > interval)
@@ -1023,11 +1021,11 @@ do_curses(struct vsm *vsm, struct vsc *vsc, double delay)
 		if (redraw)
 			draw_screen();
 
-		t = (t_sample + interval - now) * 1000;
-		if (t > 0)
-			(void)poll(&pollfd, 1, t);
+		t = (long)((t_sample + interval - now) * 1000);
+		wtimeout(w_status, t);
 
-		switch (ch = wgetch(w_status)) {
+		ch = wgetch(w_status);
+		switch (ch) {
 		case ERR:
 			break;
 #ifdef KEY_RESIZE /* sigh, Solaris lacks this.. */
