@@ -153,7 +153,7 @@ struct logexp {
 	int				tag_last;
 
 	int				d_arg;
-	int				g_arg;
+	enum VSL_grouping_e		g_arg;
 	char				*query;
 
 	struct vsm			*vsm;
@@ -171,7 +171,8 @@ logexp_delete_tests(struct logexp *le)
 	struct logexp_test *test;
 
 	CHECK_OBJ_NOTNULL(le, LOGEXP_MAGIC);
-	while ((test = VTAILQ_FIRST(&le->tests))) {
+	while (!VTAILQ_EMPTY(&le->tests)) {
+		test = VTAILQ_FIRST(&le->tests);
 		CHECK_OBJ_NOTNULL(test, LOGEXP_TEST_MAGIC);
 		VTAILQ_REMOVE(&le->tests, test, list);
 		VSB_destroy(&test->str);
@@ -191,8 +192,10 @@ logexp_delete(struct logexp *le)
 	AZ(le->vslq);
 	logexp_delete_tests(le);
 	free(le->name);
+	free(le->vname);
 	free(le->query);
 	VSM_Destroy(&le->vsm);
+	vtc_logclose(le->vl);
 	FREE_OBJ(le);
 }
 
@@ -200,7 +203,7 @@ static struct logexp *
 logexp_new(const char *name, const char *varg)
 {
 	struct logexp *le;
-	struct vsb *vsb, *n_arg;
+	struct vsb *n_arg;
 
 	ALLOC_OBJ(le, LOGEXP_MAGIC);
 	AN(le);
@@ -219,12 +222,7 @@ logexp_new(const char *name, const char *varg)
 
 	REPLACE(le->vname, varg);
 
-	vsb = VSB_new_auto();
-	AN(vsb);
-	AZ(VSB_printf(vsb, "${tmpdir}/%s", varg));
-	AZ(VSB_finish(vsb));
-	n_arg = macro_expand(le->vl, VSB_data(vsb));
-	VSB_destroy(&vsb);
+	n_arg = macro_expandf(le->vl, "${tmpdir}/%s", varg);
 	if (n_arg == NULL)
 		vtc_fatal(le->vl, "-v argument problems");
 	if (VSM_Arg(le->vsm, 'n', VSB_data(n_arg)) <= 0)
@@ -252,7 +250,7 @@ logexp_next(struct logexp *le)
 		vtc_log(le->vl, 3, "expecting| %s", VSB_data(le->test->str));
 }
 
-static int __match_proto__(VSLQ_dispatch_f)
+static int v_matchproto_(VSLQ_dispatch_f)
 logexp_dispatch(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *priv)
 {
@@ -265,7 +263,7 @@ logexp_dispatch(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 
 	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
 
-	for (i = 0; (t = pt[i]); i++) {
+	for (i = 0; (t = pt[i]) != NULL; i++) {
 		while (1 == VSL_Next(t->c)) {
 			if (!VSL_Match(vsl, t->c))
 				continue;
@@ -513,6 +511,7 @@ void
 cmd_logexpect(CMD_ARGS)
 {
 	struct logexp *le, *le2;
+	int i;
 
 	(void)priv;
 	(void)cmd;
@@ -582,10 +581,11 @@ cmd_logexpect(CMD_ARGS)
 		if (!strcmp(*av, "-g")) {
 			if (av[1] == NULL)
 				vtc_fatal(le->vl, "Missing -g argument");
-			le->g_arg = VSLQ_Name2Grouping(av[1], strlen(av[1]));
-			if (le->g_arg < 0)
+			i = VSLQ_Name2Grouping(av[1], strlen(av[1]));
+			if (i < 0)
 				vtc_fatal(le->vl, "Unknown grouping '%s'",
 				    av[1]);
+			le->g_arg = (enum VSL_grouping_e)i;
 			av++;
 			continue;
 		}
