@@ -42,7 +42,6 @@
 
 #include "vtc.h"
 
-#include "vdef.h"
 #include "vev.h"
 #include "vfil.h"
 #include "vnum.h"
@@ -82,7 +81,7 @@ unsigned vtc_maxdur = 60;
 static unsigned vtc_bufsiz = 1024 * 1024;
 
 static VTAILQ_HEAD(, vtc_tst) tst_head = VTAILQ_HEAD_INITIALIZER(tst_head);
-static struct vev_base *vb;
+static struct vev_root *vb;
 static int njob = 0;
 static int npar = 1;			/* Number of parallel tests */
 static int vtc_continue;		/* Continue on error */
@@ -96,7 +95,6 @@ char *vmod_path = NULL;
 struct vsb *params_vsb = NULL;
 int leave_temp;
 int vtc_witness = 0;
-int feature_dns;
 
 /**********************************************************************
  * Parse a -D option argument into a name/val pair, and insert
@@ -166,9 +164,9 @@ tst_cb(const struct vev *ve, int what)
 	// printf("CB %p %s %d\n", ve, jp->tst->filename, what);
 	if (what == 0) {
 		jp->killed = 1;
-		AZ(kill(jp->child, SIGKILL)); /* XXX: Timeout */
+		AZ(kill(-jp->child, SIGKILL)); /* XXX: Timeout */
 	} else {
-		assert(what & (EV_RD | EV_HUP));
+		assert(what & (VEV__RD | VEV__HUP));
 	}
 
 	*buf = '\0';
@@ -241,9 +239,10 @@ tst_cb(const struct vev *ve, int what)
 			    jp->tst->filename,
 			    ecode ? "skipped" : "passed", t);
 		}
-		if (jp->evt != NULL)
-			vev_del(vb, jp->evt);
-
+		if (jp->evt != NULL) {
+			VEV_Stop(vb, jp->evt);
+			free(jp->evt);
+		}
 		FREE_OBJ(jp);
 		return (1);
 	}
@@ -299,6 +298,7 @@ start_test(void)
 	jp->child = fork();
 	assert(jp->child >= 0);
 	if (jp->child == 0) {
+		AZ(setpgid(getpid(), 0));
 		VFIL_null_fd(STDIN_FILENO);
 		assert(dup2(p[1], STDOUT_FILENO) == STDOUT_FILENO);
 		assert(dup2(p[1], STDERR_FILENO) == STDERR_FILENO);
@@ -309,21 +309,21 @@ start_test(void)
 	}
 	closefd(&p[1]);
 
-	jp->ev = vev_new();
+	jp->ev = VEV_Alloc();
 	AN(jp->ev);
-	jp->ev->fd_flags = EV_RD | EV_HUP | EV_ERR;
+	jp->ev->fd_flags = VEV__RD | VEV__HUP | VEV__ERR;
 	jp->ev->fd = p[0];
 	jp->ev->priv = jp;
 	jp->ev->callback = tst_cb;
-	AZ(vev_add(vb, jp->ev));
+	AZ(VEV_Start(vb, jp->ev));
 
-	jp->evt = vev_new();
+	jp->evt = VEV_Alloc();
 	AN(jp->evt);
 	jp->evt->fd = -1;
 	jp->evt->timeout = vtc_maxdur;
 	jp->evt->priv = jp;
 	jp->evt->callback = tst_cb;
-	AZ(vev_add(vb, jp->evt));
+	AZ(VEV_Start(vb, jp->evt));
 }
 
 /**********************************************************************
@@ -427,44 +427,11 @@ i_mode(void)
 	AZ(putenv(strdup("MALLOC_CONF=abort:true,junk:true")));
 }
 
-/**********************************************************************
- * Most test-cases use only numeric IP#'s but a few requires non-demented
- * DNS services.  This is a basic sanity check for those.
- */
-
-static int __match_proto__(vss_resolved_f)
-dns_cb(void *priv, const struct suckaddr *sa)
-{
-	char abuf[VTCP_ADDRBUFSIZE];
-	char pbuf[VTCP_PORTBUFSIZE];
-	int *ret = priv;
-
-	VTCP_name(sa, abuf, sizeof abuf, pbuf, sizeof pbuf);
-	if (strcmp(abuf, "130.225.244.222")) {
-		fprintf(stderr, "DNS-test: Wrong response: %s\n", abuf);
-		*ret = -1;
-	} else if (*ret == 0)
-		*ret = 1;
-	return (0);
-}
-
-static int
-dns_works(void)
-{
-	int ret = 0, error;
-	const char *msg;
-
-	error = VSS_resolver("phk.freebsd.dk", NULL, dns_cb, &ret, &msg);
-	if (error || msg != NULL || ret != 1)
-		return (0);
-	return (1);
-}
-
 /**********************************************************************
  * Figure out what IP related magic
  */
 
-static int __match_proto__(vss_resolved_f)
+static int v_matchproto_(vss_resolved_f)
 bind_cb(void *priv, const struct suckaddr *sa)
 {
 	(void)priv;
@@ -491,6 +458,15 @@ ip_magic(void)
 	VTCP_myname(fd, abuf, sizeof abuf, pbuf, sizeof(pbuf));
 	extmacro_def("localhost", "%s", abuf);
 
+#if defined (__APPLE__)
+	/*
+	 * In MacOS a bound socket that is not listening will timeout
+	 * instead of refusing the connection so close it and hope
+	 * for the best.
+	 */
+	VTCP_close(&fd);
+#endif
+
 	/* Expose a backend that is forever down. */
 	extmacro_def("bad_backend", "%s %s", abuf, pbuf);
 
@@ -660,13 +636,12 @@ main(int argc, char * const *argv)
 
 	AZ(VSB_finish(params_vsb));
 
-	feature_dns = dns_works();
 	ip_magic();
 
 	if (iflg)
 		i_mode();
 
-	vb = vev_new_base();
+	vb = VEV_New();
 
 	i = 0;
 	while (!VTAILQ_EMPTY(&tst_head) || i) {
@@ -679,7 +654,7 @@ main(int argc, char * const *argv)
 			i = 1;
 			continue;
 		}
-		i = vev_schedule_one(vb);
+		i = VEV_Once(vb);
 	}
 	if (vtc_continue)
 		fprintf(stderr,
