@@ -46,13 +46,13 @@
 #define VOPT_DEFINITION
 #define VOPT_INC "varnishtop_options.h"
 
-#include "miniobj.h"
+#include "vdef.h"
+
 #include "vcurses.h"
 #include "vapi/vsl.h"
 #include "vapi/vsm.h"
 #include "vapi/voptget.h"
 #include "vas.h"
-#include "vdef.h"
 #include "vtree.h"
 #include "vut.h"
 
@@ -75,13 +75,13 @@ struct top {
 	double			count;
 };
 
-static float period = 60; /* seconds */
+static int period = 60; /* seconds */
 static int end_of_file = 0;
 static unsigned ntop;
 static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 static int f_flag = 0;
 static unsigned maxfieldlen = 0;
-static char *ident;
+static const char *ident;
 
 static volatile sig_atomic_t quit = 0;
 
@@ -115,7 +115,7 @@ VRB_GENERATE_STATIC(t_order, top, e_order, cmp_order)
 VRB_PROTOTYPE_STATIC(t_key, top, e_key, cmp_key)
 VRB_GENERATE_STATIC(t_key, top, e_key, cmp_key)
 
-static int __match_proto__(VSLQ_dispatch_f)
+static int v_matchproto_(VSLQ_dispatch_f)
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	void *priv)
 {
@@ -130,9 +130,11 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 
 	for (tr = pt[0]; tr != NULL; tr = *++pt) {
 		while ((1 == VSL_Next(tr->c))) {
+			tag = VSL_TAG(tr->c->rec.ptr);
+			if (VSL_tagflags[tag])
+				continue;
 			if (!VSL_Match(vsl, tr->c))
 				continue;
-			tag = VSL_TAG(tr->c->rec.ptr);
 			b = VSL_CDATA(tr->c->rec.ptr);
 			e = b + VSL_LEN(tr->c->rec.ptr);
 			u = 0;
@@ -181,7 +183,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	return (0);
 }
 
-static int __match_proto__(VUT_cb_f)
+static int v_matchproto_(VUT_cb_f)
 sighup(struct VUT *v)
 {
 	assert(v == vut);
@@ -192,8 +194,9 @@ sighup(struct VUT *v)
 static void
 vut_sighandler(int sig)
 {
-	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	VUT_Signaled(vut, sig);
+
+	if (vut != NULL)
+		VUT_Signaled(vut, sig);
 }
 
 static void
@@ -203,7 +206,7 @@ update(int p)
 	int l, len;
 	double t = 0;
 	static time_t last = 0;
-	static unsigned n;
+	static unsigned n = 0;
 	const char *q;
 	time_t now;
 
@@ -270,12 +273,12 @@ do_curses(void *arg)
 	AC(intrflush(stdscr, FALSE));
 	(void)curs_set(0);
 	AC(erase());
+	timeout(1000);
 	while (!quit) {
 		AZ(pthread_mutex_lock(&mtx));
 		update(period);
 		AZ(pthread_mutex_unlock(&mtx));
 
-		timeout(1000);
 		switch (getch()) {
 		case ERR:
 			break;
@@ -298,8 +301,8 @@ do_curses(void *arg)
 		case 'Q':
 		case 'q':
 			AZ(raise(SIGINT));
-			AC(endwin());
-			return (NULL);
+			quit = 1;
+			break;
 		default:
 			AC(beep());
 			break;
@@ -315,8 +318,6 @@ dump(void)
 	struct top *tp, *tp2;
 	for (tp = VRB_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
 		tp2 = VRB_NEXT(t_order, &h_order, tp);
-		if (tp->count <= 1.0)
-			break;
 		printf("%9.2f %s %*.*s\n",
 			tp->count, VSL_tags[tp->tag],
 			tp->clen, tp->clen, tp->rec_data);
@@ -324,7 +325,7 @@ dump(void)
 }
 
 //lint -sem(usage, r_no)
-static void __attribute__((__noreturn__))
+static void v_noreturn_
 usage(int status)
 {
 	const char **opt;
@@ -341,6 +342,7 @@ main(int argc, char **argv)
 {
 	int o, once = 0;
 	pthread_t thr;
+	char *e = NULL;
 
 	vut = VUT_InitProg(argc, argv, &vopt_spec);
 	AN(vut);
@@ -359,8 +361,9 @@ main(int argc, char **argv)
 			usage(0);
 		case 'p':
 			errno = 0;
-			period = strtol(optarg, NULL, 0);
-			if (errno != 0)  {
+			e = NULL;
+			period = strtoul(optarg, &e, 0);
+			if (errno != 0 || e == NULL || *e != '\0') {
 				fprintf(stderr,
 				    "Syntax error, %s is not a number", optarg);
 				exit(1);
@@ -377,23 +380,22 @@ main(int argc, char **argv)
 
 	VUT_Signal(vut_sighandler);
 	VUT_Setup(vut);
-	ident = VSM_Dup(vut->vsm, "Arg", "-i");
-	if (!once) {
-		if (pthread_create(&thr, NULL, do_curses, NULL) != 0) {
-			fprintf(stderr, "pthread_create(): %s\n",
-			    strerror(errno));
-			exit(1);
-		}
-	}
+	if (vut->vsm)
+		ident = VSM_Dup(vut->vsm, "Arg", "-i");
+	else
+		ident = strdup("");
 	vut->dispatch_f = accumulate;
 	vut->dispatch_priv = NULL;
 	vut->sighup_f = sighup;
-	VUT_Main(vut);
-	end_of_file = 1;
-	if (once)
+	if (once) {
+		(void)VUT_Main(vut);
 		dump();
-	else
-		pthread_join(thr, NULL);
+	} else {
+		AZ(pthread_create(&thr, NULL, do_curses, NULL));
+		(void)VUT_Main(vut);
+		end_of_file = 1;
+		AZ(pthread_join(thr, NULL));
+	}
 	VUT_Fini(&vut);
-	exit(0);
+	return (0);
 }
