@@ -34,7 +34,6 @@
 #include "vcli.h"
 
 struct cli;	/* NB: struct cli is opaque at this level.  */
-struct vlu;
 struct VCLS;
 
 typedef void cli_func_t(struct cli*, const char * const *av, void *priv);
@@ -51,7 +50,6 @@ struct cli_cmd_desc {
 
 #define CLI_CMD(U,l,s,h,d,m,M) extern const struct cli_cmd_desc CLICMD_##U[1];
 #include "tbl/cli_cmds.h"
-#undef CLI_CMD
 
 /* A CLI command */
 struct cli_proto {
@@ -69,41 +67,42 @@ struct cli_proto {
 
 /* a CLI session */
 struct cli {
-	unsigned                magic;
+	unsigned		magic;
 #define CLI_MAGIC		0x4038d570
-	struct vsb              *sb;
-	enum VCLI_status_e      result;
-	char                    *cmd;
-	unsigned                auth;
-	char                    challenge[34];
-	char                    *ident;
-	struct vlu              *vlu;
-	struct VCLS             *cls;
-	volatile unsigned       *limit;
+	void			*priv;
+	struct vsb		*sb;
+	enum VCLI_status_e	result;
+	struct vsb		*cmd;
+	unsigned		auth;
+	char			challenge[34];
+	char			*ident;
+	struct VCLS		*cls;
+	volatile unsigned	*limit;
 };
 
 /* The implementation must provide these functions */
 int VCLI_Overflow(struct cli *cli);
-void VCLI_Out(struct cli *cli, const char *fmt, ...) __v_printflike(2, 3);
+void VCLI_Out(struct cli *cli, const char *fmt, ...) v_printflike_(2, 3);
 void VCLI_Quote(struct cli *cli, const char *str);
 void VCLI_JSON_str(struct cli *cli, const char *str);
-void VCLI_JSON_ver(struct cli *cli, unsigned ver, const char * const * av);
+void VCLI_JSON_begin(struct cli *cli, unsigned ver, const char * const * av);
+void VCLI_JSON_end(struct cli *cli);
 void VCLI_SetResult(struct cli *cli, unsigned r);
 
-typedef void cls_cb_f(void *priv);
+typedef int cls_cb_f(void *priv);
 typedef void cls_cbc_f(const struct cli*);
-struct VCLS *VCLS_New(cls_cbc_f *before, cls_cbc_f *after,
-    volatile unsigned *maxlen, volatile unsigned *limit);
+struct VCLS *VCLS_New(struct VCLS *);
+void VCLS_SetHooks(struct VCLS *, cls_cbc_f *, cls_cbc_f *);
+void VCLS_SetLimit(struct VCLS *, volatile unsigned *);
 struct cli *VCLS_AddFd(struct VCLS *cs, int fdi, int fdo, cls_cb_f *closefunc,
     void *priv);
 void VCLS_AddFunc(struct VCLS *cs, unsigned auth, struct cli_proto *clp);
-int VCLS_Poll(struct VCLS *cs, int timeout);
-int VCLS_PollFd(struct VCLS *cs, int fd, int timeout);
+int VCLS_Poll(struct VCLS *cs, const struct cli*, int timeout);
 void VCLS_Destroy(struct VCLS **);
-void VCLS_Clone(struct VCLS *cs, struct VCLS *cso);
 
 /* From libvarnish/cli.c */
 cli_func_t	VCLS_func_close;
 cli_func_t	VCLS_func_help;
 cli_func_t	VCLS_func_help_json;
 cli_func_t	VCLS_func_ping;
+cli_func_t	VCLS_func_ping_json;
