@@ -32,10 +32,13 @@
 
 struct suckaddr;
 extern const int vsa_suckaddr_len;
+extern const struct suckaddr *bogo_ip;
 
+void VSA_Init(void);
 int VSA_Sane(const struct suckaddr *);
 unsigned VSA_Port(const struct suckaddr *);
 int VSA_Compare(const struct suckaddr *, const struct suckaddr *);
+int VSA_Compare_IP(const struct suckaddr *, const struct suckaddr *);
 struct suckaddr *VSA_Clone(const struct suckaddr *sua);
 
 const void *VSA_Get_Sockaddr(const struct suckaddr *, socklen_t *sl);
