@@ -31,5 +31,7 @@
 /* from libvarnish/subproc.c */
 typedef void vsub_func_f(void*);
 
-unsigned VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
+unsigned VSUB_run(struct vsb *, vsub_func_f *, void *priv, const char *name,
     int maxlines);
+
+void VSUB_closefrom(int fd);
