@@ -76,15 +76,19 @@ struct vsm_set;
 struct vsm_seg {
 	unsigned		magic;
 #define VSM_SEG_MAGIC		0xeb6c6dfd
+	unsigned		flags;
+#define VSM_FLAG_MARKSCAN	(1U<<1)
+#define VSM_FLAG_STALE		(1U<<2)
+#define VSM_FLAG_CLUSTER	(1U<<3)
 	VTAILQ_ENTRY(vsm_seg)	list;
-	int			markscan;
-	int			stale;
 	struct vsm_set		*set;
+	struct vsm_seg		*cluster;
 	char			**av;
 	int			refs;
+	void			*s;
+	size_t			sz;
 	void			*b;
 	void			*e;
-	size_t			sz;
 	uintptr_t		serial;
 };
 
@@ -94,6 +98,7 @@ struct vsm_set {
 	const char		*dname;
 	VTAILQ_HEAD(,vsm_seg)	segs;
 	VTAILQ_HEAD(,vsm_seg)	stale;
+	VTAILQ_HEAD(,vsm_seg)	clusters;
 
 	int			dfd;
 	struct stat		dst;
@@ -147,21 +152,72 @@ vsm_diag(struct vsm *vd, const char *fmt, ...)
 
 /*--------------------------------------------------------------------*/
 
+static int
+vsm_mapseg(struct vsm *vd, struct vsm_seg *vg)
+{
+	size_t of, off, sz, ps, len;
+	struct vsb *vsb;
+	int fd;
+
+	CHECK_OBJ_NOTNULL(vg, VSM_SEG_MAGIC);
+
+	if (vg->s != NULL)
+		return (0);
+
+	ps = getpagesize();
+
+	of = strtoul(vg->av[2], NULL, 10);
+	off = RDN2(of, ps);
+
+	if (vg->flags & VSM_FLAG_CLUSTER)
+		assert(of == 0);
+	assert(vg->cluster == NULL);
+
+	sz = strtoul(vg->av[3], NULL, 10);
+	assert(sz > 0);
+	assert(of >= off);
+	len = RUP2((of - off) + sz, ps);
+
+	vsb = VSB_new_auto();
+	AN(vsb);
+	VSB_printf(vsb, "%s/%s/%s", vd->dname, vg->set->dname, vg->av[1]);
+	AZ(VSB_finish(vsb));
+
+	fd = open(VSB_data(vsb), O_RDONLY);	// XXX: openat
+	if (fd < 0) {
+		VSB_destroy(&vsb);
+		return (vsm_diag(vd, "Could not open segment"));
+	}
+
+	vg->s = (void*)mmap(NULL, len,
+	    PROT_READ,
+	    MAP_HASSEMAPHORE | MAP_NOSYNC | MAP_SHARED,
+	    fd, (off_t)off);
+
+	VSB_destroy(&vsb);
+
+	closefd(&fd);
+	if (vg->s == MAP_FAILED)
+		return (vsm_diag(vd, "Could not mmap segment"));
+
+	vg->b = (char*)(vg->s) + of - off;
+	vg->e = (char *)vg->b + sz;
+	vg->sz = len;
+
+	return (0);
+}
+
 static void
 vsm_unmapseg(struct vsm_seg *vg)
 {
-	size_t sz, ps, len;
 
 	CHECK_OBJ_NOTNULL(vg, VSM_SEG_MAGIC);
 
 	AN(vg->b);
 	AN(vg->e);
-	sz = strtoul(vg->av[2], NULL, 10);
-	assert(sz > 0);
-	ps = getpagesize();
-	len = RUP2(sz, ps);
-	AZ(munmap(vg->b, len));
-	vg->b = vg->e = NULL;
+	AZ(munmap(vg->s, vg->sz));
+	vg->s = vg->b = vg->e = NULL;
+	vg->sz = 0;
 }
 
 /*--------------------------------------------------------------------*/
@@ -172,11 +228,13 @@ vsm_delseg(struct vsm_seg *vg)
 
 	CHECK_OBJ_NOTNULL(vg, VSM_SEG_MAGIC);
 
-	if (vg->b != NULL)
+	if (vg->s != NULL)
 		vsm_unmapseg(vg);
 
-	if (vg->stale)
+	if (vg->flags & VSM_FLAG_STALE)
 		VTAILQ_REMOVE(&vg->set->stale, vg, list);
+	else if (vg->flags & VSM_FLAG_CLUSTER)
+		VTAILQ_REMOVE(&vg->set->clusters, vg, list);
 	else
 		VTAILQ_REMOVE(&vg->set->segs, vg, list);
 	VAV_Free(vg->av);
@@ -194,6 +252,7 @@ vsm_newset(const char *dirname)
 	AN(vs);
 	VTAILQ_INIT(&vs->segs);
 	VTAILQ_INIT(&vs->stale);
+	VTAILQ_INIT(&vs->clusters);
 	vs->dname = dirname;
 	vs->dfd = vs->fd = -1;
 	return (vs);
@@ -215,6 +274,8 @@ vsm_delset(struct vsm_set **p)
 		vsm_delseg(VTAILQ_FIRST(&vs->stale));
 	while (!VTAILQ_EMPTY(&vs->segs))
 		vsm_delseg(VTAILQ_FIRST(&vs->segs));
+	while (!VTAILQ_EMPTY(&vs->clusters))
+		vsm_delseg(VTAILQ_FIRST(&vs->clusters));
 	FREE_OBJ(vs);
 }
 
@@ -342,6 +403,21 @@ vsm_cmp_av(char * const *a1, char * const *a2)
 	}
 }
 
+static struct vsm_seg *
+vsm_findcluster(const struct vsm_seg *vga)
+{
+	const struct vsm_set *vs = vga->set;
+	struct vsm_seg *vg;
+	AN(vs);
+	AN(vga->av[1]);
+	VTAILQ_FOREACH(vg, &vs->clusters, list) {
+		AN(vg->av[1]);
+		if (!strcmp(vga->av[1], vg->av[1]))
+			return (vg);
+	}
+	return (NULL);
+}
+
 static unsigned
 vsm_refresh_set2(struct vsm *vd, struct vsm_set *vs, struct vsb *vsb)
 {
@@ -421,9 +497,7 @@ vsm_refresh_set2(struct vsm *vd, struct vsm_set *vs, struct vsb *vsb)
 		return (retval|VSM_NUKE_ALL);
 
 	/*
-	 * Examine the ident line
-	 * XXX: for now ignore that one of the ID's is a pid which could
-	 * XXX: be kill(pid,0)'ed for more rapid abandonment detection.
+	 * First line is ident comment
 	 */
 	i = sscanf(VSB_data(vsb), "# %ju %ju\n%n", &id1, &id2, &ac);
 	if (i != 2) {
@@ -445,9 +519,21 @@ vsm_refresh_set2(struct vsm *vd, struct vsm_set *vs, struct vsb *vsb)
 	p = VSB_data(vsb) + ac;
 
 	VTAILQ_FOREACH(vg, &vs->segs, list)
-		vg->markscan = 0;
+		vg->flags &= ~VSM_FLAG_MARKSCAN;
+
+	/*
+	 * Efficient comparison by walking the two lists side-by-side because
+	 * segment inserts always happen at the tail (VSMW_Allocv()). So, as
+	 * soon as vg is exhausted, we only insert.
+	 *
+	 * For restarts, we require a tabula rasa
+	 */
+
+	if (retval & VSM_MGT_RESTARTED)
+		vg = NULL;
+	else
+		vg = VTAILQ_FIRST(&vs->segs);
 
-	vg = VTAILQ_FIRST(&vs->segs);
 	while (p != NULL && *p != '\0') {
 		e = strchr(p, '\n');
 		if (e == NULL)
@@ -456,31 +542,42 @@ vsm_refresh_set2(struct vsm *vd, struct vsm_set *vs, struct vsb *vsb)
 		av = VAV_Parse(p, &ac, 0);
 		p = e + 1;
 
-		if (av[0] != NULL || ac < 4 || ac > 5) {
+		if (av[0] != NULL || ac < 4 || ac > 6) {
 			(void)(vsm_diag(vd,
 			    "vsm_refresh_set2: bad index (%d/%s)",
 			    ac, av[0]));
 			VAV_Free(av);
 			break;
 		}
-		while (vg != NULL && !vsm_cmp_av(vg->av, av))
-			vg = VTAILQ_NEXT(vg, list);
 
-		if (vg != NULL) {
-			VAV_Free(av);
-			vg->markscan = 1;
-			vg = VTAILQ_NEXT(vg, list);
-		} else {
+		if (vg == NULL) {
 			ALLOC_OBJ(vg2, VSM_SEG_MAGIC);
 			AN(vg2);
 			vg2->av = av;
 			vg2->set = vs;
-			vg2->markscan = 1;
+			vg2->flags = VSM_FLAG_MARKSCAN;
 			vg2->serial = ++vd->serial;
-			VTAILQ_INSERT_TAIL(&vs->segs, vg2, list);
+			if (ac == 4) {
+				vg2->flags |= VSM_FLAG_CLUSTER;
+				VTAILQ_INSERT_TAIL(&vs->clusters, vg2, list);
+			} else {
+				VTAILQ_INSERT_TAIL(&vs->segs, vg2, list);
+				vg2->cluster = vsm_findcluster(vg2);
+			}
+			continue;
 		}
 
+		while (vg != NULL && vsm_cmp_av(&vg->av[1], &av[1]))
+			vg = VTAILQ_NEXT(vg, list);
 
+		VAV_Free(av);
+
+		if (vg == NULL)
+			continue;
+
+		/* entry compared equal, so it survives */
+		vg->flags |= VSM_FLAG_MARKSCAN;
+		vg = VTAILQ_NEXT(vg, list);
 	}
 	return (retval);
 }
@@ -495,10 +592,11 @@ vsm_refresh_set(struct vsm *vd, struct vsm_set *vs, struct vsb *vsb)
 	if (retval & VSM_NUKE_ALL)
 		retval |= VSM_MGT_CHANGED;
 	VTAILQ_FOREACH_SAFE(vg, &vs->segs, list, vg2) {
-		if (!vg->markscan || (retval & VSM_NUKE_ALL)) {
+		if ((vg->flags & VSM_FLAG_MARKSCAN) == 0 ||
+		    (retval & VSM_NUKE_ALL)) {
 			VTAILQ_REMOVE(&vs->segs, vg, list);
 			if (vg->refs) {
-				vg->stale = 1;
+				vg->flags |= VSM_FLAG_STALE;
 				VTAILQ_INSERT_TAIL(&vs->stale, vg, list);
 			} else {
 				VAV_Free(vg->av);
@@ -663,8 +761,8 @@ VSM__itern(struct vsm *vd, struct vsm_fantom *vf)
 		return (0);
 	memset(vf, 0, sizeof *vf);
 	vf->priv = vg2->serial;
-	vf->class = vg2->av[3];
-	vf->ident = vg2->av[4];
+	vf->class = vg2->av[4];
+	vf->ident = vg2->av[5];
 	return (1);
 }
 
@@ -673,10 +771,9 @@ VSM__itern(struct vsm *vd, struct vsm_fantom *vf)
 int
 VSM_Map(struct vsm *vd, struct vsm_fantom *vf)
 {
-	struct vsm_seg *vg;
-	size_t sz, ps, len;
-	struct vsb *vsb;
-	int fd;
+	struct vsm_seg *vg, *vgc;
+	size_t of, sz;
+	int r;
 
 	CHECK_OBJ_NOTNULL(vd, VSM_MAGIC);
 	AN(vd->attached);
@@ -686,8 +783,8 @@ VSM_Map(struct vsm *vd, struct vsm_fantom *vf)
 		return (vsm_diag(vd, "VSM_Map: bad fantom"));
 
 	assert(vg->serial == vf->priv);
-	assert(vg->av[3] == vf->class);
-	assert(vg->av[4] == vf->ident);
+	assert(vg->av[4] == vf->class);
+	assert(vg->av[5] == vf->ident);
 
 	if (vg->b != NULL) {
 		assert(vg->refs > 0);
@@ -698,35 +795,39 @@ VSM_Map(struct vsm *vd, struct vsm_fantom *vf)
 		return (0);
 	}
 
-	sz = strtoul(vg->av[2], NULL, 10);
-	assert(sz > 0);
-	ps = getpagesize();
-	len = RUP2(sz, ps);
+	assert(vg->refs == 0);
 
-	vsb = VSB_new_auto();
-	AN(vsb);
-	VSB_printf(vsb, "%s/%s/%s", vd->dname, vg->set->dname, vg->av[1]);
-	AZ(VSB_finish(vsb));
+	vgc = vg->cluster;
 
-	fd = open(VSB_data(vsb), O_RDONLY);	// XXX: openat
-	if (fd < 0) {
-		VSB_destroy(&vsb);
-		return (vsm_diag(vd, "Could not open segment"));
+	if (vgc == NULL) {
+		r = vsm_mapseg(vd, vg);
+		if (r)
+			return (r);
+		vf->b = vg->b;
+		vf->e = vg->e;
+
+		vg->refs++;
+
+		return (0);
 	}
 
-	vg->b = (void*)mmap(NULL, len,
-	    PROT_READ,
-	    MAP_HASSEMAPHORE | MAP_NOSYNC | MAP_SHARED,
-	    fd, 0);
+	assert(vgc->flags & VSM_FLAG_CLUSTER);
+	assert(vg->s == NULL);
+	assert(vg->sz == 0);
 
-	VSB_destroy(&vsb);
+	r = vsm_mapseg(vd, vgc);
+	if (r)
+		return (r);
+	vgc->refs++;
 
+	of = strtoul(vg->av[2], NULL, 10);
+	sz = strtoul(vg->av[3], NULL, 10);
+	assert(sz > 0);
 
-	closefd(&fd);
-	if (vg->b == MAP_FAILED)
-		return (vsm_diag(vd, "Could not mmap segment"));
+	assert(vgc->sz >= of + sz);
+	assert(vgc->s == vgc->b);
+	vg->b = (char *)vgc->b + of;
 	vg->e = (char *)vg->b + sz;
-	vg->sz = len;
 
 	vf->b = vg->b;
 	vf->e = vg->e;
@@ -756,8 +857,18 @@ VSM_Unmap(struct vsm *vd, struct vsm_fantom *vf)
 	vf->e = NULL;
 	if (vg->refs > 0)
 		return(0);
-	vsm_unmapseg(vg);
-	if (vg->stale)
+
+	if (vg->cluster) {
+		assert(vg->s == NULL);
+		assert(vg->sz == 0);
+		assert(vg->cluster->refs > 0);
+		if (--vg->cluster->refs == 0)
+			vsm_unmapseg(vg->cluster);
+		vg->b = vg->e = NULL;
+	} else {
+		vsm_unmapseg(vg);
+	}
+	if (vg->flags & VSM_FLAG_STALE)
 		vsm_delseg(vg);
 	return (0);
 }
@@ -772,7 +883,7 @@ VSM_StillValid(const struct vsm *vd, const struct vsm_fantom *vf)
 	CHECK_OBJ_NOTNULL(vd, VSM_MAGIC);
 	AN(vf);
 	vg = vsm_findseg(vd, vf);
-	if (vg == NULL || vg->stale)
+	if (vg == NULL || vg->flags & VSM_FLAG_STALE)
 		return (VSM_invalid);
 	return (VSM_valid);
 }
