@@ -40,7 +40,7 @@
 #include <errno.h>
 #include <string.h>
 #include <signal.h>
-#include <math.h>
+#include <sys/stat.h> /* for MUSL */
 
 #include "compat/daemon.h"
 #include "vdef.h"
@@ -62,6 +62,8 @@ static int vut_options(const struct vopt_spec *);
 static struct vpf_fh	*pfh;
 static unsigned		daemonized;
 
+static struct VUT pfh_vut;
+
 static int
 vut_daemon(struct VUT *vut)
 {
@@ -74,13 +76,21 @@ vut_daemon(struct VUT *vut)
 static void
 vut_vpf_remove(void)
 {
-	if (pfh != NULL) {
-		AZ(VPF_Remove(pfh));
-		pfh = NULL;
-	}
+
+	CHECK_OBJ(&pfh_vut, VUT_MAGIC);
+	AN(pfh);
+	AN(pfh_vut.P_arg);
+
+	if (VPF_Remove(pfh) != 0)
+		VUT_Error(&pfh_vut, 1, "Cannot remove pid file %s: %s",
+		    pfh_vut.P_arg, strerror(errno));
+
+	free(pfh_vut.P_arg);
+	ZERO_OBJ(&pfh_vut, sizeof pfh_vut);
+	pfh = NULL;
 }
 
-static int __match_proto__(VSLQ_dispatch_f)
+static int v_matchproto_(VSLQ_dispatch_f)
 vut_dispatch(struct VSL_data *vsl, struct VSL_transaction * const trans[],
     void *priv)
 {
@@ -101,7 +111,7 @@ vut_dispatch(struct VSL_data *vsl, struct VSL_transaction * const trans[],
 }
 
 //lint -sem(vut_error, r_no)
-static void __attribute__((__noreturn__)) __match_proto__(VUT_error_f)
+static void v_noreturn_ v_matchproto_(VUT_error_f)
 vut_error(struct VUT *vut, int status, const char *fmt, va_list ap)
 {
 
@@ -125,6 +135,7 @@ VUT_Error(struct VUT *vut, int status, const char *fmt, ...)
 	va_start(ap, fmt);
 	vut->error_f(vut, status, fmt, ap);
 	va_end(ap);
+	exit(2);
 }
 
 int
@@ -212,6 +223,10 @@ VUT_Init(const char *progname, int argc, char * const *argv,
 		exit(vut_synopsis(voc));
 	if (argc == 2 && !strcmp(argv[1], "--options"))
 		exit(vut_options(voc));
+	if (argc == 2 && !strcmp(argv[1], "--optstring")) {
+		(void)printf("%s\n", voc->vopt_optstring);
+		exit(0);
+	}
 
 	vut->progname = progname;
 	vut->g_arg = VSL_g_vxid;
@@ -239,9 +254,9 @@ VUT_Signaled(struct VUT *vut, int sig)
 {
 
 	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	vut->sighup |= (sig == SIGHUP);
-	vut->sigint |= (sig == SIGINT || sig == SIGTERM);
-	vut->sigusr1 |= (sig == SIGUSR1);
+	vut->sighup |= (int)(sig == SIGHUP);
+	vut->sigint |= (int)(sig == SIGINT || sig == SIGTERM);
+	vut->sigusr1 |= (int)(sig == SIGUSR1);
 }
 
 void
@@ -260,7 +275,8 @@ VUT_Setup(struct VUT *vut)
 		VUT_Error(vut, 1, "Only one of -n and -r options may be used");
 
 	/* Create and validate the query expression */
-	vut->vslq = VSLQ_New(vut->vsl, NULL, vut->g_arg, vut->q_arg);
+	vut->vslq = VSLQ_New(vut->vsl, NULL,
+	    (enum VSL_grouping_e)vut->g_arg, vut->q_arg);
 	if (vut->vslq == NULL)
 		VUT_Error(vut, 1, "Query expression error:\n%s",
 		    VSL_Error(vut->vsl));
@@ -301,6 +317,13 @@ VUT_Setup(struct VUT *vut)
 	if (vut->P_arg) {
 		AN(pfh);
 		AZ(VPF_Write(pfh));
+
+		/* NB: move ownership to a global pseudo-VUT. */
+		INIT_OBJ(&pfh_vut, VUT_MAGIC);
+		pfh_vut.P_arg = vut->P_arg;
+		pfh_vut.error_f = vut->error_f;
+		vut->P_arg = NULL;
+
 		AZ(atexit(vut_vpf_remove));
 	}
 }
@@ -314,13 +337,10 @@ VUT_Fini(struct VUT **vutp)
 	AN(vut->progname);
 
 	free(vut->n_arg);
-	free(vut->P_arg);
 	free(vut->q_arg);
 	free(vut->r_arg);
 	free(vut->t_arg);
-
-	vut_vpf_remove();
-	AZ(pfh);
+	AZ(vut->P_arg);
 
 	if (vut->vslq)
 		VSLQ_Delete(&vut->vslq);
@@ -358,12 +378,26 @@ VUT_Main(struct VUT *vut)
 			(void)VSLQ_Flush(vut->vslq, vut_dispatch, vut);
 		}
 
-		// We must repeatedly call VSM_Status() when !hascursor
-		// to make VSM discover our segment.
+		/* We must repeatedly call VSM_Status() when !hascursor
+		 * to make VSM discover our segment.
+		 *
+		 * XXX consider moving the error handling to VSLQ_Dispatch.
+		 * or some other VSL utility function
+		 * Reasons:
+		 *
+		 * - it does not seem to make much sense to call VSM_StillValid
+		 *   in vsl if that can only detect invalid segments after
+		 *   VSM_Status has run, so it appears both should be
+		 *   consolidated
+		 *
+		 * - not all VSL Clients will use VUT, yet the log abandoned/
+		 *   overrun situation will be occur for all of them.
+		 */
+
 		if (vut->vsm != NULL &&
 		    (VSM_Status(vut->vsm) & VSM_WRK_RESTARTED)) {
 			if (hascursor < 1) {
-				fprintf(stderr, "Log abandonned\n");
+				fprintf(stderr, "Log abandoned (vsm)\n");
 				VSLQ_SetCursor(vut->vslq, NULL);
 				hascursor = 0;
 			}
@@ -386,12 +420,13 @@ VUT_Main(struct VUT *vut)
 			AZ(c);
 		}
 
-		i = VSLQ_Dispatch(vut->vslq, vut_dispatch, vut);
-		if (i == 1)
-			/* Call again */
+		do
+			i = VSLQ_Dispatch(vut->vslq, vut_dispatch, vut);
+		while (i == vsl_more && !vut->sighup && !vut->sigusr1);
+
+		if (i == vsl_more)
 			continue;
-		else if (i == 0) {
-			/* Nothing to do but wait */
+		else if (i == vsl_end) {
 			if (vut->idle_f) {
 				i = vut->idle_f(vut);
 				if (i)
@@ -399,10 +434,8 @@ VUT_Main(struct VUT *vut)
 			}
 			VTIM_sleep(0.01);
 			continue;
-		} else if (i == -1) {
-			/* EOF */
+		} else if (i == vsl_e_eof)
 			break;
-		}
 
 		if (vut->vsm == NULL)
 			break;
@@ -411,14 +444,16 @@ VUT_Main(struct VUT *vut)
 
 		(void)VSLQ_Flush(vut->vslq, vut_dispatch, vut);
 
-		if (i == -2) {
-			/* Abandoned */
-			fprintf(stderr, "Log abandoned\n");
+		if (i == vsl_e_abandon) {
+			fprintf(stderr, "Log abandoned (vsl)\n");
 			VSLQ_SetCursor(vut->vslq, NULL);
 			hascursor = 0;
-		} else if (i < -2)
-			/* Overrun */
+		} else if (i == vsl_e_overrun) {
 			fprintf(stderr, "Log overrun\n");
+			VSLQ_SetCursor(vut->vslq, NULL);
+			hascursor = 0;
+		} else
+			fprintf(stderr, "Error %d from VSLQ_Dispatch()", i);
 	}
 
 	return (i);
@@ -485,5 +520,8 @@ vut_options(const struct vopt_spec *voc)
 
 	for (i = 0; i < voc->vopt_list_n; i++)
 		print_opt(&voc->vopt_list[i]);
+	printf("--optstring\n"
+	    "\tPrint the optstring parameter to ``getopt(3)`` to help"
+	    " writing wrapper scripts.\n\n");
 	return (0);
 }
