@@ -35,6 +35,19 @@
 
 #include "vcc_compile.h"
 
+const char *
+vcc_default_probe(struct vcc *tl)
+{
+
+	if (tl->default_probe != NULL)
+		return (tl->default_probe);
+	VSB_printf(tl->sb, "No default probe defined\n");
+	vcc_ErrToken(tl, tl->t);
+	VSB_printf(tl->sb, " at\n");
+	vcc_ErrWhere(tl, tl->t);
+	return ("");
+}
+
 /*--------------------------------------------------------------------
  * Struct sockaddr is not really designed to be a compile time
  * initialized data structure, so we encode it as a byte-string
@@ -68,20 +81,19 @@ Emit_Sockaddr(struct vcc *tl, const struct token *t_host,
 		Fb(tl, 0, "\t.ipv6_addr = \"%s\",\n", ipv6a);
 	}
 	Fb(tl, 0, "\t.port = \"%s\",\n", pa);
+	Fb(tl, 0, "\t.path = (void *) 0,\n");
 }
 
 /*--------------------------------------------------------------------
- * Parse a backend probe specification
+ * Disallow mutually exclusive field definitions
  */
 
 static void
-vcc_ProbeRedef(struct vcc *tl, struct token **t_did,
+vcc_Redef(struct vcc *tl, const char *redef, struct token **t_did,
     struct token *t_field)
 {
-	/* .url and .request are mutually exclusive */
-
 	if (*t_did != NULL) {
-		VSB_printf(tl->sb, "Probe request redefinition at:\n");
+		VSB_printf(tl->sb, "%s redefinition at:\n", redef);
 		vcc_ErrWhere(tl, t_field);
 		VSB_printf(tl->sb, "Previous definition:\n");
 		vcc_ErrWhere(tl, *t_did);
@@ -90,6 +102,10 @@ vcc_ProbeRedef(struct vcc *tl, struct token **t_did,
 	*t_did = t_field;
 }
 
+/*--------------------------------------------------------------------
+ * Parse a backend probe specification
+ */
+
 static void
 vcc_ParseProbeSpec(struct vcc *tl, const struct symbol *sym, char **name)
 {
@@ -139,7 +155,7 @@ vcc_ParseProbeSpec(struct vcc *tl, const struct symbol *sym, char **name)
 		vcc_IsField(tl, &t_field, fs);
 		ERRCHK(tl);
 		if (vcc_IdIs(t_field, "url")) {
-			vcc_ProbeRedef(tl, &t_did, t_field);
+			vcc_Redef(tl, "Probe request", &t_did, t_field);
 			ERRCHK(tl);
 			ExpectErr(tl, CSTR);
 			Fh(tl, 0, "\t.url = ");
@@ -147,7 +163,7 @@ vcc_ParseProbeSpec(struct vcc *tl, const struct symbol *sym, char **name)
 			Fh(tl, 0, ",\n");
 			vcc_NextToken(tl);
 		} else if (vcc_IdIs(t_field, "request")) {
-			vcc_ProbeRedef(tl, &t_did, t_field);
+			vcc_Redef(tl, "Probe request", &t_did, t_field);
 			ERRCHK(tl);
 			ExpectErr(tl, CSTR);
 			Fh(tl, 0, "\t.request =\n");
@@ -249,7 +265,6 @@ vcc_ParseProbeSpec(struct vcc *tl, const struct symbol *sym, char **name)
 void
 vcc_ParseProbe(struct vcc *tl)
 {
-	struct token *t_probe;
 	struct symbol *sym;
 	char *p;
 
@@ -257,17 +272,15 @@ vcc_ParseProbe(struct vcc *tl)
 
 	vcc_ExpectVid(tl, "backend probe");	/* ID: name */
 	ERRCHK(tl);
-	t_probe = tl->t;
-	vcc_NextToken(tl);
-
-	sym = VCC_HandleSymbol(tl, t_probe, PROBE, "vgc_probe");
-	ERRCHK(tl);
-	AN(sym);
-
-	vcc_ParseProbeSpec(tl, sym, &p);
-	if (vcc_IdIs(t_probe, "default")) {
-		(void)vcc_AddRef(tl, t_probe, SYM_PROBE);
+	if (vcc_IdIs(tl->t, "default")) {
+		vcc_NextToken(tl);
+		vcc_ParseProbeSpec(tl, NULL, &p);
 		tl->default_probe = p;
+	} else {
+		sym = VCC_HandleSymbol(tl, PROBE, "vgc_probe");
+		ERRCHK(tl);
+		AN(sym);
+		vcc_ParseProbeSpec(tl, sym, &p);
 	}
 }
 
@@ -284,8 +297,10 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 	struct token *t_val;
 	struct token *t_host = NULL;
 	struct token *t_port = NULL;
+	struct token *t_path = NULL;
 	struct token *t_hosthdr = NULL;
 	struct symbol *pb;
+	struct token *t_did = NULL;
 	struct fld_spec *fs;
 	struct inifin *ifp;
 	struct vsb *vsb;
@@ -294,8 +309,9 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 	double t;
 
 	fs = vcc_FldSpec(tl,
-	    "!host",
+	    "?host",
 	    "?port",
+	    "?path",
 	    "?host_header",
 	    "?connect_timeout",
 	    "?first_byte_timeout",
@@ -335,6 +351,8 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 		vcc_IsField(tl, &t_field, fs);
 		ERRCHK(tl);
 		if (vcc_IdIs(t_field, "host")) {
+			vcc_Redef(tl, "Address", &t_did, t_field);
+			ERRCHK(tl);
 			ExpectErr(tl, CSTR);
 			assert(tl->t->dec != NULL);
 			t_host = tl->t;
@@ -346,6 +364,23 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 			t_port = tl->t;
 			vcc_NextToken(tl);
 			SkipToken(tl, ';');
+		} else if (vcc_IdIs(t_field, "path")) {
+			if (tl->syntax < VCL_41) {
+				VSB_printf(tl->sb,
+				    "Unix socket backends only supported"
+				    " in VCL4.1 and higher.\n");
+				vcc_ErrToken(tl, tl->t);
+				VSB_printf(tl->sb, " at ");
+				vcc_ErrWhere(tl, tl->t);
+				return;
+			}
+			vcc_Redef(tl, "Address", &t_did, t_field);
+			ERRCHK(tl);
+			ExpectErr(tl, CSTR);
+			assert(tl->t->dec != NULL);
+			t_path = tl->t;
+			vcc_NextToken(tl);
+			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "host_header")) {
 			ExpectErr(tl, CSTR);
 			assert(tl->t->dec != NULL);
@@ -392,16 +427,16 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 			Fb(tl, 0, "\t.probe = %s,\n", p);
 			ERRCHK(tl);
 		} else if (vcc_IdIs(t_field, "probe") && tl->t->tok == ID) {
-			pb = VCC_SymbolTok(tl, NULL, tl->t, SYM_PROBE, 0);
-			if (pb == NULL) {
-				VSB_printf(tl->sb, "Probe %.*s not found\n",
-				    PF(tl->t));
-				vcc_ErrWhere(tl, tl->t);
-				return;
+			if (vcc_IdIs(tl->t, "default")) {
+				vcc_NextToken(tl);
+				(void)vcc_default_probe(tl);
+			} else {
+				pb = VCC_SymbolGet(tl, SYM_PROBE,
+				    "Probe not found", XREF_REF);
+				ERRCHK(tl);
+				AN(pb);
+				Fb(tl, 0, "\t.probe = %s,\n", pb->rname);
 			}
-			Fb(tl, 0, "\t.probe = %s,\n", pb->rname);
-			(void)vcc_AddRef(tl, tl->t, SYM_PROBE);
-			vcc_NextToken(tl);
 			SkipToken(tl, ';');
 		} else if (vcc_IdIs(t_field, "probe")) {
 			VSB_printf(tl->sb,
@@ -420,9 +455,19 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 	vcc_FieldsOk(tl, fs);
 	ERRCHK(tl);
 
-	/* Check that the hostname makes sense */
-	assert(t_host != NULL);
-	Emit_Sockaddr(tl, t_host, t_port);
+	if (t_host == NULL && t_path == NULL) {
+		VSB_printf(tl->sb, "Expected .host or .path.\n");
+		vcc_ErrWhere(tl, t_be);
+		return;
+	}
+
+	assert(t_host != NULL || t_path != NULL);
+	if (t_host != NULL)
+		/* Check that the hostname makes sense */
+		Emit_Sockaddr(tl, t_host, t_port);
+	else
+		/* Check that the path can be a legal UDS */
+		Emit_UDS_Path(tl, t_path, "Backend path");
 	ERRCHK(tl);
 
 	ExpectErr(tl, '}');
@@ -430,11 +475,14 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 	/* We have parsed it all, emit the ident string */
 
 	/* Emit the hosthdr field, fall back to .host if not specified */
+	/* If .path is specified, set "0.0.0.0". */
 	Fb(tl, 0, "\t.hosthdr = ");
 	if (t_hosthdr != NULL)
 		EncToken(tl->fb, t_hosthdr);
-	else
+	else if (t_host != NULL)
 		EncToken(tl->fb, t_host);
+	else
+		Fb(tl, 0, "\"0.0.0.0\"");
 	Fb(tl, 0, ",\n");
 
 	/* Close the struct */
@@ -449,8 +497,10 @@ vcc_ParseHostDef(struct vcc *tl, const struct token *t_be, const char *vgcname)
 
 	ifp = New_IniFin(tl);
 	VSB_printf(ifp->ini,
-	    "\t%s =\n\t    VRT_new_backend(ctx, &vgc_dir_priv_%s);",
+	    "\t%s =\n\t    VRT_new_backend_clustered(ctx, vsc_cluster,\n"
+	    "\t\t&vgc_dir_priv_%s);",
 	    vgcname, vgcname);
+	VSB_printf(ifp->fin, "\t\tVRT_delete_backend(ctx, &%s);", vgcname);
 }
 
 /*--------------------------------------------------------------------
@@ -462,33 +512,60 @@ vcc_ParseBackend(struct vcc *tl)
 {
 	struct token *t_first, *t_be;
 	struct symbol *sym;
+	const char *dn;
 
+	tl->ndirector++;
 	t_first = tl->t;
-	vcc_NextToken(tl);		/* ID: backend */
+	SkipToken(tl, ID);		/* ID: backend */
 
 	vcc_ExpectVid(tl, "backend");	/* ID: name */
 	ERRCHK(tl);
 
 	t_be = tl->t;
-	vcc_NextToken(tl);
-
-	sym = VCC_HandleSymbol(tl, t_be, BACKEND, "vgc_backend");
-	ERRCHK(tl);
-
-	Fh(tl, 0, "\nstatic struct director *%s;\n", sym->rname);
-
-	vcc_ParseHostDef(tl, t_be, sym->rname);
-	ERRCHK(tl);
-
+	if (vcc_IdIs(tl->t, "default")) {
+		if (tl->first_director != NULL) {
+			tl->first_director->noref = 0;
+			tl->first_director = NULL;
+			tl->default_director = NULL;
+		}
+		if (tl->default_director != NULL) {
+			VSB_printf(tl->sb,
+			    "Only one default director possible.\n");
+			vcc_ErrWhere(tl, t_first);
+			return;
+		}
+		vcc_NextToken(tl);
+		dn = "vgc_backend_default";
+		tl->default_director = dn;
+	} else {
+		sym = VCC_HandleSymbol(tl, BACKEND, "vgc_backend");
+		ERRCHK(tl);
+		dn = sym->rname;
+		if (tl->default_director == NULL) {
+			tl->first_director = sym;
+			tl->default_director = dn;
+			sym->noref = 1;
+		}
+	}
+	Fh(tl, 0, "\nstatic VCL_BACKEND %s;\n", dn);
+	vcc_ParseHostDef(tl, t_be, dn);
 	if (tl->err) {
 		VSB_printf(tl->sb,
 		    "\nIn %.*s specification starting at:\n", PF(t_first));
 		vcc_ErrWhere(tl, t_first);
 		return;
 	}
+}
 
-	if (tl->default_director == NULL || vcc_IdIs(t_be, "default")) {
-		tl->default_director = sym->rname;
-		tl->t_default_director = t_be;
-	}
+void
+vcc_Backend_Init(struct vcc *tl)
+{
+	struct inifin *ifp;
+
+	Fh(tl, 0, "\nstatic struct vsmw_cluster *vsc_cluster;\n");
+	ifp = New_IniFin(tl);
+	VSB_printf(ifp->ini, "\tvsc_cluster = VRT_VSM_Cluster_New(ctx,\n"
+	    "\t    ndirector * VRT_backend_vsm_need(ctx));\n");
+	VSB_printf(ifp->ini, "\tif (vsc_cluster == 0)\n\t\treturn(1);");
+	VSB_printf(ifp->fin, "\t\tVRT_VSM_Cluster_Destroy(ctx, &vsc_cluster);");
 }
