@@ -131,13 +131,25 @@ const struct type STEVEDORE[1] = {{
 const struct type STRING[1] = {{
 	.magic =		TYPE_MAGIC,
 	.name =			"STRING",
+	.stringform =		1,
+}};
+
+const struct type STRANDS[1] = {{
+	.magic =		TYPE_MAGIC,
+	.name =			"STRANDS",
+	.stringform =		1,
+}};
+
+const struct type STRINGS[1] = {{
+	.magic =		TYPE_MAGIC,
+	.name =			"STRINGS",
 	.tostring =		"",
 }};
 
 const struct type STRING_LIST[1] = {{
 	.magic =		TYPE_MAGIC,
 	.name =			"STRING_LIST",
-	.tostring =		"VRT_CollectString(ctx,\n\v1,\nvrt_magic_string_end)",
+	.stringform =		1,
 }};
 
 const struct type SUB[1] = {{
