@@ -36,99 +36,51 @@
 
 /*--------------------------------------------------------------------*/
 
-void __match_proto__(sym_wildcard_t)
-vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent,
-    const char *b, const char *e)
+void v_matchproto_(sym_wildcard_t)
+vcc_Var_Wildcard(struct vcc *tl, struct symbol *parent, struct symbol *sym)
 {
-	struct symbol *sym;
-	struct var *v;
-	const struct var *vh;
 	struct vsb *vsb;
-	unsigned len;
 
-	vh = parent->wildcard_priv;
-	assert(vh->fmt == HEADER);
+	assert(parent->type == HEADER);
 
-	if (b + 127 <= e) {
-		VSB_printf(tl->sb, "HTTP header (%.20s..) is too long.\n", b);
+	if (sym->nlen >= 127) {
+		VSB_printf(tl->sb, "HTTP header (%.20s..) is too long.\n",
+		    sym->name);
 		VSB_cat(tl->sb, "\nAt: ");
 		vcc_ErrWhere(tl, tl->t);
-		return;
 	}
 
-	v = TlAlloc(tl, sizeof *v);
-	AN(v);
-	v->r_methods = vh->r_methods;
-	v->w_methods = vh->w_methods;
-	v->fmt = vh->fmt;
+	AN(sym);
+	sym->noref = 1;
+	sym->kind = SYM_VAR;
+	sym->type = parent->type;
+	sym->eval = vcc_Eval_Var;
+	sym->r_methods = parent->r_methods;
+	sym->w_methods = parent->w_methods;
+	sym->u_methods = parent->u_methods;
 
 	/* Create a C-name version of the header name */
 	vsb = VSB_new_auto();
 	AN(vsb);
-	VSB_printf(vsb, "&VGC_%s_", vh->rname);
-	VCC_PrintCName(vsb, b, e);
+	VSB_printf(vsb, "&VGC_%s_", parent->rname);
+	VCC_PrintCName(vsb, sym->name, NULL);
 	AZ(VSB_finish(vsb));
 
 	/* Create the static identifier */
-	len = (unsigned)(e - b);
 	Fh(tl, 0, "static const struct gethdr_s %s =\n", VSB_data(vsb) + 1);
-	Fh(tl, 0, "    { %s, \"\\%03o%.*s:\"};\n",
-	    vh->rname, len + 1, len, b);
+	Fh(tl, 0, "    { %s, \"\\%03o%s:\"};\n",
+	    parent->rname, sym->nlen + 1, sym->name);
 
 	/* Create the symbol r/l values */
-	v->rname = TlDup(tl, VSB_data(vsb));
+	sym->rname = TlDup(tl, VSB_data(vsb));
 	VSB_clear(vsb);
-	VSB_printf(vsb, "VRT_SetHdr(ctx, %s,", v->rname);
+	VSB_printf(vsb, "VRT_SetHdr(ctx, %s,", sym->rname);
 	AZ(VSB_finish(vsb));
-	v->lname = TlDup(tl, VSB_data(vsb));
+	sym->lname = TlDup(tl, VSB_data(vsb));
+	VSB_clear(vsb);
+	VSB_printf(vsb, "VRT_SetHdr(ctx, %s, vrt_magic_string_unset)",
+	    sym->rname);
+	AZ(VSB_finish(vsb));
+	sym->uname = TlDup(tl, VSB_data(vsb));
 	VSB_destroy(&vsb);
-
-	sym = VCC_Symbol(tl, parent, b, e, SYM_VAR, 1);
-	AN(sym);
-	sym->fmt = v->fmt;
-	sym->eval = vcc_Eval_Var;
-	sym->r_methods = v->r_methods;
-	sym->w_methods = v->w_methods;
-	sym->lname = v->lname;
-	REPLACE(sym->rname, v->rname);
-}
-
-/*--------------------------------------------------------------------*/
-
-const struct symbol *
-vcc_FindVar(struct vcc *tl, const struct token *t, int wr_access,
-    const char *use)
-{
-	const struct symbol *sym;
-
-	sym = VCC_SymbolTok(tl, NULL, t, SYM_VAR, 0);
-	if (tl->err)
-		return (NULL);
-	if (sym != NULL) {
-		if (wr_access && sym->w_methods == 0) {
-			VSB_printf(tl->sb, "Variable ");
-			vcc_ErrToken(tl, t);
-			VSB_printf(tl->sb, " is read only.");
-			VSB_cat(tl->sb, "\nAt: ");
-			vcc_ErrWhere(tl, t);
-			return (NULL);
-		} else if (wr_access) {
-			vcc_AddUses(tl, t, sym->w_methods, use);
-		} else if (sym->r_methods == 0) {
-			VSB_printf(tl->sb, "Variable ");
-			vcc_ErrToken(tl, t);
-			VSB_printf(tl->sb, " is write only.");
-			VSB_cat(tl->sb, "\nAt: ");
-			vcc_ErrWhere(tl, t);
-			return (NULL);
-		} else {
-			vcc_AddUses(tl, t, sym->r_methods, use);
-		}
-		return (sym);
-	}
-	VSB_printf(tl->sb, "Unknown variable ");
-	vcc_ErrToken(tl, t);
-	VSB_cat(tl->sb, "\nAt: ");
-	vcc_ErrWhere(tl, t);
-	return (NULL);
 }
