@@ -31,24 +31,17 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <unistd.h>
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
+#include "cache/cache_filter.h"
 
-#include "vcl.h"
-#include "vrt.h"
 #include "vsa.h"
-#include "vsb.h"
-#include "vtcp.h"
 #include "vtim.h"
 #include "vcc_if.h"
-
-#define WARN_RETIRED()							\
-	do {								\
-		VSL(SLT_Error, 0,					\
-		    "debug.%s is deprecated, use vmod-vtc instead.",	\
-		    __func__);						\
-	} while (0)
-
+#include "VSC_debug.h"
 
 struct priv_vcl {
 	unsigned		magic;
@@ -61,39 +54,64 @@ struct priv_vcl {
 
 static VCL_DURATION vcl_release_delay = 0.0;
 
-VCL_VOID __match_proto__(td_debug_panic)
-vmod_panic(VRT_CTX, const char *str, ...)
-{
-	va_list ap;
-	const char *b;
+static pthread_mutex_t vsc_mtx = PTHREAD_MUTEX_INITIALIZER;
+static struct vsc_seg *vsc_seg = NULL;
+static struct VSC_debug *vsc = NULL;
 
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-	va_start(ap, str);
-	b = VRT_String(ctx->ws, "PANIC: ", str, ap);
-	va_end(ap);
-	VAS_Fail("VCL", "", 0, b, VAS_VCL);
+/**********************************************************************/
+
+static enum vfp_status v_matchproto_(vfp_pull_f)
+xyzzy_rot13_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
+    ssize_t *lp)
+{
+	enum vfp_status vp;
+	char *q;
+	ssize_t l;
+
+	(void)vfe;
+	vp = VFP_Suck(vc, p, lp);
+	if (vp == VFP_ERROR)
+		return (vp);
+	q = p;
+	for (l = 0; l < *lp; l++, q++) {
+		if (*q >= 'A' && *q <= 'Z')
+			*q = (((*q - 'A') + 13) % 26) + 'A';
+		if (*q >= 'a' && *q <= 'z')
+			*q = (((*q - 'a') + 13) % 26) + 'a';
+	}
+	return (vp);
 }
 
-VCL_STRING __match_proto__(td_debug_author)
-vmod_author(VRT_CTX, VCL_ENUM person, VCL_ENUM someone)
+static const struct vfp xyzzy_rot13 = {
+	.name = "rot13",
+	.pull = xyzzy_rot13_pull,
+};
+
+/**********************************************************************/
+
+VCL_STRING v_matchproto_(td_debug_author)
+xyzzy_author(VRT_CTX, VCL_ENUM person, VCL_ENUM someone)
 {
 	(void)someone;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	if (!strcmp(person, "phk"))
+	if (person == xyzzy_enum_phk)
 		return ("Poul-Henning");
-	if (!strcmp(person, "des"))
+	assert(strcmp(person, "phk"));
+	if (person == xyzzy_enum_des)
 		return ("Dag-Erling");
-	if (!strcmp(person, "kristian"))
+	assert(strcmp(person, "des"));
+	if (person == xyzzy_enum_kristian)
 		return ("Kristian");
-	if (!strcmp(person, "mithrandir"))
+	assert(strcmp(person, "kristian"));
+	if (person == xyzzy_enum_mithrandir)
 		return ("Tollef");
+	assert(strcmp(person, "mithrandir"));
 	WRONG("Illegal VMOD enum");
 }
 
-VCL_VOID __match_proto__(td_debug_test_priv_call)
-vmod_test_priv_call(VRT_CTX, struct vmod_priv *priv)
+VCL_VOID v_matchproto_(td_debug_test_priv_call)
+xyzzy_test_priv_call(VRT_CTX, struct vmod_priv *priv)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -105,16 +123,27 @@ vmod_test_priv_call(VRT_CTX, struct vmod_priv *priv)
 	}
 }
 
-VCL_STRING __match_proto__(td_debug_test_priv_task)
-vmod_test_priv_task(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
+static void
+priv_task_free(void *ptr)
+{
+	AN(ptr);
+	VSL(SLT_Debug, 0, "priv_task_free(%p)", ptr);
+	free(ptr);
+}
+
+VCL_STRING v_matchproto_(td_debug_test_priv_task)
+xyzzy_test_priv_task(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (s == NULL || *s == '\0') {
-		return priv->priv;
+		VSL(SLT_Debug, 0, "test_priv_task(%p) = %p (exists)",
+		    priv, priv->priv);
 	} else if (priv->priv == NULL) {
 		priv->priv = strdup(s);
-		priv->free = free;
+		priv->free = priv_task_free;
+		VSL(SLT_Debug, 0, "test_priv_task(%p) = %p (new)",
+		    priv, priv->priv);
 	} else {
 		char *n = realloc(priv->priv,
 		    strlen(priv->priv) + strlen(s) + 2);
@@ -123,12 +152,16 @@ vmod_test_priv_task(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 		strcat(n, " ");
 		strcat(n, s);
 		priv->priv = n;
+		VSL(SLT_Debug, 0, "test_priv_task(%p) = %p (update)",
+		    priv, priv->priv);
 	}
+	if (priv->priv != NULL)
+		assert(priv->free == priv_task_free);
 	return (priv->priv);
 }
 
-VCL_STRING __match_proto__(td_debug_test_priv_top)
-vmod_test_priv_top(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
+VCL_STRING v_matchproto_(td_debug_test_priv_top)
+xyzzy_test_priv_top(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -139,8 +172,8 @@ vmod_test_priv_top(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 	return (priv->priv);
 }
 
-VCL_VOID __match_proto__(td_debug_test_priv_vcl)
-vmod_test_priv_vcl(VRT_CTX, struct vmod_priv *priv)
+VCL_VOID v_matchproto_(td_debug_test_priv_vcl)
+xyzzy_test_priv_vcl(VRT_CTX, struct vmod_priv *priv)
 {
 	struct priv_vcl *priv_vcl;
 
@@ -151,26 +184,8 @@ vmod_test_priv_vcl(VRT_CTX, struct vmod_priv *priv)
 	assert(!strcmp(priv_vcl->foo, "FOO"));
 }
 
-VCL_BACKEND
-vmod_no_backend(VRT_CTX)
-{
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-	return (NULL);
-}
-
-VCL_STEVEDORE __match_proto__(td_debug_no_stevedore)
-vmod_no_stevedore(VRT_CTX)
-{
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-	return (NULL);
-}
-
-VCL_VOID __match_proto__(td_debug_rot52)
-vmod_rot52(VRT_CTX, VCL_HTTP hp)
+VCL_VOID v_matchproto_(td_debug_rot52)
+xyzzy_rot52(VRT_CTX, VCL_HTTP hp)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -179,24 +194,26 @@ vmod_rot52(VRT_CTX, VCL_HTTP hp)
 	http_PrintfHeader(hp, "Encrypted: ROT52");
 }
 
-VCL_STRING __match_proto__(td_debug_argtest)
-vmod_argtest(VRT_CTX, VCL_STRING one, VCL_REAL two, VCL_STRING three,
-    VCL_STRING comma, VCL_INT four)
+VCL_STRING v_matchproto_(td_debug_argtest)
+xyzzy_argtest(VRT_CTX, struct xyzzy_argtest_arg *arg)
 {
 	char buf[100];
 
-	bprintf(buf, "%s %g %s %s %ld", one, two, three, comma, four);
+	AN(arg);
+	bprintf(buf, "%s %g %s %s %jd %d %s",
+	    arg->one, arg->two, arg->three, arg->comma, (intmax_t)arg->four,
+	    arg->valid_opt, arg->valid_opt ? arg->opt : "<undef>");
 	return (WS_Copy(ctx->ws, buf, -1));
 }
 
-VCL_INT __match_proto__(td_debug_vre_limit)
-vmod_vre_limit(VRT_CTX)
+VCL_INT v_matchproto_(td_debug_vre_limit)
+xyzzy_vre_limit(VRT_CTX)
 {
 	(void)ctx;
 	return (cache_param->vre_limits.match);
 }
 
-static void __match_proto__(obj_event_f)
+static void v_matchproto_(obj_event_f)
 obj_cb(struct worker *wrk, void *priv, struct objcore *oc, unsigned event)
 {
 	const struct priv_vcl *priv_vcl;
@@ -216,8 +233,8 @@ obj_cb(struct worker *wrk, void *priv, struct objcore *oc, unsigned event)
 	    (intmax_t)(uintptr_t)oc);
 }
 
-VCL_VOID __match_proto__(td_debug_register_obj_events)
-vmod_register_obj_events(VRT_CTX, struct vmod_priv *priv)
+VCL_VOID v_matchproto_(td_debug_register_obj_events)
+xyzzy_register_obj_events(VRT_CTX, struct vmod_priv *priv)
 {
 	struct priv_vcl *priv_vcl;
 
@@ -229,14 +246,14 @@ vmod_register_obj_events(VRT_CTX, struct vmod_priv *priv)
 	VSL(SLT_Debug, 0, "Subscribed to Object Events");
 }
 
-VCL_VOID __match_proto__(td_debug_fail)
-vmod_fail(VRT_CTX)
+VCL_VOID v_matchproto_(td_debug_fail)
+xyzzy_fail(VRT_CTX)
 {
 
 	VRT_fail(ctx, "Forced failure");
 }
 
-static void __match_proto__(vmod_priv_free_f)
+static void v_matchproto_(vmod_priv_free_f)
 priv_vcl_free(void *priv)
 {
 	struct priv_vcl *priv_vcl;
@@ -271,6 +288,12 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	AN(priv_vcl->foo);
 	priv->priv = priv_vcl;
 	priv->free = priv_vcl_free;
+
+	/*
+	 * NB: This is a proof of concept, until we decide what the real
+	 * API should look like, do NOT do this anywhere else.
+	 */
+	VRT_AddVFP(ctx, &xyzzy_rot13);
 	return (0);
 }
 
@@ -340,7 +363,7 @@ event_cold(VRT_CTX, const struct vmod_priv *priv)
 	return (0);
 }
 
-int __match_proto__(vmod_event_f)
+int v_matchproto_(vmod_event_f)
 event_function(VRT_CTX, struct vmod_priv *priv, enum vcl_event_e e)
 {
 
@@ -348,131 +371,17 @@ event_function(VRT_CTX, struct vmod_priv *priv, enum vcl_event_e e)
 	case VCL_EVENT_LOAD: return (event_load(ctx, priv));
 	case VCL_EVENT_WARM: return (event_warm(ctx, priv));
 	case VCL_EVENT_COLD: return (event_cold(ctx, priv));
+	case VCL_EVENT_DISCARD:
+		VRT_RemoveVFP(ctx, &xyzzy_rot13);
+		if (vsc)
+			VSC_debug_Destroy(&vsc_seg);
+		return (0);
 	default: return (0);
 	}
 }
 
-VCL_VOID __match_proto__(td_debug_sleep)
-vmod_sleep(VRT_CTX, VCL_DURATION t)
-{
-
-	CHECK_OBJ_ORNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-	VTIM_sleep(t);
-}
-
-static struct ws *
-wsfind(VRT_CTX, VCL_ENUM which)
-{
-	if (!strcmp(which, "client"))
-		return (ctx->ws);
-	else if (!strcmp(which, "backend"))
-		return (ctx->bo->ws);
-	else if (!strcmp(which, "session"))
-		return (ctx->req->sp->ws);
-	else if (!strcmp(which, "thread"))
-		return (ctx->req->wrk->aws);
-	else
-		WRONG("No such workspace.");
-}
-
-void
-vmod_workspace_allocate(VRT_CTX, VCL_ENUM which, VCL_INT size)
-{
-	struct ws *ws;
-	char *s;
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-
-	WS_Assert(ws);
-	AZ(ws->r);
-
-	if (size < 0) {
-		size += WS_Reserve(ws, 0);
-		WS_Release(ws, 0);
-	}
-	s = WS_Alloc(ws, size);
-	if (!s)
-		return;
-	memset(s, '\0', size);
-}
-
-VCL_INT
-vmod_workspace_free(VRT_CTX, VCL_ENUM which)
-{
-	struct ws *ws;
-	unsigned u;
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-
-	WS_Assert(ws);
-	u = WS_Reserve(ws, 0);
-	WS_Release(ws, 0);
-
-	return (u);
-}
-
-VCL_BOOL
-vmod_workspace_overflowed(VRT_CTX, VCL_ENUM which)
-{
-	struct ws *ws;
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-	WS_Assert(ws);
-
-	return (WS_Overflowed(ws));
-}
-
-static uintptr_t debug_ws_snap;
-
-void
-vmod_workspace_snap(VRT_CTX, VCL_ENUM which)
-{
-	struct ws *ws;
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-	WS_Assert(ws);
-
-	debug_ws_snap = WS_Snapshot(ws);
-}
-
-void
-vmod_workspace_reset(VRT_CTX, VCL_ENUM which)
-{
-	struct ws *ws;
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-	WS_Assert(ws);
-
-	WS_Reset(ws, debug_ws_snap);
-}
-
-void
-vmod_workspace_overflow(VRT_CTX, VCL_ENUM which)
-{
-	struct ws *ws;
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-
-	ws = wsfind(ctx, which);
-	WS_Assert(ws);
-
-	WS_MarkOverflow(ws);
-}
-
-VCL_VOID __match_proto__(td_debug_vcl_release_delay)
-vmod_vcl_release_delay(VRT_CTX, VCL_DURATION delay)
+VCL_VOID v_matchproto_(td_debug_vcl_release_delay)
+xyzzy_vcl_release_delay(VRT_CTX, VCL_DURATION delay)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -480,8 +389,8 @@ vmod_vcl_release_delay(VRT_CTX, VCL_DURATION delay)
 	vcl_release_delay = delay;
 }
 
-VCL_BOOL __match_proto__(td_debug_match_acl)
-vmod_match_acl(VRT_CTX, VCL_ACL acl, VCL_IP ip)
+VCL_BOOL v_matchproto_(td_debug_match_acl)
+xyzzy_match_acl(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 {
 
 	CHECK_OBJ_ORNULL(ctx, VRT_CTX_MAGIC);
@@ -491,41 +400,8 @@ vmod_match_acl(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 	return (VRT_acl_match(ctx, acl, ip));
 }
 
-VCL_BOOL
-vmod_barrier_sync(VRT_CTX, VCL_STRING addr)
-{
-	const char *err;
-	char buf[32];
-	int sock, i;
-	ssize_t sz;
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WARN_RETIRED();
-	AN(addr);
-	AN(*addr);
-
-	VSLb(ctx->vsl, SLT_Debug, "barrier_sync(\"%s\")", addr);
-	sock = VTCP_open(addr, NULL, 0., &err);
-	if (sock < 0) {
-		VSLb(ctx->vsl, SLT_Error, "Barrier connection failed: %s", err);
-		return (0);
-	}
-
-	sz = read(sock, buf, sizeof buf);
-	i = errno;
-	closefd(&sock);
-	if (sz == 0)
-		return (1);
-	if (sz < 0)
-		VSLb(ctx->vsl, SLT_Error,
-		    "Barrier read failed: %s (errno=%d)", strerror(i), i);
-	if (sz > 0)
-		VSLb(ctx->vsl, SLT_Error, "Barrier unexpected data (%zdB)", sz);
-	return (0);
-}
-
-VCL_VOID __match_proto__(td_debug_test_probe)
-vmod_test_probe(VRT_CTX, VCL_PROBE probe, VCL_PROBE same)
+VCL_VOID v_matchproto_(td_debug_test_probe)
+xyzzy_test_probe(VRT_CTX, VCL_PROBE probe, VCL_PROBE same)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -534,27 +410,145 @@ vmod_test_probe(VRT_CTX, VCL_PROBE probe, VCL_PROBE same)
 	AZ(same == NULL || probe == same);
 }
 
-VCL_INT
-vmod_typesize(VRT_CTX, VCL_STRING s)
+VCL_VOID
+xyzzy_vsc_new(VRT_CTX)
 {
-	size_t i = 0;
-	const char *p;
-
-	WARN_RETIRED();
 	(void)ctx;
-	for (p = s; *p; p++) {
-		switch (*p) {
-		case 'p':	i += sizeof(void *); break;
-		case 'i':	i += sizeof(int); break;
-		case 'd':	i += sizeof(double); break;
-		case 'f':	i += sizeof(float); break;
-		case 'l':	i += sizeof(long); break;
-		case 's':	i += sizeof(short); break;
-		case 'z':	i += sizeof(size_t); break;
-		case 'o':	i += sizeof(off_t); break;
-		case 'j':	i += sizeof(intmax_t); break;
-		default:	return(-1);
+	AZ(pthread_mutex_lock(&vsc_mtx));
+	if (vsc == NULL) {
+		AZ(vsc_seg);
+		vsc = VSC_debug_New(NULL, &vsc_seg, "");
+	}
+	AN(vsc);
+	AN(vsc_seg);
+	AZ(pthread_mutex_unlock(&vsc_mtx));
+}
+
+VCL_VOID
+xyzzy_vsc_count(VRT_CTX, VCL_INT cnt)
+{
+	(void)ctx;
+	AN(vsc);
+	vsc->count += cnt;
+}
+
+VCL_VOID
+xyzzy_vsc_destroy(VRT_CTX)
+{
+	(void)ctx;
+	AZ(pthread_mutex_lock(&vsc_mtx));
+	if (vsc != NULL) {
+		AN(vsc_seg);
+		VSC_debug_Destroy(&vsc_seg);
+	}
+	AZ(vsc_seg);
+	vsc = NULL;
+	AZ(pthread_mutex_unlock(&vsc_mtx));
+}
+
+struct xyzzy_debug_concat {
+	unsigned	magic;
+#define CONCAT_MAGIC 0x6b746493
+	VCL_STRING	s;
+};
+
+VCL_VOID
+xyzzy_concat__init(VRT_CTX, struct xyzzy_debug_concat **concatp,
+		   const char *vcl_name, VCL_STRANDS s)
+{
+	struct xyzzy_debug_concat *concat;
+	size_t sz = 0;
+	char *p;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(concatp);
+	AZ(*concatp);
+	AN(vcl_name);
+
+	ALLOC_OBJ(concat, CONCAT_MAGIC);
+	AN(concat);
+	*concatp = concat;
+
+	for (int i = 0; i < s->n; i++)
+		if (s->p[i] != NULL)
+			sz += strlen(s->p[i]);
+	p = malloc(sz + 1);
+	AN(p);
+	(void)VRT_Strands(p, sz + 1, s);
+	concat->s = p;
+}
+
+VCL_VOID
+xyzzy_concat__fini(struct xyzzy_debug_concat **concatp)
+{
+	struct xyzzy_debug_concat *concat;
+	void *p;
+
+	if (concatp == NULL || *concatp == NULL)
+		return;
+	CHECK_OBJ(*concatp, CONCAT_MAGIC);
+	concat = *concatp;
+	*concatp = NULL;
+	p = TRUST_ME(concat->s);
+	free(p);
+	FREE_OBJ(concat);
+}
+
+VCL_STRING
+xyzzy_concat_get(VRT_CTX, struct xyzzy_debug_concat *concat)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(concat, CONCAT_MAGIC);
+	return (concat->s);
+}
+
+VCL_STRING
+xyzzy_concatenate(VRT_CTX, VCL_STRANDS s)
+{
+	VCL_STRING r;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	r = VRT_StrandsWS(ctx->ws, NULL, s);
+	if (r != NULL && *r != '\0')
+		WS_Assert_Allocated(ctx->ws, r, strlen(r) + 1);
+	return (r);
+}
+
+VCL_STRING
+xyzzy_collect(VRT_CTX, VCL_STRANDS s)
+{
+	VCL_STRING r;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	r = VRT_CollectStrands(ctx, s);
+	if (r != NULL && *r != '\0')
+		WS_Assert_Allocated(ctx->ws, r, strlen(r) + 1);
+	return (r);
+}
+
+/* cf. VRT_SetHdr() */
+VCL_VOID
+xyzzy_sethdr(VRT_CTX, VCL_HEADER hs, VCL_STRANDS s)
+{
+	struct http *hp;
+	const char *b;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(hs);
+	AN(hs->what);
+	hp = VRT_selecthttp(ctx, hs->where);
+	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
+	if (s->n == 0) {
+		http_Unset(hp, hs->what);
+	} else {
+		b = VRT_StrandsWS(hp->ws, hs->what + 1, s);
+		if (b == NULL) {
+			VSLb(ctx->vsl, SLT_LostHeader, "%s", hs->what + 1);
+		} else {
+			if (*b != '\0')
+				WS_Assert_Allocated(hp->ws, b, strlen(b) + 1);
+			http_Unset(hp, hs->what);
+			http_SetHeader(hp, b);
 		}
 	}
-	return ((VCL_INT)i);
 }
