@@ -29,13 +29,13 @@
 #include "config.h"
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "cache/cache.h"
 
-#include "vrt.h"
 #include "vcc_if.h"
 
-struct vmod_debug_obj {
+struct xyzzy_debug_obj {
 	unsigned		magic;
 #define VMOD_DEBUG_OBJ_MAGIC	0xccbd9b77
 	int foobar;
@@ -43,10 +43,10 @@ struct vmod_debug_obj {
 };
 
 VCL_VOID
-vmod_obj__init(VRT_CTX, struct vmod_debug_obj **op,
+xyzzy_obj__init(VRT_CTX, struct xyzzy_debug_obj **op,
     const char *vcl_name, VCL_STRING s, VCL_ENUM e)
 {
-	struct vmod_debug_obj *o;
+	struct xyzzy_debug_obj *o;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	(void)vcl_name;
@@ -62,7 +62,7 @@ vmod_obj__init(VRT_CTX, struct vmod_debug_obj **op,
 }
 
 VCL_VOID
-vmod_obj__fini(struct vmod_debug_obj **op)
+xyzzy_obj__fini(struct xyzzy_debug_obj **op)
 {
 
 	AN(op);
@@ -71,8 +71,8 @@ vmod_obj__fini(struct vmod_debug_obj **op)
 	*op = NULL;
 }
 
-VCL_VOID __match_proto__()
-vmod_obj_enum(VRT_CTX, struct vmod_debug_obj *o, VCL_ENUM e)
+VCL_VOID v_matchproto_()
+xyzzy_obj_enum(VRT_CTX, struct xyzzy_debug_obj *o, VCL_ENUM e)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -80,8 +80,8 @@ vmod_obj_enum(VRT_CTX, struct vmod_debug_obj *o, VCL_ENUM e)
 	assert(!strcmp(e, "martin"));
 }
 
-VCL_STRING __match_proto__()
-vmod_obj_foo(VRT_CTX, struct vmod_debug_obj *o, VCL_STRING s)
+VCL_STRING v_matchproto_()
+xyzzy_obj_foo(VRT_CTX, struct xyzzy_debug_obj *o, VCL_STRING s)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -91,8 +91,8 @@ vmod_obj_foo(VRT_CTX, struct vmod_debug_obj *o, VCL_STRING s)
 	return ("BOO");
 }
 
-VCL_TIME __match_proto__()
-vmod_obj_date(VRT_CTX, struct vmod_debug_obj *o)
+VCL_TIME v_matchproto_()
+xyzzy_obj_date(VRT_CTX, struct xyzzy_debug_obj *o)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -101,8 +101,8 @@ vmod_obj_date(VRT_CTX, struct vmod_debug_obj *o)
 	return (21.4);
 }
 
-VCL_STRING __match_proto__()
-vmod_obj_string(VRT_CTX, struct vmod_debug_obj *o)
+VCL_STRING v_matchproto_()
+xyzzy_obj_string(VRT_CTX, struct xyzzy_debug_obj *o)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -111,8 +111,8 @@ vmod_obj_string(VRT_CTX, struct vmod_debug_obj *o)
 	return (o->string);
 }
 
-VCL_STRING __match_proto__()
-vmod_obj_number(VRT_CTX, struct vmod_debug_obj *o)
+VCL_STRING v_matchproto_()
+xyzzy_obj_number(VRT_CTX, struct xyzzy_debug_obj *o)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -121,31 +121,34 @@ vmod_obj_number(VRT_CTX, struct vmod_debug_obj *o)
 	return (o->number);
 }
 
-VCL_VOID __match_proto__()
-vmod_obj_test_priv_call(VRT_CTX,
-    struct vmod_debug_obj *o, struct vmod_priv *priv)
+VCL_VOID v_matchproto_()
+xyzzy_obj_test_priv_call(VRT_CTX,
+    struct xyzzy_debug_obj *o, struct vmod_priv *priv)
 {
 	(void)o;
-	vmod_test_priv_call(ctx, priv);
+	xyzzy_test_priv_call(ctx, priv);
 }
-VCL_VOID __match_proto__()
-vmod_obj_test_priv_vcl(VRT_CTX,
-    struct vmod_debug_obj *o, struct vmod_priv *priv)
+
+VCL_VOID v_matchproto_()
+xyzzy_obj_test_priv_vcl(VRT_CTX,
+    struct xyzzy_debug_obj *o, struct vmod_priv *priv)
 {
 	(void)o;
-	vmod_test_priv_vcl(ctx, priv);
+	xyzzy_test_priv_vcl(ctx, priv);
 }
-VCL_STRING __match_proto__()
-vmod_obj_test_priv_task(VRT_CTX,
-    struct vmod_debug_obj *o, struct vmod_priv *priv, VCL_STRING s)
+
+VCL_STRING v_matchproto_()
+xyzzy_obj_test_priv_task(VRT_CTX,
+    struct xyzzy_debug_obj *o, struct vmod_priv *priv, VCL_STRING s)
 {
 	(void)o;
-	return (vmod_test_priv_task(ctx, priv, s));
+	return (xyzzy_test_priv_task(ctx, priv, s));
 }
-VCL_STRING __match_proto__()
-vmod_obj_test_priv_top(VRT_CTX,
-    struct vmod_debug_obj *o, struct vmod_priv *priv, VCL_STRING s)
+
+VCL_STRING v_matchproto_()
+xyzzy_obj_test_priv_top(VRT_CTX,
+    struct xyzzy_debug_obj *o, struct vmod_priv *priv, VCL_STRING s)
 {
 	(void)o;
-	return (vmod_test_priv_top(ctx, priv, s));
+	return (xyzzy_test_priv_top(ctx, priv, s));
 }
