@@ -29,11 +29,10 @@
 #include "config.h"
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "cache/cache.h"
-#include "cache/cache_director.h"
 
-#include "vrt.h"
 #include "vcc_if.h"
 
 #include "vdir.h"
@@ -46,27 +45,26 @@ struct vmod_directors_fallback {
 	unsigned				cur;
 };
 
-static unsigned __match_proto__(vdi_healthy)
-vmod_fallback_healthy(const struct director *dir, const struct busyobj *bo,
-    double *changed)
+static VCL_BOOL v_matchproto_(vdi_healthy)
+vmod_fallback_healthy(VRT_CTX, VCL_BACKEND dir, VCL_TIME *changed)
 {
 	struct vmod_directors_fallback *fb;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(fb, dir->priv, VMOD_DIRECTORS_FALLBACK_MAGIC);
-	return (vdir_any_healthy(fb->vd, bo, changed));
+	return (vdir_any_healthy(ctx, fb->vd, changed));
 }
 
-static const struct director * __match_proto__(vdi_resolve_f)
-vmod_fallback_resolve(const struct director *dir, struct worker *wrk,
-    struct busyobj *bo)
+static VCL_BACKEND v_matchproto_(vdi_resolve_f)
+vmod_fallback_resolve(VRT_CTX, VCL_BACKEND dir)
 {
 	struct vmod_directors_fallback *fb;
 	unsigned u;
 	VCL_BACKEND be = NULL;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
-	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
-	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	CAST_OBJ_NOTNULL(fb, dir->priv, VMOD_DIRECTORS_FALLBACK_MAGIC);
 
 	vdir_wrlock(fb->vd);
@@ -75,7 +73,7 @@ vmod_fallback_resolve(const struct director *dir, struct worker *wrk,
 	for (u = 0; u < fb->vd->n_backend; u++) {
 		be = fb->vd->backend[fb->cur];
 		CHECK_OBJ_NOTNULL(be, DIRECTOR_MAGIC);
-		if (be->healthy(be, bo, NULL))
+		if (VRT_Healthy(ctx, be, NULL))
 			break;
 		if (++fb->cur == fb->vd->n_backend)
 			fb->cur = 0;
@@ -86,7 +84,26 @@ vmod_fallback_resolve(const struct director *dir, struct worker *wrk,
 	return (be);
 }
 
-VCL_VOID __match_proto__()
+static void v_matchproto_(vdi_destroy_f)
+vmod_fallback_destroy(VCL_BACKEND dir)
+{
+	struct vmod_directors_fallback *fallback;
+
+	CAST_OBJ_NOTNULL(fallback, dir->priv, VMOD_DIRECTORS_FALLBACK_MAGIC);
+	vdir_delete(&fallback->vd);
+	FREE_OBJ(fallback);
+}
+
+static const struct vdi_methods vmod_fallback_methods[1] = {{
+	.magic =		VDI_METHODS_MAGIC,
+	.type =			"fallback",
+	.healthy =		vmod_fallback_healthy,
+	.resolve =		vmod_fallback_resolve,
+	.destroy =		vmod_fallback_destroy
+}};
+
+
+VCL_VOID v_matchproto_()
 vmod_fallback__init(VRT_CTX,
     struct vmod_directors_fallback **fbp, const char *vcl_name, VCL_BOOL sticky)
 {
@@ -98,43 +115,43 @@ vmod_fallback__init(VRT_CTX,
 	ALLOC_OBJ(fb, VMOD_DIRECTORS_FALLBACK_MAGIC);
 	AN(fb);
 	*fbp = fb;
-	vdir_new(&fb->vd, "fallback", vcl_name, vmod_fallback_healthy,
-	    vmod_fallback_resolve, fb);
+	vdir_new(ctx, &fb->vd, vcl_name, vmod_fallback_methods, fb);
 	fb->st = sticky;
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_fallback__fini(struct vmod_directors_fallback **fbp)
 {
 	struct vmod_directors_fallback *fb;
 
-	fb = *fbp;
-	*fbp = NULL;
-	CHECK_OBJ_NOTNULL(fb, VMOD_DIRECTORS_FALLBACK_MAGIC);
-	vdir_delete(&fb->vd);
-	FREE_OBJ(fb);
+	// XXX 2297
+	if (*fbp == NULL)
+		return;
+
+	TAKE_OBJ_NOTNULL(fb, fbp, VMOD_DIRECTORS_FALLBACK_MAGIC);
+	VRT_DelDirector(&fb->vd->dir);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_fallback_add_backend(VRT_CTX,
     struct vmod_directors_fallback *fb, VCL_BACKEND be)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(fb, VMOD_DIRECTORS_FALLBACK_MAGIC);
-	(void)vdir_add_backend(fb->vd, be, 0.0);
+	vdir_add_backend(ctx, fb->vd, be, 0.0);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_fallback_remove_backend(VRT_CTX,
     struct vmod_directors_fallback *fb, VCL_BACKEND be)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(fb, VMOD_DIRECTORS_FALLBACK_MAGIC);
-	vdir_remove_backend(fb->vd, be, &fb->cur);
+	vdir_remove_backend(ctx, fb->vd, be, &fb->cur);
 }
 
-VCL_BACKEND __match_proto__()
+VCL_BACKEND v_matchproto_()
 vmod_fallback_backend(VRT_CTX,
     struct vmod_directors_fallback *fb)
 {
