@@ -31,11 +31,9 @@
 #include <stdlib.h>
 
 #include "cache/cache.h"
-#include "cache/cache_director.h"
 
 #include "vbm.h"
 #include "vrnd.h"
-#include "vrt.h"
 
 #include "vdir.h"
 
@@ -47,35 +45,53 @@ struct vmod_directors_random {
 	struct vdir				*vd;
 };
 
-static unsigned __match_proto__(vdi_healthy)
-vmod_random_healthy(const struct director *dir, const struct busyobj *bo,
-    double *changed)
+static VCL_BOOL v_matchproto_(vdi_healthy)
+vmod_random_healthy(VRT_CTX, VCL_BACKEND dir, VCL_TIME *changed)
 {
 	struct vmod_directors_random *rr;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(rr, dir->priv, VMOD_DIRECTORS_RANDOM_MAGIC);
-	return (vdir_any_healthy(rr->vd, bo, changed));
+	return (vdir_any_healthy(ctx, rr->vd, changed));
 }
 
-static const struct director * __match_proto__(vdi_resolve_f)
-vmod_random_resolve(const struct director *dir, struct worker *wrk,
-    struct busyobj *bo)
+static VCL_BACKEND v_matchproto_(vdi_resolve_f)
+vmod_random_resolve(VRT_CTX, VCL_BACKEND dir)
 {
 	struct vmod_directors_random *rr;
 	VCL_BACKEND be;
 	double r;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
-	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
-	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	CAST_OBJ_NOTNULL(rr, dir->priv, VMOD_DIRECTORS_RANDOM_MAGIC);
 	r = scalbn(VRND_RandomTestable(), -31);
 	assert(r >= 0 && r < 1.0);
-	be = vdir_pick_be(rr->vd, r, bo);
+	be = vdir_pick_be(ctx, rr->vd, r);
 	return (be);
 }
 
-VCL_VOID __match_proto__()
+static void v_matchproto_(vdi_destroy_f)
+vmod_random_destroy(VCL_BACKEND dir)
+{
+	struct vmod_directors_random *rr;
+
+	CAST_OBJ_NOTNULL(rr, dir->priv, VMOD_DIRECTORS_RANDOM_MAGIC);
+	vdir_delete(&rr->vd);
+	FREE_OBJ(rr);
+}
+
+static const struct vdi_methods vmod_random_methods[1] = {{
+	.magic =		VDI_METHODS_MAGIC,
+	.type =			"random",
+	.healthy =		vmod_random_healthy,
+	.resolve =		vmod_random_resolve,
+	.destroy =		vmod_random_destroy
+}};
+
+
+VCL_VOID v_matchproto_()
 vmod_random__init(VRT_CTX, struct vmod_directors_random **rrp,
     const char *vcl_name)
 {
@@ -87,41 +103,41 @@ vmod_random__init(VRT_CTX, struct vmod_directors_random **rrp,
 	ALLOC_OBJ(rr, VMOD_DIRECTORS_RANDOM_MAGIC);
 	AN(rr);
 	*rrp = rr;
-	vdir_new(&rr->vd, "random", vcl_name, vmod_random_healthy,
-	    vmod_random_resolve, rr);
+	vdir_new(ctx, &rr->vd, vcl_name, vmod_random_methods, rr);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_random__fini(struct vmod_directors_random **rrp)
 {
 	struct vmod_directors_random *rr;
 
-	rr = *rrp;
-	*rrp = NULL;
-	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_RANDOM_MAGIC);
-	vdir_delete(&rr->vd);
-	FREE_OBJ(rr);
+	// XXX 2297
+	if (*rrp == NULL)
+		return;
+
+	TAKE_OBJ_NOTNULL(rr, rrp, VMOD_DIRECTORS_RANDOM_MAGIC);
+	VRT_DelDirector(&rr->vd->dir);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_random_add_backend(VRT_CTX,
     struct vmod_directors_random *rr, VCL_BACKEND be, double w)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_RANDOM_MAGIC);
-	(void)vdir_add_backend(rr->vd, be, w);
+	vdir_add_backend(ctx, rr->vd, be, w);
 }
 
 VCL_VOID vmod_random_remove_backend(VRT_CTX,
-    struct vmod_directors_random *rr, VCL_BACKEND be) __match_proto__()
+    struct vmod_directors_random *rr, VCL_BACKEND be) v_matchproto_()
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_RANDOM_MAGIC);
-	vdir_remove_backend(rr->vd, be, NULL);
+	vdir_remove_backend(ctx, rr->vd, be, NULL);
 }
 
-VCL_BACKEND __match_proto__()
+VCL_BACKEND v_matchproto_()
 vmod_random_backend(VRT_CTX, struct vmod_directors_random *rr)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
