@@ -29,17 +29,15 @@
 
 #include "config.h"
 
+#include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "cache/cache.h"
-#include "cache/cache_director.h"
-
-#include "vrt.h"
 
 #include "shard_dir.h"
 #include "shard_cfg.h"
-#include "shard_hash.h"
 
 /*lint -esym(749,  shard_change_task_e::*) */
 enum shard_change_task_e {
@@ -68,9 +66,9 @@ struct shard_change {
 
 struct backend_reconfig {
 	struct sharddir * const shardd;
-	int			hint;	// on number of backends after reconfig
-	int			hole_n; // number of holes in backends array
-	int			hole_i; // index hint on first hole
+	unsigned		hint;	// on number of backends after reconfig
+	unsigned		hole_n; // number of holes in backends array
+	unsigned		hole_i; // index hint on first hole
 };
 
 /*
@@ -232,11 +230,12 @@ circlepoint_compare(const struct shard_circlepoint *a,
 }
 
 static void
-shardcfg_hashcircle(struct sharddir *shardd, VCL_INT replicas, enum alg_e alg)
+shardcfg_hashcircle(struct sharddir *shardd, VCL_INT replicas)
 {
 	int i, j;
 	const char *ident;
-	int len;
+	const int len = 12; // log10(UINT32_MAX) + 2;
+	char s[len];
 
 	CHECK_OBJ_NOTNULL(shardd, SHARDDIR_MAGIC);
 	AZ(shardd->hashcircle);
@@ -255,18 +254,15 @@ shardcfg_hashcircle(struct sharddir *shardd, VCL_INT replicas, enum alg_e alg)
 
 		ident = shardd->backend[i].ident
 		    ? shardd->backend[i].ident
-		    : shardd->backend[i].backend->vcl_name;
+		    : VRT_BACKEND_string(shardd->backend[i].backend);
 
+		AN(ident);
 		assert(ident[0] != '\0');
 
-		len = strlen(ident) + 12; // log10(UINT32_MAX) + 2;
-
-		char s[len];
-
 		for (j = 0; j < replicas; j++) {
-			assert(snprintf(s, len, "%s%d", ident, j) < len);
+			assert(snprintf(s, len, "%d", j) < len);
 			shardd->hashcircle[i * replicas + j].point =
-			    shard_hash_f[alg](s);
+				sharddir_sha256(ident, s, vrt_magic_string_end);
 			shardd->hashcircle[i * replicas + j].host = i;
 		}
 		/* not used in current interface */
@@ -282,9 +278,9 @@ shardcfg_hashcircle(struct sharddir *shardd, VCL_INT replicas, enum alg_e alg)
 	for (i = 0; i < shardd->n_backend; i++)
 		for (j = 0; j < replicas; j++)
 			SHDBG(SHDBG_CIRCLE, shardd,
-			    "hashcircle[%5ld] = "
+			    "hashcircle[%5jd] = "
 			    "{point = %8x, host = %2u}\n",
-			    i * replicas + j,
+			    (intmax_t)(i * replicas + j),
 			    shardd->hashcircle[i * replicas + j].point,
 			    shardd->hashcircle[i * replicas + j].host);
 }
@@ -297,8 +293,8 @@ shardcfg_hashcircle(struct sharddir *shardd, VCL_INT replicas, enum alg_e alg)
 static void
 shardcfg_backend_free(struct shard_backend *f)
 {
-	if (f->ident)
-		free (TRUST_ME(f->ident));
+	if (f->freeptr)
+		free (f->freeptr);
 	memset(f, 0, sizeof(*f));
 }
 
@@ -329,11 +325,13 @@ shardcfg_backend_cmp(const struct shard_backend *a,
 		return a->backend != b->backend;
 
 	if (ai == NULL)
-		ai = a->backend->vcl_name;
+		ai = VRT_BACKEND_string(a->backend);
 
 	if (bi == NULL)
-		bi = b->backend->vcl_name;
+		bi = VRT_BACKEND_string(b->backend);
 
+	AN(ai);
+	AN(bi);
 	return strcmp(ai, bi);
 }
 
@@ -354,7 +352,7 @@ static const struct shard_backend *
 shardcfg_backend_lookup(const struct backend_reconfig *re,
     const struct shard_backend *b)
 {
-	int i, max = re->shardd->n_backend + re->hole_n;
+	unsigned i, max = re->shardd->n_backend + re->hole_n;
 	const struct shard_backend *bb = re->shardd->backend;
 
 	for (i = 0; i < max; i++)
@@ -367,7 +365,7 @@ shardcfg_backend_lookup(const struct backend_reconfig *re,
 static void
 shardcfg_backend_expand(const struct backend_reconfig *re)
 {
-	int min = re->hint;
+	unsigned min = re->hint;
 
 	CHECK_OBJ_NOTNULL(re->shardd, SHARDDIR_MAGIC);
 
@@ -393,7 +391,7 @@ static void
 shardcfg_backend_add(struct backend_reconfig *re,
     const struct shard_backend *b)
 {
-	int i;
+	unsigned i;
 	struct shard_backend *bb = re->shardd->backend;
 
 	if (re->hole_n == 0) {
@@ -432,7 +430,7 @@ static void
 shardcfg_backend_del(struct backend_reconfig *re,
     const struct shard_backend *spec)
 {
-	int i, max = re->shardd->n_backend + re->hole_n;
+	unsigned i, max = re->shardd->n_backend + re->hole_n;
 	struct shard_backend * const bb = re->shardd->backend;
 
 	for (i = 0; i < max; i++) {
@@ -452,7 +450,7 @@ shardcfg_backend_del(struct backend_reconfig *re,
 static void
 shardcfg_backend_finalize(struct backend_reconfig *re)
 {
-	int i;
+	unsigned i;
 	struct shard_backend * const bb = re->shardd->backend;
 
 	while (re->hole_n > 0) {
@@ -553,7 +551,7 @@ shardcfg_apply_change(VRT_CTX, struct sharddir *shardd,
 
 			shard_err(ctx, shardd, "(notice) backend %s%s%s "
 			    "already exists - skipping",
-			    b->backend->vcl_name,
+			    VRT_BACKEND_string(b->backend),
 			    ident ? "/" : "",
 			    ident ? ident : "");
 			break;
@@ -574,7 +572,7 @@ shardcfg_apply_change(VRT_CTX, struct sharddir *shardd,
 
 VCL_BOOL
 shardcfg_reconfigure(VRT_CTX, struct vmod_priv *priv,
-    struct sharddir *shardd, VCL_INT replicas, enum alg_e alg)
+    struct sharddir *shardd, VCL_INT replicas)
 {
 	struct shard_change *change;
 
@@ -607,7 +605,7 @@ shardcfg_reconfigure(VRT_CTX, struct vmod_priv *priv,
 		return 0;
 	}
 
-	shardcfg_hashcircle(shardd, replicas, alg);
+	shardcfg_hashcircle(shardd, replicas);
 	sharddir_unlock(shardd);
 	return (1);
 }
@@ -660,8 +658,7 @@ shardcfg_get_rampup(const struct sharddir *shardd, int host)
 	// assert sharddir_rdlock_held(shardd);
 	assert (host < shardd->n_backend);
 
-	// magic value for default
-	if (shardd->backend[host].rampup == 973279260)
+	if (isnan(shardd->backend[host].rampup))
 		r = shardd->rampup_duration;
 	else
 		r = shardd->backend[host].rampup;
