@@ -27,7 +27,26 @@
  * SUCH DAMAGE.
  */
 
-#include "shard_parse_vcc_enums.h"
+enum by_e {
+	_BY_E_INVALID = 0,
+#define VMODENUM(x) BY_ ## x,
+#include "tbl_by.h"
+	_BY_E_MAX
+};
+
+enum healthy_e {
+	_HEALTHY_E_INVALID = 0,
+#define VMODENUM(x) x,
+#include "tbl_healthy.h"
+	_HEALTHY_E_MAX
+};
+
+enum resolve_e {
+	_RESOLVE_E_INVALID = 0,
+#define VMODENUM(x) x,
+#include "tbl_resolve.h"
+	_RESOLVE_E_MAX
+};
 
 struct vbitmap;
 
@@ -38,11 +57,16 @@ struct shard_circlepoint {
 
 struct shard_backend {
 	VCL_BACKEND		backend;
-	const char		*ident;	// XXX COPY IN !
+	union {
+		const char	*ident;
+		void		*freeptr;
+	};
 	VCL_DURATION		rampup;
 	uint32_t		canon_point;
 };
 
+struct vmod_directors_shard_param;
+
 #define	SHDBG_LOOKUP	 1
 #define	SHDBG_CIRCLE	(1<<1)
 #define	SHDBG_RAMPWARM	(1<<2)
@@ -54,13 +78,13 @@ struct sharddir {
 
 	pthread_rwlock_t			mtx;
 
-	const char				*name;
-
 	unsigned				n_backend;
 	unsigned				l_backend;
 	struct shard_backend			*backend;
 
+	const char				*name;
 	struct shard_circlepoint		*hashcircle;
+	const struct vmod_directors_shard_param	*param;
 
 	VCL_DURATION				rampup_duration;
 	VCL_REAL				warmup;
@@ -103,10 +127,16 @@ sharddir_backend_ident(const struct sharddir *shardd, int host)
 
 void sharddir_debug(struct sharddir *shardd, const uint32_t flags);
 void sharddir_err(VRT_CTX, enum VSL_tag_e tag,  const char *fmt, ...);
-void sharddir_new(struct sharddir **sharddp, const char *vcl_name);
+uint32_t sharddir_sha256v(const char *s, va_list ap);
+uint32_t sharddir_sha256(const char *s, ...);
+void sharddir_new(struct sharddir **sharddp, const char *vcl_name,
+    const struct vmod_directors_shard_param *param);
+void sharddir_set_param(struct sharddir *shardd,
+    const struct vmod_directors_shard_param *param);
 void sharddir_delete(struct sharddir **sharddp);
 void sharddir_wrlock(struct sharddir *shardd);
 void sharddir_unlock(struct sharddir *shardd);
+VCL_BOOL sharddir_any_healthy(VRT_CTX, struct sharddir *, VCL_TIME *);
 VCL_BACKEND sharddir_pick_be(VRT_CTX, struct sharddir *, uint32_t, VCL_INT,
    VCL_REAL, VCL_BOOL, enum healthy_e);
 
