@@ -31,9 +31,7 @@
 #include <stdlib.h>
 
 #include "cache/cache.h"
-#include "cache/cache_director.h"
 
-#include "vrt.h"
 #include "vbm.h"
 
 #include "vdir.h"
@@ -51,12 +49,13 @@ vdir_expand(struct vdir *vd, unsigned n)
 }
 
 void
-vdir_new(struct vdir **vdp, const char *name, const char *vcl_name,
-    vdi_healthy_f *healthy, vdi_resolve_f *resolve, void *priv)
+vdir_new(VRT_CTX, struct vdir **vdp, const char *vcl_name,
+    const struct vdi_methods *m, void *priv)
 {
 	struct vdir *vd;
 
-	AN(name);
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(m, VDI_METHODS_MAGIC);
 	AN(vcl_name);
 	AN(vdp);
 	AZ(*vdp);
@@ -64,14 +63,7 @@ vdir_new(struct vdir **vdp, const char *name, const char *vcl_name,
 	AN(vd);
 	*vdp = vd;
 	AZ(pthread_rwlock_init(&vd->mtx, NULL));
-
-	ALLOC_OBJ(vd->dir, DIRECTOR_MAGIC);
-	AN(vd->dir);
-	vd->dir->name = name;
-	REPLACE(vd->dir->vcl_name, vcl_name);
-	vd->dir->priv = priv;
-	vd->dir->healthy = healthy;
-	vd->dir->resolve = resolve;
+	vd->dir = VRT_AddDirector(ctx, m, priv, "%s", vcl_name);
 	vd->vbm = vbit_new(8);
 	AN(vd->vbm);
 }
@@ -83,11 +75,10 @@ vdir_delete(struct vdir **vdp)
 
 	TAKE_OBJ_NOTNULL(vd, vdp, VDIR_MAGIC);
 
+	AZ(vd->dir);
 	free(vd->backend);
 	free(vd->weight);
 	AZ(pthread_rwlock_destroy(&vd->mtx));
-	free(vd->dir->vcl_name);
-	FREE_OBJ(vd->dir);
 	vbit_destroy(vd->vbm);
 	FREE_OBJ(vd);
 }
@@ -114,12 +105,18 @@ vdir_unlock(struct vdir *vd)
 }
 
 
-unsigned
-vdir_add_backend(struct vdir *vd, VCL_BACKEND be, double weight)
+void
+vdir_add_backend(VRT_CTX, struct vdir *vd, VCL_BACKEND be, double weight)
 {
 	unsigned u;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vd, VDIR_MAGIC);
+	if (be == NULL) {
+		VRT_fail(ctx, "%s: NULL backend cannot be added",
+		    VRT_BACKEND_string(vd->dir));
+		return;
+	}
 	AN(be);
 	vdir_wrlock(vd);
 	if (vd->n_backend >= vd->l_backend)
@@ -130,17 +127,20 @@ vdir_add_backend(struct vdir *vd, VCL_BACKEND be, double weight)
 	vd->weight[u] = weight;
 	vd->total_weight += weight;
 	vdir_unlock(vd);
-	return (u);
 }
 
 void
-vdir_remove_backend(struct vdir *vd, VCL_BACKEND be, unsigned *cur)
+vdir_remove_backend(VRT_CTX, struct vdir *vd, VCL_BACKEND be, unsigned *cur)
 {
 	unsigned u, n;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vd, VDIR_MAGIC);
-	if (be == NULL)
+	if (be == NULL) {
+		VRT_fail(ctx, "%s: NULL backend cannot be removed",
+		    VRT_BACKEND_string(vd->dir));
 		return;
+	}
 	CHECK_OBJ(be, DIRECTOR_MAGIC);
 	vdir_wrlock(vd);
 	for (u = 0; u < vd->n_backend; u++)
@@ -166,23 +166,23 @@ vdir_remove_backend(struct vdir *vd, VCL_BACKEND be, unsigned *cur)
 	vdir_unlock(vd);
 }
 
-unsigned
-vdir_any_healthy(struct vdir *vd, const struct busyobj *bo, double *changed)
+VCL_BOOL
+vdir_any_healthy(VRT_CTX, struct vdir *vd, VCL_TIME *changed)
 {
 	unsigned retval = 0;
 	VCL_BACKEND be;
 	unsigned u;
 	double c;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vd, VDIR_MAGIC);
-	CHECK_OBJ_ORNULL(bo, BUSYOBJ_MAGIC);
 	vdir_rdlock(vd);
 	if (changed != NULL)
 		*changed = 0;
 	for (u = 0; u < vd->n_backend; u++) {
 		be = vd->backend[u];
 		CHECK_OBJ_NOTNULL(be, DIRECTOR_MAGIC);
-		retval = be->healthy(be, bo, &c);
+		retval = VRT_Healthy(ctx, be, &c);
 		if (changed != NULL && c > *changed)
 			*changed = c;
 		if (retval)
@@ -214,15 +214,17 @@ vdir_pick_by_weight(const struct vdir *vd, double w,
 }
 
 VCL_BACKEND
-vdir_pick_be(struct vdir *vd, double w, const struct busyobj *bo)
+vdir_pick_be(VRT_CTX, struct vdir *vd, double w)
 {
 	unsigned u;
 	double tw = 0.0;
 	VCL_BACKEND be = NULL;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(vd, VDIR_MAGIC);
 	vdir_wrlock(vd);
 	for (u = 0; u < vd->n_backend; u++) {
-		if (vd->backend[u]->healthy(vd->backend[u], bo, NULL)) {
+		if (VRT_Healthy(ctx, vd->backend[u], NULL)) {
 			vbit_clr(vd->vbm, u);
 			tw += vd->weight[u];
 		} else
