@@ -37,18 +37,17 @@ struct vdir {
 	VCL_BACKEND				*backend;
 	double					*weight;
 	double					total_weight;
-	struct director				*dir;
+	VCL_BACKEND				dir;
 	struct vbitmap				*vbm;
 };
 
-void vdir_new(struct vdir **vdp, const char *name, const char *vcl_name,
-    vdi_healthy_f *healthy, vdi_resolve_f *resolve, void *priv);
+void vdir_new(VRT_CTX, struct vdir **vdp, const char *vcl_name,
+    const struct vdi_methods *, void *priv);
 void vdir_delete(struct vdir **vdp);
 void vdir_rdlock(struct vdir *vd);
 void vdir_wrlock(struct vdir *vd);
 void vdir_unlock(struct vdir *vd);
-unsigned vdir_add_backend(struct vdir *, VCL_BACKEND be, double weight);
-void vdir_remove_backend(struct vdir *, VCL_BACKEND be, unsigned *cur);
-unsigned vdir_any_healthy(struct vdir *, const struct busyobj *,
-    double *changed);
-VCL_BACKEND vdir_pick_be(struct vdir *, double w, const struct busyobj *);
+void vdir_add_backend(VRT_CTX, struct vdir *, VCL_BACKEND, double weight);
+void vdir_remove_backend(VRT_CTX, struct vdir *, VCL_BACKEND, unsigned *cur);
+VCL_BOOL vdir_any_healthy(VRT_CTX, struct vdir *, VCL_TIME *);
+VCL_BACKEND vdir_pick_be(VRT_CTX, struct vdir *, double w);
